<?php

namespace Ems\Controllers;

use Ems\Services\HierarquiaService;
use Ems\Util\ExpectedException;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class HierarquiaController {
    /** @param HierarquiaService */
    private $hierarquiaService;

    public function __construct(
        HierarquiaService $hierarquiaService
    ) {
        $this->hierarquiaService = $hierarquiaService;
    }

    public function getTiposHierarquia(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $tiposHierarquia = $this->hierarquiaService->getTiposHierarquia();

            $response->addDataSet(new DataSet('getTiposHierarquia', $tiposHierarquia));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getHierarquiaOrganizacao(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrVinculo = (int) $row['NRVINCULOM'];
            $hierarquia = $this->hierarquiaService->getHierarquiaOrganizacao($nrVinculo);

            $response->addDataSet(new DataSet('getHierarquiaOrganizacao', $hierarquia));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}