<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\ReconhecimentoFacialService;
use Ems\Util\ArrayUtil;
use Ems\Util\Base64Util;
use Ems\Util\ExpectedException;

class ReconhecimentoFacialController {
    /** @var ReconhecimentoFacialService */
    protected $reconhecimentoFacialService;

    public function __construct(
        ReconhecimentoFacialService $reconhecimentoFacialService
    ) {
        $this->reconhecimentoFacialService = $reconhecimentoFacialService;
    }

    public function helloWorld(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $responseBody = $this->reconhecimentoFacialService->helloWorld();

            $response->addDataSet(new DataSet('helloWorld', $responseBody));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function encodaImagem(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $file = $row['FILE'];

            $responseBody = $this->reconhecimentoFacialService->encodaImagem($file);

            $response->addDataSet(new DataSet('encodaImagem', $responseBody));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function verificaRostoEncodaECompara(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $file1 = $row['FILE1'];
            $file2 = $row['FILE2'];

            $responseBody = $this->reconhecimentoFacialService->verificaRostoEncodaECompara($file1, $file2);

            $response->addDataSet(new DataSet('verificaRostoEncodaECompara', [$responseBody]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveHashFacial(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $nrparcnegocio = $row['NRPARCNEGOCIO'];
            $file = $row['FILE'];

            $success = $this->reconhecimentoFacialService->saveHashFacial($nrparcnegocio, $file);

            $response->addDataSet(new DataSet('saveHashFacial', array($success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function updateHashFacial(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $nrfaceid = $row['NRFACEID'] ?? null;
            $nrparcnegocio = $row['NRPARCNEGOCIO'] ?? null;
            $file = $row['FILE'] ?? null;

            $success = $this->reconhecimentoFacialService->updateHashFacial($nrfaceid, $nrparcnegocio, $file);

            $response->addDataSet(new DataSet('updateHashFacial', array($success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteHashFacial(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $nrfaceid = $row['NRFACEID'] ?? null;
            $nrparcnegocio = $row['NRPARCNEGOCIO'] ?? null;

            $success = $this->reconhecimentoFacialService->deleteHashFacial($nrfaceid, $nrparcnegocio);

            $response->addDataSet(new DataSet('deleteHashFacial', array($success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function reconhecimentoFacial(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $row = $request->getRow()->getArrayCopy();

            $file = $row['FILE'];
            $hashConhecidos = $row['HASHS_CONHECIDOS'];

            $responseBody = $this->reconhecimentoFacialService->reconhecimentoFacial($file, $hashConhecidos);

            $response->addDataSet(new DataSet('reconhecimentoFacial', [$responseBody]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getBiometriasVinculo(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrParcnegocio   = (int) $row['NRPARCNEGOCIO'];

            $biometrias = $this->reconhecimentoFacialService->getBiometriasVinculo($nrParcnegocio);

            $response->addDataSet(new DataSet('getBiometriasVinculo', $biometrias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

}
