<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

use Ems\Services\SolicitacaoService;
use Ems\Services\BeneficioService;
use Ems\Services\MovimentacaoService;
use Ems\Services\MinhaHistoriaService;
use Ems\Util\ExpectedException;
use Ems\Util\Utils;

class SolicitacaoController {

    /** @var OcorrenciaService */
    protected $solicitacaoService;

    protected $fieldsToDsValor = ["NRTIPOALTERA", "DTRETORNOFERIAS", "CDPAIS", "NRCURSO", "DTINIPLANEJADOTRE", "DSPESSOA", "CDFORMACOMU", "IDTPBENEFICIARIO", 
    "DTAPROVAHE", "NRESTRUTGEREN", "QTLINHATARIFA", "DSOBSBENEFICIARIO", "QTDIASFERIAS", "VRREEMBOLSO", "CDTIPOVC", "VRTARIFA_8", "QTLINHATARIFA_5", "NRTIPOLINHAVINC_6", 
    "NMPESSOAINDICADO", "QTDEDIASFERIAS", "CDBANCO", "NRMOTIVORESC", "NRBENEFICIOM", "NROCUPACAOM", "NRMOTIVOOCUPA", "NRTIPOTRANSFER", "DTINIMOVIMENT", 
    "NRTIPORELACIONA", "NRTIPOLINHAVINC", "NREVENTOM", "VREVENTOFIXO", "NRTIPOLINHAVINC_3", "NRLINHATRANSP_4", "VRTARIFA_5", "NRTIPOLINHAVINC_4", 
    "NRTIPOLINHAVINC_7", "NRTPMODALIDSAL", "NRIMOVELENDERECO", "IDFATORRHPES", "NRNACIONALIDADE", "CDTIPOENDERECO", "CDLOGRADOURO", "NMBAIRROENDERECO", 
    "NRCEPENDERECO", "NRPERFACESAPROV", "DTFIMPLANEJADOTRE", "DTHABCARTPES", "IDSEXOPESSOA", "DTINIESCALA", "NRPESSOA", "IDPUBLICAVAGA", "DTINISITUFUNC", 
    "VRTARIFA", "NRTIPOVINCULOM", "NRLINHATRANSP_5", "NRLINHATRANSP_6", "VRTARIFA_6", "NRLINHATRANSP_7", "NRPROCESSO", "DTALTESALARIO", "IDADIANT13SALAR", 
    "ABONOPECUNIARIO", "DSPRONOMETRAT", "DSCOMPLEENDERECO", "CDCOMUNICAPARC", "NRGRAUINSTR", "NRRACAPESSOA", "IDTREINAMENTOINT", "NRCARTHABPES", "CDCONTCORR", 
    "NRTURNO", "NRTPAVIPRE", "NMPROCESSO", "NRSENTIDOLINHA", "DTFIMLINHAVINC", "NRSITUFUNCM", "DTFIMSITUFUNC", "DTCANCBENEFICIARIO", "CDTIPOALTEVT", 
    "NRVINCULOM", "DTMESCOMPETENC", "VRTARIFA_2", "QTLINHATARIFA_2", "NRLINHATRANSP_3", "VRBENEFICIO", "QTLINHATARIFA_4", "NRCPFPESSOA", "NOVOVALORSALARIO", 
    "DTFIMGOZOFERIAS", "DSREFERENCIAENDE", "DSENDERECO", "NMTREINAMENTO", "NRTPDEMISSAO", "DTNASCPESSOA", "DSCATEGHABCART", "VRSALARIO", "NRTIPOOCUPACAO", 
    "NRTPMOVTRANSFM", "NRTIPOPROCESSO", "DTLIMSOLICITA", "QTINIHOR", "CDPESSOANAOGRATA", "CDPOSSUIBENSDAEMPRESA", "CDTIPOALTEVC", "QTBENEFICIO", "NMOCUPACAOH",
    "NRESTRUTLEGAL", "QTLINHATARIFA_3", "NRMOTIVOADMISSAO", "NRLINHATRANSP_8", "DTRESCISAOPROJETADA", "IDGRUPSANGUPES", "NRCONDFISPES", "CDPREFIXCOMUPARC", 
    "NRTPCONTCORR", "DTINIOCUPACAO", "NRMOTIVOTRANSF", "NRESTRUTURAM", "NRVAGAS", "NRPARCNEGOCIORESP", "NRTEMPLATE", "QTFIMHOR", "LOCALIDADE", "NRQTDIASITUFUNC", 
    "CDREPOSICAOVAGA", "DTALTELINHAVINC", "NMVINCULOM", "DTINIVIGENCIA", "NRTIPOLINHAVINC_8", "DTPREVISTAADMISSAO", "DSOBSERVACAOAUXIMORADIA", "NRMETODOTRE", 
    "DTRESCISAOVINC", "DTAVISOPREVIO", "CDAGENCIA", "NRCERTCASAPES", "DTCASAPESSOA", "NRESCALATRABM", "DTINSCBENEFICIARIO", "DSMOTIVO", "DTSOLICITA", 
    "NMPESSOADEP", "DTINILINHAVINC", "NRLINHATRANSP", "NRFERIAS", "DSGOZOFERIAS", "IDTIPOMOVIMENTO", "NRLINHATRANSP_2", "DSOBSALTEOCUPACAO", "IDTPSALARIO", "DTINIGOZOFERIAS", 
    "NMPESSOA", "CDESTACIVIL", "DSAPELIDOPESS", "SGESTADO", "CDMUNICIPIO", "NRPERFACESEXEC", "DTVALHABCARTPES", "SGUFHABCARPES", "NROCORRENCIA", "DTFIMMOVIMENT", 
    "DTNASCIFUNDPARC", "NRCPFDEP", "DSDESCRICAO", "QTDIASABONOFERI", "DSOBSLINHAVINC", "DSOBSALTESALARIO", "NRTIPOLINHAVINC_2", "VRTARIFA_3", "NRVINCULOEMPREG", 
    "VRTARIFA_4", "VRTARIFA_7", "QTLINHATARIFA_6", "QTLINHATARIFA_7", "QTLINHATARIFA_8", "NRTIPOLINHAVINC_5", "DSANOTACAO"];
    
    public function __construct(
        EntityManager $entityManager,
        SolicitacaoService $solicitacaoService,
        BeneficioService $beneficioService,
        MovimentacaoService $movimentacaoService,
        MinhaHistoriaService $minhaHistoriaService
    ) {
        $this->entityManager        = $entityManager;
        $this->solicitacaoService   = $solicitacaoService;
        $this->beneficioService     = $beneficioService;
        $this->movimentacaoService  = $movimentacaoService;
        $this->minhaHistoriaService = $minhaHistoriaService;
    }


    public function getSolicitacoesVinculosGestor(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutura            = (int) $row['NRESTRUTURAM'];
            $cdOperadorAprovante    =       $row['CDOPERADOR_APROVANTE'] ?? null;

            $solicitacoes = $this->solicitacaoService->
                getSolicitacoesVinculosGestor($nrEstrutura, $cdOperadorAprovante);

            $response->addDataSet(new DataSet('getSolicitacoesVinculosGestor', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getItemSolicitacao(Request\Row $request, Response $response) {
        try {
            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'];

            $solicitacoes = $this->solicitacaoService->getItemSolicitacao($nrsolicitacao);

            $response->addDataSet(new DataSet('getItemSolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getTiposolicitacao(Request\Row $request, Response $response) {
        try {

            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'] ?? null;
            $nrvinculom = $params['NRVINCULOM'] ?? null;

            $solicitacoes = $this->solicitacaoService->getTiposolicitacao($nrsolicitacao, $nrvinculom);

            $response->addDataSet(new DataSet('getTiposolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTipoRelacionamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposRel = $this->solicitacaoService->getTipoRelacionamento();

            $response->addDataSet(new DataSet('getTipoRelacionamento', $tiposRel));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getLinhaTransporteAtiva(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $linhas = $this->solicitacaoService->getLinhaTransporteAtiva();

            $response->addDataSet(new DataSet('getLinhaTransporteAtiva', $linhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipolinhavinc(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposLinhas = $this->solicitacaoService->getTipolinhavinc();

            $response->addDataSet(new DataSet('getTipolinhavinc', $tiposLinhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getIntegtransp(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $numerosIntegracao = $this->solicitacaoService->getIntegtransp();

            $response->addDataSet(new DataSet('getIntegtransp', $numerosIntegracao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSentidolinha(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $sentidos = $this->solicitacaoService->getSentidolinha();

            $response->addDataSet(new DataSet('getSentidolinha', $sentidos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getBeneficios(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosComFiltro();

            $response->addDataSet(new DataSet('getBeneficios', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoBeneficio(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipoBeneficio = $this->beneficioService->getTipoBeneficioComFiltro();

            $response->addDataSet(new DataSet('getTipoBeneficio', $tipoBeneficio));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getPessoaRelacionamentos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrPessoa = (int) $row['NRPESSOA'];
            
            $pessoas = $this->solicitacaoService->getPessoaRelacionamentos($nrPessoa);

            $response->addDataSet(new DataSet('getPessoaRelacionamentos', $pessoas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoDependente(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposDep = $this->solicitacaoService->getTipoDependente();

            $response->addDataSet(new DataSet('getTipoDependente', $tiposDep));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTemplate(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $templates = $this->solicitacaoService->getTemplate();

            $response->addDataSet(new DataSet('getTemplate', $templates));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoprocessos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipos = $this->solicitacaoService->getTipoprocessos();

            $response->addDataSet(new DataSet('getTipoprocessos', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getParcnegocioVinculos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $parcs = $this->solicitacaoService->getParcnegocioVinculos();

            $response->addDataSet(new DataSet('getParcnegocioVinculos', $parcs));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipos = $this->solicitacaoService->getTipoTransferencia();

            $response->addDataSet(new DataSet('getTipoTransferencia', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMovimentacaoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $transferencias = $this->solicitacaoService->getMovimentacaoTransferencia();

            $response->addDataSet(new DataSet('getMovimentacaoTransferencia', $transferencias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMotivoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $motivos = $this->solicitacaoService->getMotivoTransferencia();

            $response->addDataSet(new DataSet('getMotivoTransferencia', $motivos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTiposAnotacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposAnotacao = $this->solicitacaoService->getTiposAnotacao();

            $response->addDataSet(new DataSet('getTiposAnotacao', $tiposAnotacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function aprovaVariasSolicitacoes(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $rows = $request->getRow();
            unset($rows['NRORG']);

            $resp = [
                'SUCCESS' => false,
                'ERRORS' => [],
                'ERROR_NRSOLICITACOES' => []
            ];
    
            foreach($rows as $row) {
                $nrOrg                  = (int)    $row['NRORG'];
                $nrSolicitacao          = (string) $row['NRSOLICITACAO'];
                $cdOperadorAprovante    = (string) $row['CDOPERADOR_APROVANTE'];
                $dsObservacao           = $row['DSOBSERVACAO'] ?? null;
                $extraRow               = $row['EXTRA_ROW'] ?? [];
                
                try {
                    $this->solicitacaoService->aprovaSolicitacao(
                        $nrSolicitacao,
                        $cdOperadorAprovante,
                        $dsObservacao,
                        $extraRow,
                        true
                    );
                } catch (\Exception $e) {
                    $resp['ERRORS'][] = 'Erro Nr. Solicitação - ' . $nrSolicitacao . ': ' . $e->getMessage();
                    $resp['ERROR_NRSOLICITACOES'][] = $nrSolicitacao;
                }
            }
    
            if (!empty($resp['ERRORS'])) {
                $response->addDataSet(new DataSet('aprovaVariasSolicitacoes', $resp));
            } else {
                $resp['SUCCESS'] = true;
                $response->addDataSet(new DataSet('aprovaVariasSolicitacoes', $resp));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    

    public function reprovaVariasSolicitacoes(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $rows = $request->getRow();
            unset($rows['NRORG']);

            $resp = [
                'SUCCESS' => false,
                'ERRORS' => [],
                'ERROR_NRSOLICITACOES' => []
            ];
    
            foreach($rows as $row) {
                $nrSolicitacao          = (string) $row['NRSOLICITACAO'];
                $cdOperadorAprovante    = (string) $row['CDOPERADOR_APROVANTE'];
                $dsObservacao           = $row['DSOBSERVACAO'] ?? null;
                $extraRow               = $row['EXTRA_ROW'] ?? [];
                
                try {
                    $this->solicitacaoService->reprovaSolicitacao(
                        $nrSolicitacao,
                        $cdOperadorAprovante,
                        $dsObservacao,
                        $extraRow
                    );
                } catch (\Exception $e) {
                    $resp['ERRORS'][] = 'Erro Nr. Solicitação ' . $nrSolicitacao . ': ' . $e->getMessage();
                    $resp['ERROR_NRSOLICITACOES'][] = $nrSolicitacao;
                }
            }
    
            if (!empty($resp['ERRORS'])) {
                $response->addDataSet(new DataSet('reprovaVariasSolicitacoes', $resp));
            } else {
                $resp['SUCCESS'] = true;
                $response->addDataSet(new DataSet('reprovaVariasSolicitacoes', $resp));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function aprovaSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $extraRow               =           $row['EXTRA_ROW']               ?? [];
            
            $success = $this->solicitacaoService->aprovaSolicitacao(
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('aprovaSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function reprovaSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $cdStatus               =           $row['CDSTATUS']                ?? null;
            $extraRow               =  (array)  ($row['EXTRA_ROW']              ?? []);
            
            $success = $this->solicitacaoService->reprovaSolicitacao(
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('reprovaSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function notificaProximosAprovadoresSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            
            $nrSolicitacao = (string) $row['NRSOLICITACAO'];
            
            $success = Utils::getInstanceByClassName(EmailService::class)->notificaProximosAprovadoresSolicitacao($nrSolicitacao);

            $response->addDataSet(new DataSet('notificaProximosAprovadoresSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function notificaOperadoresReprovacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            
            $nrSolicitacao = (string) $row['NRSOLICITACAO'];
            
            $success = Utils::getInstanceByClassName(EmailService::class)->notificaOperadoresReprovacao($nrSolicitacao);

            $response->addDataSet(new DataSet('notificaOperadoresReprovacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getFasesSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao = (string) $row['NRSOLICITACAO'];

            $fases = $this->solicitacaoService->getFasesSolicitacao(
                $nrSolicitacao
            );

            $response->addDataSet(new DataSet('getFasesSolicitacao', $fases));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getBeneficiosVCombust(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosVCombust();

            $response->addDataSet(new DataSet('getBeneficiosVCombust', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getBeneficiosVManutencao(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosVManutencao();

            $response->addDataSet(new DataSet('getBeneficiosVManutencao', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosPreviosSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo           = (int) $row['NRVINCULOM'];
            $nrTipoSolicitacao   =  (int) $row['NRTIPOSOLICITACAO'];

            $solicitacaoMethods = [
                1  => 'getDadosFeriasVinculo',
                2  => 'getAlteracaoTransferencia',
                9  => 'getAlteracaoOcupacao',
                14 => 'getAlteracaoEscalaTrabalho',
                10 => 'getLinhasVinculosVT'
            ];

            $solicitacoes = [];
    
            if (array_key_exists($nrTipoSolicitacao, $solicitacaoMethods)) {
                $methodName = $solicitacaoMethods[$nrTipoSolicitacao];
                $solicitacoes = $this->solicitacaoService->$methodName($nrVinculo);
            } else {
                throw new \Exception('Tipo de solicitação inválido');
            }

            $response->addDataSet(new DataSet('getDadosPreviosSolicitacao', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function verificaTipoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrtipotransf     = (int) $row['NRTIPOTRANSFER'];
            $nrTipoEstrutura  =  (int) $row['NRTIPOESTRUTURA'];
            
            $this->movimentacaoService->verificaTipoTransferencia($nrtipotransf, $nrTipoEstrutura);
            
            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('verificaTipoTransferencia', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacaoSubstituicaoOrVinculoSubst(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];

            $solicitacao = $this->solicitacaoService->getSolicitacaoSubstituicaoOrVinculoSubst($nrVinculo);

            $response->addDataSet(new DataSet('getSolicitacaoSubstituicaoOrVinculoSubst', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];

            $solicitacao = $this->solicitacaoService->getDadosVinculo($nrVinculo);

            $response->addDataSet(new DataSet('getDadosVinculo', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function setDataSoliSubstituicaoVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];
            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $this->solicitacaoService->setDataSoliSubstituicaoVinculo($nrVinculo, $nrSolicitacao);

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('setDataSoliSubstituicaoVinculo', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getFolhaFechada(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculom  = (string) $row['NRVINCULOM'];
            $dtAlteracao = (string) $row['DTALTERACAO'];

            $folha = $this->solicitacaoService->getFolhaFechada($nrVinculom, $dtAlteracao);

            $response->addDataSet(new DataSet('getFolhaFechada', $folha));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getHierarquiaByEstruturaLegalAndGeren(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutLegal = (string) $row['NRESTRUTLEGAL'];
            $nrEstrutGeren  = (string) $row['NRESTRUTGEREN'];

            $hierarquia = $this->solicitacaoService->getHierarquiaByEstruturaLegalAndGeren($nrEstrutLegal, $nrEstrutGeren);

            $response->addDataSet(new DataSet('getHierarquiaByEstruturaLegalAndGeren', $hierarquia));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    public function getSolicitacoesAdmissao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $solicitacao = $this->solicitacaoService->getSolicitacoesAdmissao();

            $response->addDataSet(new DataSet('getSolicitacoesAdmissao', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function editSolicitacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrorg = $row['NRORG'];
            $nrSolicitacao = $row['NRSOLICITACAO'];

            $this->minhaHistoriaService->editSolicitacao($row);

            // Anexo ao email
            $filesPath = $this->minhaHistoriaService->lidarComFiles($row);

            $this->minhaHistoriaService->setDocumentosSolicitacao($row);


            $this->solicitacaoService->resetFases($nrSolicitacao, $nrorg);
            
            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('editSolicitacao', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMotivoRecusa(Request\Row $request, Response $response) {
        try {

            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'];

            $motivoRecusas = $this->solicitacaoService->getMotivoRecusa($nrsolicitacao);
            $motivos = [];

            if(!empty($motivoRecusas)) {
                foreach ($motivoRecusas as $motivoRecusa) {
                    $motivo = 'Solicitação reprovada na fase: "' . $motivoRecusa['NMFASEAPROVSOLI'] . '" pelo operador: "' . $motivoRecusa['CDOPERINCLUSAO'] . '-' . $motivoRecusa['NMOPERADOR'] . '" no dia: ' . $motivoRecusa['DTINCLUSAO'] . ' pelo motivo: "' . $motivoRecusa['DSOBSERVACAO'] . '"';
                    $motivos[] = $motivo;
                }
            }
            
            $response->addDataSet(new DataSet('getMotivoRecusa', $motivos));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTodosVinculosRescindidos(Request\Row $request, Response $response) {
        try {

            $vinculos = $this->solicitacaoService->getTodosVinculosRescindidos();

            $response->addDataSet(new DataSet('getTodosVinculosRescindidos', $vinculos));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getDadosSolicitacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrorg = $row['NRORG'];

            $filters = json_decode($row['FILTERS'], true);
            
            if(isset($filters['__is_valid'])) {
                unset($filters['__is_valid']);
            }
            if(isset($filters['NRTIPOHIERARQ']) || $filters['NRTIPOHIERARQ'] == null) {
                unset($filters['NRTIPOHIERARQ']);
            }

            $reportName = substr($filters['reportName'], 12);
            unset($filters['reportName']);

            $dataset = $this->solicitacaoService->getDadosSolicitacaoToExport($filters, $reportName, $nrorg);
            
            if(!empty($dataset)) {
                $dataset = $this->minimizeDataset($dataset, $reportName);
            }
            
            $response->addDataSet(new DataSet('getDadosSolicitacao', $dataset));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    private function minimizeDataset($dataset, $reportName) {
        $newDataSet = array();
        $selectedSoli = null;
        $contador = 0;
        
        foreach($dataset as $data) {
            if($selectedSoli != $data['NRSOLICITACAO']) {
                if($selectedSoli != null) {
                    $newDataSet[$contador] = $this->fillEmptyFields($newDataSet[$contador], $reportName);
                    $contador++;
                }

                $selectedSoli = $data['NRSOLICITACAO'];
                
                $newDataSet[] = $this->setNewNamesToColumns($data);

                if($data['DSLABELITEMSOLI'] == 'Estrutura Legal' || $data['DSLABELITEMSOLI'] == 'Estrutura Gerencial') {
                    $newDataSet[$contador][$data['DSLABELITEMSOLI'].' (Item)'] = $data['DSVALORITEMSOLI'];
                } else {
                    $newDataSet[$contador][$data['DSLABELITEMSOLI']] = $data['DSVALORITEMSOLI'];
                }

                unset($newDataSet[$contador]['DSCHAVEITEMSOLI']);
                unset($newDataSet[$contador]['DSLABELITEMSOLI']);
                unset($newDataSet[$contador]['DSVALORITEMSOLI']);
            } else {
                if($data['DSLABELITEMSOLI'] == 'Estrutura Legal' || $data['DSLABELITEMSOLI'] == 'Estrutura Gerencial') {
                    $newDataSet[$contador][$data['DSLABELITEMSOLI'].' (Item)'] = $data['DSVALORITEMSOLI'];
                } else {
                    $newDataSet[$contador][$data['DSLABELITEMSOLI']] = $data['DSVALORITEMSOLI'];
                }
            }
        }
        
        return $newDataSet;
    }

    private function setNewNamesToColumns($dataset) {
        $newDataSet = array(
            'N. Solicitação'       => $dataset['NRSOLICITACAO'],
            'Solicitação'          => $dataset['NMTIPOSOLICITACAO'],
            'N. Vínculo'           => $dataset['NRVINCULOM'],
            'Vínculo'              => $dataset['NMVINCULOM'],
            'Estrutura Legal'      => $dataset['NMESTRUTLEGAL'],
            'Estrutura Gerencial'  => $dataset['NMESTRUTGEREN'],
            'Data de abertura'     => $dataset['DTSOLICITACAO'],
            'Status'               => $dataset['DSAPROVACAOSEC'],
            'Observação'           => $dataset['DSOBSSOLICITACAO'],
            'Fase de Solicitação'  => $dataset['NMFASEAPROVSOLI'],
            'Fase seguinte'        => $dataset['NMFASEAPROVSOLI_SEGUINTE'],
            'Fase seguinte'        => $dataset['NMFASEAPROVSOLI_SEGUINTE'],
        );

        return $newDataSet;
    }
    
    // Preenche os campos de itens não obrigatórios caso estejam faltando
    private function fillEmptyFields($dataset, $reportName) {
        $newDataSet = array();

        switch ($reportName){
            case 'alteracao_salarial':
                if(!isset($dataset['DSOBSALTESALARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSALTESALARIO'] = '';
                }
                
                break;
            
            case 'alteracao_ocupacao':
                if(!isset($dataset['DSOBSALTEOCUPACAO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSALTEOCUPACAO'] = '';
                }

                break;
            
            case 'aumento_quadro':
                $newDataSet = $dataset;
                if(!isset($dataset['NRTEMPLATE'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTEMPLATE'] = '';
                }

                if(!isset($dataset['NRTIPOPROCESSO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOPROCESSO'] = '';
                }

                if(!isset($dataset['DSMOTIVO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSMOTIVO'] = '';
                }

                if(!isset($dataset['LOCALIDADE'])) {
                    $newDataSet = $dataset;
                    $newDataSet['LOCALIDADE'] = '';
                }

                if(!isset($dataset['DSDESCRICAO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSDESCRICAO'] = '';
                }

                if(!isset($dataset['NRVINCULOEMPREG'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRVINCULOEMPREG'] = '';
                }

                if(!isset($dataset['NRGRAUINSTR'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRGRAUINSTR'] = '';
                }

                break;
            
            case 'desligamento':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRTPAVIPRE'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTPAVIPRE'] = '';
                }

                if(!isset($dataset['NRMOTIVORESC'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRMOTIVORESC'] = '';
                }

                break;
            
                /**aqui */
            
            case 'reducao_salarial':
                $newDataSet = $dataset;

                if(!isset($dataset['NRESTRUTLEGAL_ITEM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRESTRUTLEGAL_ITEM'] = '';
                }

                if(!isset($dataset['NRESTRUTGEREN_ITEM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRESTRUTGEREN_ITEM'] = '';
                }
                
                if(!isset($dataset['NRTPMOVTRANSFM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTPMOVTRANSFM'] = '';
                }

                if(!isset($dataset['NRMOTIVOTRANSF'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRMOTIVOTRANSF'] = '';
                }
                
                if(!isset($dataset['NROCUPACAOM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NROCUPACAOM'] = '';
                }

                if(!isset($dataset['NRMOTIVOOCUPA'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRMOTIVOOCUPA'] = '';
                }

                if(!isset($dataset['NRTIPOOCUPACAO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOOCUPACAO'] = '';
                }

                if(!isset($dataset['DTFIMMOVIMENT'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTFIMMOVIMENT'] = '';
                }

                if(!isset($dataset['DSOBSALTESALARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSALTESALARIO'] = '';
                }
                break;
            
            case 'auxilio_moradia':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DTFIMVIGENCIA'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTFIMVIGENCIA'] = '';
                }

                if(!isset($dataset['DSOBSERVACAOAUXIMORADIA'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSERVACAOAUXIMORADIA'] = '';
                }
                break;
            
            case 'beneficios':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DTCANCBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTCANCBENEFICIARIO'] = '';
                }

                if(!isset($dataset['DSOBSBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSBENEFICIARIO'] = '';
                }
                break;
            
            case 'ferias':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DSGOZOFERIAS'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSGOZOFERIAS'] = '';
                }

                break;
            
            case 'inclusao_dependentes':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DTNASCIFUNDPARC'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTNASCIFUNDPARC'] = '';
                }

                if(!isset($dataset['NRCPFDEP'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRCPFDEP'] = '';
                }
                break;
            
            case 'movimentacao_ocupacao_salario':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRESTRUTLEGAL_ITEM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRESTRUTLEGAL_ITEM'] = '';
                }

                if(!isset($dataset['NRESTRUTGEREN_ITEM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRESTRUTGEREN_ITEM'] = '';
                }

                if(!isset($dataset['NRTPMOVTRANSFM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTPMOVTRANSFM'] = '';
                }

                if(!isset($dataset['NRMOTIVOTRANSF'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRMOTIVOTRANSF'] = '';
                }

                if(!isset($dataset['VRSALARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRSALARIO'] = '';
                }

                if(!isset($dataset['IDTPSALARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['IDTPSALARIO'] = '';
                }

                if(!isset($dataset['NRTIPOALTERA'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOALTERA'] = '';
                }

                if(!isset($dataset['DTFIMMOVIMENT'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTFIMMOVIMENT'] = '';
                }
                
                if(!isset($dataset['DSOBSALTESALARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSALTESALARIO'] = '';
                }
                break;

            case 'transferencia_gerencial':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRMOTIVOTRANSF'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRMOTIVOTRANSF'] = '';
                }

                break;
            
            case 'transferencia_legal':
                $newDataSet = $dataset;

                if(!isset($dataset['NRESTRUTGEREN_ITEM'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRESTRUTGEREN_ITEM'] = '';
                }
                
                if(!isset($dataset['NRMOTIVOTRANSF'])) {
                    $newDataSet = $dataset; 
                    $newDataSet['NRMOTIVOTRANSF'] = '';
                }

                break;
            
            case 'substituicao':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRVINCULOEMPREG'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRVINCULOEMPREG'] = '';
                }
                
                if(!isset($dataset['NRGRAUINSTR'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRGRAUINSTR'] = '';
                }
                
                if(!isset($dataset['DTPREVISTAADMISSAO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTPREVISTAADMISSAO'] = '';
                }

                break;
            
            case 'treinamento':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRPERFACESAPROV'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRPERFACESAPROV'] = '';
                }

                if(!isset($dataset['NRPERFACESEXEC'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRPERFACESEXEC'] = '';
                }
                break;

            case 'vale_combustivel':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DTCANCBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTCANCBENEFICIARIO'] = '';
                }

                if(!isset($dataset['DSOBSBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSBENEFICIARIO'] = '';
                }
                break;

            case 'vale_manutencao':
                $newDataSet = $dataset;
                
                if(!isset($dataset['DSOBSBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DSOBSBENEFICIARIO'] = '';
                }

                if(!isset($dataset['DTCANCBENEFICIARIO'])) {
                    $newDataSet = $dataset;
                    $newDataSet['DTCANCBENEFICIARIO'] = '';
                }
                break;

            case 'vale_transporte':
                $newDataSet = $dataset;
                
                if(!isset($dataset['NRLINHATRANSP_2'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_2'] = '';
                }

                if(!isset($dataset['VRTARIFA_2'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_2'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_2'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_2'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_3'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_3'] = '';
                }

                if(!isset($dataset['VRTARIFA_3'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_3'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_3'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_3'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_4'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_4'] = '';
                }

                if(!isset($dataset['VRTARIFA_4'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_4'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_4'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_4'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_5'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_5'] = '';
                }

                if(!isset($dataset['VRTARIFA_5'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_5'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_5'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_5'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_6'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_6'] = '';
                }

                if(!isset($dataset['VRTARIFA_6'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_6'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_6'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_6'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_7'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_7'] = '';
                }

                if(!isset($dataset['VRTARIFA_7'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_7'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_7'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_7'] = '';
                }
                
                if(!isset($dataset['NRLINHATRANSP_8'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRLINHATRANSP_8'] = '';
                }

                if(!isset($dataset['VRTARIFA_8'])) {
                    $newDataSet = $dataset;
                    $newDataSet['VRTARIFA_8'] = '';
                }

                if(!isset($dataset['NRTIPOLINHAVINC_8'])) {
                    $newDataSet = $dataset;
                    $newDataSet['NRTIPOLINHAVINC_8'] = '';
                }
                break;

            default:
                $newDataSet = $dataset;
                break;
        }

        return $newDataSet;
    }
}
