<?php

namespace Ems\Logic\ProcessamentoSolicitacao;
use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoAdvertenciaAnotVinc extends ProcessamentoSolicitacao {
        
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        $solicitacao = $this->getSolicitacao();
        
        $nrorg              = Utils::getParamters()->getNrorg();
        $nrvinculom         = $this->getSolicitacao()->getNrvinculom();
        $dsobssolicitacao   = $this->getSolicitacao()->getDsobssolicitacao();
        $dtcompetencia      = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);
        $observacao         = $this->getObservacaoComProtocolo($dsobssolicitacao);
        
        // ADVÊRTENCIA/SUSPENSÃO
        $nrsitufuncm        = $itens['NRSITUFUNCM']     ?? null; 
        $dtinisitufunc      = isset($itens['DTINISITUFUNC']) ? DateUtil::getDataDeString(substr($itens['DTINISITUFUNC'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimsitufunc      = isset($itens['DTFIMSITUFUNC']) ? DateUtil::getDataDeString(substr($itens['DTFIMSITUFUNC'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $nrqtdiasitufunc    = $itens['NRQTDIASITUFUNC']  ?? null;
        $nrtipoanotacao     = $itens['NRTIPOANOTACAO']  ?? null;
        $dsanotacao         = $itens['DSANOTACAO']  ?? null;

        // VALIDAÇÕES
        $vinculom = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg, "nrvinculom" => $nrvinculom));
        if($vinculom->getDtrescisaovinc())
            throw new \Exception("O vínculo está rescindido!");
        // END
        
        // Advertência não gera situação funcional
        if($nrsitufuncm != '18') {
            // VALIDAÇÕES
            $existSituFunc = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(array("nrorg"=> $nrorg, "nrvinculom" => $nrvinculom,"dtinisitufunc" => $dtinisitufunc));
            if($existSituFunc)
                throw new \Exception("Já existe uma alteração de situação funcional com esta data");
            
            
            $proximaSituFunc = $this->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, null);
            if($proximaSituFunc)
                throw new \Exception("Operação bloqueada! <br>Já existe uma alteração funcional com data posterior a alteração funcional adicionada!");
        
            
            $dtinisitufuncAnteriorAdmissaoVinc = DateUtil::getDataDeString(substr($dtinisitufunc,0, 10),DateUtil::FORMATO_BRASILEIRO, true)->format('Y/m/d') < $vinculom->getDtadmissaovinc()->format('Y/m/d');
            if($dtinisitufuncAnteriorAdmissaoVinc)
                throw new \Exception("Data de Início da Situação Funcional anterior à admissão do vínculo");
            // END
            
            // Atualizar data fim da ultima situação funcional se necessario        
            $ultimaSituFunc =  ProcessamentoSolicitacaoUtil::atualizaDatafimUltimaSituFunc($nrorg, $nrvinculom, $dtinisitufunc);
            
            // Cria a nova situação funcional
            $nrproximasitufunc = $ultimaSituFunc->getNrsitufuncm() ? $ultimaSituFunc->getNrsitufuncm() : null;
            $alteSituFunc = ProcessamentoSolicitacaoUtil::novaAlteSituacaoFuncional($nrorg, $nrvinculom, $nrsitufuncm, $observacao, $dtinisitufunc, $dtfimsitufunc, $nrqtdiasitufunc, $nrproximasitufunc);
            
            // Cria a proxima situação funcional
            if($ultimaSituFunc) {
                $proxdtinisitufunc = DateUtil::adicionaIntervalo($dtfimsitufunc, 1);
                $ultimanrsitufuncm = $ultimaSituFunc->getNrsitufuncm() ? $ultimaSituFunc->getNrsitufuncm() : '1';
                $alteSituFunc = ProcessamentoSolicitacaoUtil::novaAlteSituacaoFuncional($nrorg, $nrvinculom, $ultimanrsitufuncm, $observacao, $proxdtinisitufunc);
            }
        }

        $existeAnotacao = $this->retornaAnotacaoVinculo($nrorg, $nrvinculom, $dtinisitufunc);
        if($existeAnotacao) {
            throw new \Exception("Já existe uma anotação decorrente de solicitação com esta data");
        }

        // Cria a nova anotação de vínculo
        $this->criaNovaAnotacaoVinculo($nrorg, $solicitacao->getNrSolicitacao(), $nrvinculom, $nrtipoanotacao, $dtinisitufunc, $dtfimsitufunc, $dsanotacao);

        // $this->atualizaVinculoHistorico($nrorg, $nrvinculom, $nrsitufuncm, Utils::getParamters()->getCdoperador());
    }
    
    public function rejeitar() {}
    
    public function atualizaVinculoHistorico($nrorg, $nrvinculom, $nrsitufuncm, $cdoperador) {
        $sql = <<<SQL
        UPDATE GPE_VINCULOH VH
        SET VH.NRSITUFUNCM = :P_NRSITUFUNCM,
            VH.NRORGULTATU = :P_NRORG,
            VH.CDOPERULTATU = :P_CDOPERADOR
        WHERE VH.DTMESCOMPETENC = (
            SELECT MAX(DTMESCOMPETENC)
            FROM GPE_VINCULOH VHH
            WHERE VHH.NRORG = VH.NRORG
            AND VHH.NRVINCULOM = VH.NRVINCULOM
        )
        AND VH.NRORG = :P_NRORG
        AND VH.NRVINCULOM = :P_NRVINCULOM
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"        => $nrorg,
                "P_NRVINCULOM"   => $nrvinculom,
                "P_NRSITUFUNCM"  => $nrsitufuncm,
                "P_CDOPERADOR"   => $cdoperador
            )
        );
    }

    public function retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc=null) {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $sql = <<<SQL
            SELECT a
            FROM $gpeAltesitufunc a
            WHERE a.nrorg         = :NRORG
            AND a.nrvinculom    = :NRVINCULOM
            AND a.dtinisitufunc > :DTINISITUFUNC
            AND (:NRSITUFUNCM IS NULL OR a.nrsitufuncm = :NRSITUFUNCM)
  
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'NRVINCULOM' => $nrvinculom,
            'NRSITUFUNCM' => $nrproxsitufunc,
            'DTINISITUFUNC' => $dtinisitufunc
        );

        $result = Utils::getEntityManager()->createQuery($sql)->execute($params);

        return empty($result) ? $result : $result[0];
    }

    public function retornaAnotacaoVinculo($nrorg, $nrvinculom, $dtinisitufunc) {
        $sql = <<<SQL
            SELECT
                1
            FROM
                GPE_ANOTACOESVINCULO
            WHERE
                NRORG = :P_NRORG
                AND NRVINCULOM = :P_NRVINCULOM
                AND DSANOTACAO LIKE :P_DSANOTACAO

  
SQL;
        
        $dtinisitufunc = $dtinisitufunc->format('d/m/y');
        $dsanotacao = '%com data incial em: ' . $dtinisitufunc . '%';

        $params = array(
            
        );

        $result = Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DSANOTACAO' => $dsanotacao
            )
        );

        return empty($result) ? $result : $result[0];
    }

    public function criaNovaAnotacaoVinculo($nrorg, $nrsolicitacao, $nrvinculom, $nrtipoanotacao, $dtinisitufunc, $dtfimsitufunc, $dsanotacao) {

        $sql = <<<SQL
            INSERT INTO
                GPE_ANOTACOESVINCULO(
                    NRANOTACAOVINCULO, 
                    NRORG, 
                    NRTIPOANOTACAO, 
                    NRVINCULOM, 
                    DSANOTACAO, 
                    DTINCLUSAO, 
                    DTULTATU, 
                    NRORGINCLUSAO, 
                    CDOPERINCLUSAO, 
                    NRORGULTATU, 
                    CDOPERULTATU, 
                    IDATIVO
                )
            VALUES(
                :P_NRANOTACAOVINCULO, 
                :P_NRORG, 
                :P_NRTIPOANOTACAO, 
                :P_NRVINCULOM, 
                :P_DSANOTACAO, 
                :P_DTINCLUSAO, 
                :P_DTULTATU, 
                :P_NRORGINCLUSAO, 
                :P_CDOPERINCLUSAO, 
                :P_NRORGULTATU, 
                :P_CDOPERULTATU, 
                'S'
            )
SQL;
        
        $nranotacaovinc = NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ANOTACOESVINCULO', 1, 12, true);
        $cdoperador = Utils::getParamters()->getCdoperador();
        $dsanotacaoaux = 'Anotação criada pela Solicitacação: '. $nrsolicitacao . ' com data incial em: ' . $dtinisitufunc->format('d/m/y');
        if(!empty($dtfimsitufunc)) {
            $dsanotacaoaux .= ' e com data final em: ' . $dtfimsitufunc;
        }
        $dsanotacao = $dsanotacaoaux . '
' . $dsanotacao; 
        
        $params = array(
            'P_NRANOTACAOVINCULO'   => $nranotacaovinc,
            'P_NRORG'               => $nrorg,
            'P_NRTIPOANOTACAO'      => $nrtipoanotacao,
            'P_NRVINCULOM'          => $nrvinculom,
            'P_DSANOTACAO'          => $dsanotacao,
            'P_DTINCLUSAO'          => DateUtil::getDataAtual(false)->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
            'P_DTULTATU'            => DateUtil::getDataAtual(false)->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
            'P_NRORGINCLUSAO'       => $nrorg,
            'P_CDOPERINCLUSAO'      => $cdoperador,
            'P_NRORGULTATU'         => $nrorg,
            'P_CDOPERULTATU'        => $cdoperador
        );

        return Utils::getEntityManager()->getConnection()->prepare($sql)->execute($params);
    }

}