<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class CertisignRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function get(
        int $nrOrg,
        int $nrIntegracao
    ) {
        $sql = <<<SQL
            SELECT *
            FROM GPE_INTELETRDIGIT
            WHERE
                NRINTELETRDIGIT = :P_NRINTELETRDIGIT AND
                NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRINTELETRDIGIT' => $nrIntegracao,
            'P_NRORG' => $nrOrg,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        $result = $stmt->fetch();

        return $result;
    }

    public function deleteIntegracao(
        int $nrOrg,
        int $nrIntegracao
    ) {
        $sql = <<<SQL
            DELETE FROM GPE_INTELETRDIGIT
            WHERE
                NRINTELETRDIGIT = :P_NRINTELETRDIGIT AND
                NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRINTELETRDIGIT' => $nrIntegracao,
            'P_NRORG' => $nrOrg,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function setDocumentAsRejected(
        array $params
    ) {
        $sql = <<<SQL
            UPDATE GPE_INTELETRDIGIT
            SET
                IDSTATUS = 'R',
                DTREJEICAO = SYSDATE,
                DSREJEICAO = :P_DSREJEICAO
            WHERE
                NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
                DSIDDOCUMENTO = :P_IDDOCUMENTO AND
                NRORG = :P_NRORG
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function setDocumentAsSigned(
        array $params
    ) {
        $sql = <<<SQL
            UPDATE GPE_INTELETRDIGIT
            SET
                DTASSINATURA = SYSDATE,
                IDSTATUS = 'A'
            WHERE
                NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
                DSIDDOCUMENTO = :P_IDDOCUMENTO AND
                NRORG = :P_NRORG
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function getDocumentosPendentesAssinaturaVinculo($params) {
        $sql = <<<SQL
            SELECT
                GID.*,
                GP.*,
                CAT.NMCATEGDOCUMENTO,
                TIPO.NMTIPODOCUMENTO,
                GVM.NMVINCULOM,
                GVM.NRVINCULOM,
                'N' AS ISACEITEGESTOR,
                CASE
                    WHEN GID.IDACEITEVINCULO = 'P' THEN 'Aguardando Apr. Vínculo'
                    WHEN GID.IDACEITEVINCULO = 'R' THEN 'Reprovado pelo vinculo'
                    WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'P' THEN 'Aguardando Apr. Gestor'
                    WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'R' THEN 'Reprovado pelo Gestor'
                    WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'A' THEN 'Aprovado pelo Gestor'
                    ELSE 'Status Não Identificado'
                END AS IDACEITESTATUS
            FROM
                GPE_INTELETRDIGIT GID
            JOIN
                GPE_ANEXOS GP ON
                    GP.NRORG = GID.NRORG AND
                    GP.NRPARCNEGOCIO = GID.NRPARCNEGOCIO AND
                    GP.NRANEXO = GID.NRANEXO
            LEFT JOIN
                GER_CATEGDOCUMENTO CAT ON
                    CAT.NRORG = GID.NRORG AND
                    CAT.NRCATEGDOCUMENTO = GID.NRCATEGDOCUMENTO
            LEFT JOIN
                GER_TIPODOCUMENTO TIPO ON
                    TIPO.NRORG = GID.NRORG AND
                    TIPO.NRTIPODOCUMENTO = GID.NRTIPODOCUMENTO
            LEFT JOIN
                GPE_VINCULOM GVM ON
                    GVM.NRORG = GID.NRORG AND
                    GVM.NRVINCULOM = :P_NRVINCULO
            WHERE
                GID.NRORG = :P_NRORG AND
                GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $docs = $std->fetchAll();

        return $docs;
    }

    public function getDocsPendentesAssiVinculosDoGestor($params) {
        $sql = <<<SQL
        SELECT
            GID.*,
            GP.*,
            CAT.NMCATEGDOCUMENTO,
            TIPO.NMTIPODOCUMENTO,
            GVM.NMVINCULOM,
            GVM.NRVINCULOM,
            'S' AS ISACEITEGESTOR,
            CASE
                WHEN GID.IDACEITEVINCULO = 'P' THEN 'Aguardando Apr. Vínculo'
                WHEN GID.IDACEITEVINCULO = 'R' THEN 'Reprovado pelo Vínculo'
                WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'P' THEN 'Aguardando Apr. Gestor'
                WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'R' THEN 'Reprovado pelo Gestor'
                WHEN GID.IDACEITEVINCULO = 'A' AND GID.IDACEITEGESTOR = 'A' THEN 'Aprovado pelo Gestor'
                ELSE 'Status Não Identificado'
            END AS IDACEITESTATUS
        FROM
            GPE_INTELETRDIGIT GID
        JOIN
            GPE_ANEXOS GP ON
                GP.NRORG = GID.NRORG AND
                GP.NRPARCNEGOCIO = GID.NRPARCNEGOCIO AND
                GP.NRANEXO = GID.NRANEXO
        LEFT JOIN
            GER_CATEGDOCUMENTO CAT ON
                CAT.NRORG = GID.NRORG AND
                CAT.NRCATEGDOCUMENTO = GID.NRCATEGDOCUMENTO
        LEFT JOIN
            GER_TIPODOCUMENTO TIPO ON
                TIPO.NRORG = GID.NRORG AND
                TIPO.NRTIPODOCUMENTO = GID.NRTIPODOCUMENTO
        LEFT JOIN
            GPE_VINCULOM GVM ON
                GVM.NRORG = GID.NRORG AND
                GVM.NRVINCULOM = :P_NRVINCULO
        WHERE
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.IDACEITEVINCULO <> 'P'
SQL;

    
    $std = $this->connection->prepare($sql);
    $std->execute($params);

    $docs = $std->fetchAll();

    return $docs;
    }

    public function getAssinaturaRelatorio($params) {
        $sql = <<<SQL
        SELECT
            *
        FROM
            GPE_INTELETRDIGIT GID
        WHERE
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.NRINTELETRDIGIT = :P_NRINTELETRDIGIT
SQL;

    
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $docs = $std->fetch();

        return $docs;
    }

    public function AprovaAssinaturaGestor(
        array $params
    ) {
        $sql = <<<SQL
        UPDATE GPE_INTELETRDIGIT GID
        SET
            DTACEITEGESTOR = SYSDATE,
            IDACEITEGESTOR = 'A',
            DSPATH = :P_DSPATH,
            DTULTATU = SYSDATE,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
        WHERE 
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.NRINTELETRDIGIT = :P_NRINTELETRDIGIT
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function AprovaAssinaturaVinculo(
        array $params
    ) {
        $sql = <<<SQL
        UPDATE GPE_INTELETRDIGIT GID
        SET
            DTACEITEVINCULO = SYSDATE,
            IDACEITEVINCULO = 'A',
            DSPATH = :P_DSPATH,
            DTULTATU = SYSDATE,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
        WHERE 
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.NRINTELETRDIGIT = :P_NRINTELETRDIGIT
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function reprovaAssinaturaGestor(
        array $params
    ) {
        $sql = <<<SQL
        UPDATE GPE_INTELETRDIGIT GID
        SET
            DTREJEICAO = SYSDATE,
            DSREJEICAO = :P_DSREJEICAO,
            DTACEITEGESTOR = SYSDATE,
            IDACEITEGESTOR = 'R',
            DTULTATU = SYSDATE,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
        WHERE 
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.NRINTELETRDIGIT = :P_NRINTELETRDIGIT
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function reprovaAssinaturaVinculo(
        array $params
    ) {
        $sql = <<<SQL
        UPDATE GPE_INTELETRDIGIT GID
        SET
            DTREJEICAO = SYSDATE,
            DSREJEICAO = :P_DSREJEICAO,
            DTACEITEVINCULO = SYSDATE,
            IDACEITEVINCULO = 'R',
            DTULTATU = SYSDATE,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
        WHERE 
            GID.NRORG = :P_NRORG AND
            GID.NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND
            GID.NRINTELETRDIGIT = :P_NRINTELETRDIGIT
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
    
    public function insertIntegracao(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO GPE_INTELETRDIGIT (
                NRORG,              NRINTELETRDIGIT,    NRPARCNEGOCIO,
                NRTIPODOCUMENTO,    NRCATEGDOCUMENTO,   IDSTATUS,
                DSIDDOCUMENTO,      DSLINKASSINAT,      NRANEXO,
                DSPATH,             DTTRANSMISSAO,      DTASSINATURA,
                IDATIVO,            DTINCLUSAO,         DTULTATU,
                NRORGINCLUSAO,      CDOPERINCLUSAO,     NRORGULTATU,
                CDOPERULTATU,       DTREJEICAO,         DSREJEICAO,
                DSCHAVEDOCUMENTO,   NMDOCUMENTO,        IDTIPOASSINATURA,
                NRIDSIGNATARIO,     NMFILE
            ) VALUES (
                :NRORG,             :NRINTELETRDIGIT,   :NRPARCNEGOCIO,
                :NRTIPODOCUMENTO,   :NRCATEGDOCUMENTO,  :IDSTATUS,
                :DSIDDOCUMENTO,     :DSLINKASSINAT,     :NRANEXO,
                :DSPATH,            :DTTRANSMISSAO,     :DTASSINATURA,
                :IDATIVO,           :DTINCLUSAO,        :DTULTATU,
                :NRORGINCLUSAO,     :CDOPERINCLUSAO,    :NRORGULTATU,
                :CDOPERULTATU,      :DTREJEICAO,        :DSREJEICAO,
                :DSCHAVEDOCUMENTO,  :NMDOCUMENTO,       :IDTIPOASSINATURA,
                :NRIDSIGNATARIO,    :NMFILE
            )
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function saveDocumentoAssinaturaDigital(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO GPE_INTELETRDIGIT
            (
                NRINTELETRDIGIT,    NRORG,              NRPARCNEGOCIO,
                NMFILE,             IDATIVO,             DSPATH,
                DTINCLUSAO,         CDOPERINCLUSAO,     IDSTATUS,
                NRCATEGDOCUMENTO,   NRTIPODOCUMENTO,    NRANEXO,
                NMDOCUMENTO
            )
            VALUES
            (
                :NRINTELETRDIGIT,   :NRORG,             :NRPARC,
                :NMFILE,            :IDATIVO,           :DSPATH,
                SYSDATE,            :CDOPERADOR,        :IDSTATUS,
                :NRCATEGORIA,       :NRTIPO,            :NRANEXO,
                :NMDOCUMENTO

            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}