<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;

class EscalaRepository {
    
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getEscalaTrabalhoVinculo(int $nrOrg, int $nrVinculom) {

        $sql = <<<SQL
            SELECT
                ESCM.*,
                ESCH.*
            FROM
                GPE_VINCULOM      VM
                JOIN GPE_VINCULOH      VH ON VH.NRORG = VM.NRORG
                                        AND VH.NRVINCULOM = VM.NRVINCULOM
                                        AND VH.DTMESCOMPETENC = (
                    SELECT
                        MAX(VHH.DTMESCOMPETENC)
                    FROM
                        GPE_VINCULOH VHH
                    WHERE
                        VHH.NRORG = VH.NRORG
                        AND VHH.NRVINCULOM = VH.NRVINCULOM
                )
                JOIN GPE_ESCALATRABM   ESCM ON ESCM.NRORG = VM.NRORG
                                             AND ESCM.NRESCALATRABM = VH.NRESCALATRABM
                JOIN GPE_ESCALATRABH   ESCH ON ESCH.NRORG = ESCM.NRORG
                                             AND ESCH.NRESCALATRABM = ESCM.NRESCALATRABM
                                             AND ESCH.DTMESCOMPETENC = (
                    SELECT
                        MAX(ESCHH.DTMESCOMPETENC)
                    FROM
                        GPE_ESCALATRABH ESCHH
                    WHERE
                        ESCHH.NRORG = ESCH.NRORG
                        AND ESCHH.NRESCALATRABM = ESCH.NRESCALATRABM
                )
            WHERE
                VM.NRORG = :P_NRORG
                AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(array(
            ':P_NRORG'      => $nrOrg,
            ':P_NRVINCULOM' => $nrVinculom,
        ));

        $marcacao = $std->fetch();

        return $marcacao;
    }

    public function getEscalaTrab(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                ETM.*,
                ETH.*,
                ETH.NRESCALATRABM || ' | ' ||  ETH.NMESCALATRABH AS NRNMESCALATRABM
            FROM
                GPE_ESCALATRABM ETM
                JOIN
                GPE_ESCALATRABH ETH ON
                    ETM.NRORG = ETH.NRORG AND
                    ETM.NRESCALATRABM = ETH.NRESCALATRABM AND
                    ETH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = ETH.NRORG AND
                            NRESCALATRABM = ETH.NRESCALATRABM AND
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                LEFT JOIN
                PTO_TIPOJORNADA TJ ON
                    (
                        TJ.NRORG = :P_NRORG OR
                        TJ.NRORG = :P_NRORG_PADRAO
                    ) AND
                    ETH.NRTPJORNADA = TJ.NRTPJORNADA
                LEFT JOIN
                PTO_TIPOINTERVALO TI ON
                    (
                        TI.NRORG = :P_NRORG OR
                        TI.NRORG = :P_NRORG_PADRAO
                    ) AND
                    ETH.NRTPINTERVALO = TI.NRTPINTERVALO
            WHERE
                (
                    ETM.NRORG = :P_NRORG OR
                    ETM.NRORG = :P_NRORG_PADRAO
                ) AND
                ETH.IDATIVO = 'S'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $escalas = $std->fetchAll();

        return $escalas;
    }

    public function getTurnos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                GPE_TURNO.*,
                GPE_TURNO.NRTURNO || ' | ' ||  GPE_TURNO.DSTURNO AS NRDSTURNO
            FROM
                GPE_TURNO
            WHERE
                NRESCALATRABM = :P_NRESCALATRABM AND
                NRORG = :P_NRORG AND
                IDATIVO = 'S'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $turnos = $std->fetchAll();

        return $turnos;
    }

    public function getTurnoVinculo(int $nrOrg, int $nrVinculom, int $nrEscalaTraBM) {

        $sql = <<<SQL
            SELECT
                GPE_TURNO.*
            FROM
                GPE_ALTEESCALA
            JOIN
                GPE_TURNO ON GPE_TURNO.NRTURNO = GPE_ALTEESCALA.NRTURNO
                    AND GPE_TURNO.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM
                    AND GPE_TURNO.IDATIVO = 'S'
            WHERE
                GPE_ALTEESCALA.NRVINCULOM = :P_NRVINCULOM
                AND GPE_ALTEESCALA.NRORG = :P_NRORG
                AND GPE_ALTEESCALA.NRESCALATRABM = :P_NRESCALATRABM
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(array(
            'P_NRORG'      => $nrOrg,
            'P_NRVINCULOM' => $nrVinculom,
            'P_NRESCALATRABM' => $nrEscalaTraBM
        ));

        $marcacao = $std->fetch();

        return $marcacao;
    }

    public function getTurnoParOuImpar(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT *
            FROM GPE_TURNO
            WHERE NRORG = :P_NRORG
            AND NRESCALATRABM = :P_NRESCALATRABM
            AND ROWNUM = 1
            AND EXISTS (SELECT 1
                            FROM ( SELECT MAX(NRSEQHORARIO) NRSEQHORARIO
                                    FROM GPE_HORARIOTURNO
                                    WHERE NRORG       = :P_NRORG
                                    AND NRTURNO       = GPE_TURNO.NRTURNO
                                    AND IDATIVO       = 'S') AUX 
                            JOIN DATA DT ON DT.DTDATA = TO_DATE( :P_DIA )
                            JOIN GPE_HORARIOTURNO HT ON HT.NRORG        = :P_NRORG
                                        AND HT.NRTURNO      = GPE_TURNO.NRTURNO
                                        AND HT.IDATIVO      = 'S'
                                        AND HT.NRSEQHORARIO = NVL( NULLIF( MOD( TO_CHAR( trunc(DT.DTDATA,'MM')+:P_NUMBER - 4, 'J' ), AUX.NRSEQHORARIO ), 0 ), AUX.NRSEQHORARIO )
                            JOIN GPE_HORDIAH HDH ON HDH.NRORG = :P_NRORG 
                                    AND HDH.NRHORDIAM = HT.NRHORDIAM
                                    AND HDH.IDTIPOHORARIO <> 'F'
                                    AND HDH.DTMESCOMPETENC = (
                                                    (SELECT DTMESCOMPETENC FROM 
                                                        (SELECT 
                                                            HDHH.DTMESCOMPETENC 
                                                        FROM 
                                                            GPE_HORDIAH HDHH
                                                        WHERE 
                                                            HDHH.NRORG = HDH.NRORG 
                                                            AND HDHH.NRHORDIAM = HDH.NRHORDIAM 
                                                        ORDER BY HDHH.DTMESCOMPETENC DESC) 
                                                    WHERE ROWNUM = 1)
                                                )
                        )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $turnos = $std->fetch();

        return !empty($turnos) ? $turnos : [];
    }
}