<?php

namespace Ems\Services;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Ems\Repository\OcorrenciaRepository;
use Util\{ NovoCodigo, Environment, DateUtil, Repositories };
use Ems\Util\{ ExpectedException, ArrayUtil, Utils };
use Ems\Repository\AdmissaoRepository;
use Ems\Repository\ParamOrgRepository;
use Ems\Services\OperadorService;
use Ems\Services\EstruturaService;
use Ems\Services\PessoaService;
use Ems\Services\DocumentoService;
use Ems\Services\ParceiroNegocioService;
use Ems\Services\VinculoService;
use Ems\Services\ValeTransporteService;
use Ems\Services\DependenteService;
use Ems\Services\BeneficioService;
use Ems\Services\OcorrenciaService;
use Ems\Services\BancoHorasService;
use Ems\Services\MovimentacaoService;
use Ems\Services\SituacaoFuncionalService;
use Ems\Services\AlteSalarioService;
use Ems\Services\OcupacaoService;
use Ems\Services\FormacaoService;
use Ems\Services\EmailService;
use Ems\Services\ParamOrgService;
use \Setforms2\Shared\ORM\DateTime;
use Ems\Util;
use Util\Messages;

class AdmissaoService {
    /** @var Environment */
    protected $environment;

    /** @var AdmissaoRepository */
    protected $admissaoRepository;

    /** @var ParamOrgRepository */
    protected $paramOrgRepository;

    /** @var OperadorService */
    protected $operadorService;

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var PessoaService */
    protected $pessoaService;

    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;

    /** @var DocumentoService */
    protected $documentoService;

    /** @var VinculoService */
    protected $vinculoService;

    /** @var ValeTransporteService */
    protected $valeTransporteService;

    /** @var DependenteService */
    protected $dependenteService;

    /** @var BeneficioService */
    protected $beneficioService;

    /** @var OcorrenciaService */
    protected $ocorrenciaService;

    /** @var OcorrenciaRepository */
    protected $ocorrenciaRepository;

    /** @var BancoHorasService */
    protected $bancoHorasService;

    /** @var MovimentacaoService */
    protected $movimentacaoService;

    /** @var SituacaoFuncionalService */
    protected $situacaoFuncionalService;

    /** @var AlteSalarioService */
    protected $alteSalarioService;

    /** @var OcupacaoService */
    protected $ocupacaoService;

    /** @var FormacaoService */
    protected $formacaoService;

    /** @var EmailService */
    protected $emailService;

    /** @var ParamOrgService */
    protected $paramOrgService;

    /** @var Connection */
    protected $connection;

    /** @var array */
    protected $dadosPessoaAdmissao;

    const ADMISSAO_SIMPLIFICADA_DADOS_PESSOAIS_REQUIRED_FIELDS = [
        'NMPESSOA', 'DTNASCPESSOA', 'NRCPFPESSOA',
    ];

    const ADMISSAO_SIMPLIFICADA_DADOS_VINCULO_REQUIRED_FIELDS = [
        'NRTIPOVINCULOM', 'DTADMISSAOVINC', 'NRVINCULOEMPREG', 'NRTIPOOCUPACAO',
        'NROCUPACAOM', 'NRESTRUTGEREN', 'NRESTRUTLEGAL', 'IDTPSALARIO',
        'VRSALARIO', 'NRTPMODALIDSAL',
    ];

    const ADMISSAO_COMPLETA_DADOS_PESSOAIS_REQUIRED_FIELDS = [
        'NMPESSOA', 'DTNASCPESSOA', 'NRCPFPESSOA', 'NRRGPESSOA',
        'IDSEXOPESSOA', 'CDESTACIVIL', 'NRRACAPESSOA', 'CDPAISNATURALI',
        'NRNACIONALID', 'NRRGPESSOA', 'CDEXRGPESSOA', 'DTEXRGPESSOA',
        'SGUFRGPESSOA', 'NRCTPSPESSOA', 'NRSERIECTPSPES', 'SGUFCTPSPES',
        'DTCTPSPESSOA', 'NRPISPASEPPES',
    ];

    const ADMISSAO_COMPLETA_DADOS_VINCULO_REQUIRED_FIELDS = [
        'NRTIPOVINCULOM', 'DTADMISSAOVINC', 'NRVINCULOEMPREG', 'NRTIPOOCUPACAO',
        'NROCUPACAOM', 'NRESTRUTGEREN', 'NRESTRUTLEGAL', 'IDTPSALARIO',
        'VRSALARIO', 'NRTPMODALIDSAL', 'DTEXAMEMEDICO', 'NRTPADMISSAO',
        'IDREMUNERACAO', 'IDTPPAGAMENTO', 'IDCONTRIBUISIND', 'DTINIOCUPACAO',
        'NRESCALATRABM', 'DTINIESCALA', 'NROCORRENCIA',
    ];

    const ADMISSAO_COMPLETA_FILIACAO_REQUIRED_FIELDS = [
        'NMMAE',
    ];

    const ADMISSAO_COMPLETA_ENDERECO_REQUIRED_FIELDS = [
        'NRCEPENDERECO', 'CDTIPOENDERECO', 'CDPAIS', 'SGESTADO',
        'CDMUNICIPIO', 'NMBAIRROENDERECO',
    ];

    public function __construct(
        Environment                 $environment,
        AdmissaoRepository          $admissaoRepository,
        OperadorService             $operadorService,
        EstruturaService            $estruturaService,
        PessoaService               $pessoaService,
        ParceiroNegocioService      $parceiroNegocioService,
        EntityManager               $entityManager,
        DocumentoService            $documentoService,
        ParamOrgRepository          $paramOrgRepository,
        VinculoService              $vinculoService,
        ValeTransporteService       $valeTransporteService,
        DependenteService           $dependenteService,
        BeneficioService            $beneficioService,
        OcorrenciaService           $ocorrenciaService,
        OcorrenciaRepository        $ocorrenciaRepository,
        BancoHorasService           $bancoHorasService,
        MovimentacaoService         $movimentacaoService,
        SituacaoFuncionalService    $situacaoFuncionalService,
        AlteSalarioService          $alteSalarioService,
        OcupacaoService             $ocupacaoService,
        FormacaoService             $formacaoService,
        EmailService                $emailService,
        ParamOrgService             $paramOrgService
    ) {
        $this->environment              = $environment;
        $this->admissaoRepository       = $admissaoRepository;
        $this->operadorService          = $operadorService;
        $this->estruturaService         = $estruturaService;
        $this->pessoaService            = $pessoaService;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->connection               = $entityManager->getConnection();
        $this->documentoService         = $documentoService;
        $this->paramOrgRepository       = $paramOrgRepository;
        $this->vinculoService           = $vinculoService;
        $this->valeTransporteService    = $valeTransporteService;
        $this->dependenteService        = $dependenteService;
        $this->beneficioService         = $beneficioService;
        $this->ocorrenciaService        = $ocorrenciaService;
        $this->ocorrenciaRepository     = $ocorrenciaRepository;
        $this->bancoHorasService        = $bancoHorasService;
        $this->movimentacaoService      = $movimentacaoService;
        $this->situacaoFuncionalService = $situacaoFuncionalService;
        $this->alteSalarioService       = $alteSalarioService;
        $this->ocupacaoService          = $ocupacaoService;
        $this->formacaoService          = $formacaoService;
        $this->emailService             = $emailService;
        $this->paramOrgService          = $paramOrgService;

        $this->dadosPessoaAdmissao = [];
    }

    public function getVinculosNaoAprovados(
        string $cdOperador,
               $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $isDP = $this->operadorService->operadorIsDP($cdOperador);

        $nrEstruturas = null;

        if (!$isDP) {
            $estruturas = $this->estruturaService->getTodasEstruturasInferiores($nrEstrutura);
            $nrEstruturas = ArrayUtil::toStringSQL($estruturas);
        }

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
            'P_NRESTRUTURAS'    => $nrEstruturas,
        ];

        $admissoes = $this->admissaoRepository->getVinculosNaoAprovados(
            $params
        );

        return $admissoes;
    }

    public function setDadosPessoaisAdmissao(
        array $row
    ): array {
        $dadosPessoais = (array) $row['dadosPessoais'];
        $enderecoRow = (array) $row['endereco'];
        $contaRow = (array) $row['conta'];
        $comuRow = (array) $row['comunicacao'];
        $documentos = (array) $row['documentos'];
        $dadosVinculo = isset($row['dadosVinculo']) ? (array) $row['dadosVinculo'] : array();
        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        $cdOperador = (string) $row['CDOPERADOR'];

        $this->connection->beginTransaction();

        try {
            
            $this->verificaPessoaCadastrada($dadosPessoais);
            $nrParcNegocio = $this->setPessoaParcNegocio($dadosPessoais, $cdOperador);
            [$nrPessoa, $nrPessoaH] = $this->setPessoaAdmissao($dadosPessoais, $nrParcNegocio, $cdOperador);
            $this->setEnderecoAdmissao($enderecoRow, $nrParcNegocio, $cdOperador);
            $this->setContaCorrAdmissao($contaRow, $nrParcNegocio, $cdOperador);
            $this->setComunicaParcAdmissao($comuRow, $nrParcNegocio, $cdOperador);
            $this->setDocumentosAdmissao($documentos, $nrParcNegocio);
            $this->insertOperador($dadosPessoais, $nrParcNegocio);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }

        $result = [
            $nrParcNegocio,
            $nrPessoa,
            $nrPessoaH,
        ];

        return $result;
    }

    /**
     * Verifica se já existe alguma pessoa cadastrada no sistema com o CPF ou PIS informado.
     * Caso positivo, será lançada uma exceção.
     */
    public function verificaPessoaCadastrada(
        array $dadosPessoais
    ) {
        $cpf = (string) $dadosPessoais['NRCPFPESSOA'];

        // Verificação necessária pois o campo não é obrigatório
        $pis = isset($dadosPessoais['NRPISPASEPPES']) ?
            (string) $dadosPessoais['NRPISPASEPPES'] :
            null;

        $this->validaCpfExistePessoa($cpf);

        if ($pis) {
            $this->validaPisExistePessoa($pis);
        }
    }

    /**
     * Verifica se já existe algum pessoa cadastrada com o CPF informado. Caso positivo, retorna true.
     */
    public function validaCpfExistePessoa(
        string $cpf,
        $throwException = true
    ): bool {
        $pessoa = $this->pessoaService->getPessoaByCpf($cpf);

        if ($pessoa) {
            if ($throwException) {
                $nome = (string) $pessoa['NMPESSOA'];
                $message = "Já existe uma pessoa cadastrada com o CPF $cpf! ($nome)";
                throw new ExpectedException($message);
            }

            return true;
        }

        return false;
    }

    /**
     * Verifica se já existe algum pessoa cadastrada com o PIS informado. Caso positivo, retorna true.
     */
    public function validaPisExistePessoa(
        string $pis,
        $throwException = true
    ): bool {
        $pessoa = $this->pessoaService->getPessoaByPis($pis);

        if ($pessoa) {
            if ($throwException) {
                $nome = (string) $pessoa['NMPESSOA'];
                $message = "Já existe uma pessoa cadastrada com o PIS $pis! ($nome)";
                throw new ExpectedException($message);
            }

            return true;
        }

        return false;
    }

    public function setPessoaParcNegocio(
        array $dadosPessoais,
        string $cdOperador
    ): int {
        $parcNegocioRow = [];

        $nome = Utils::fixName($dadosPessoais['NMPESSOA']);

        $parcNegocioRow['NMPRINCIPALPARC'] = $nome;
        $parcNegocioRow['CDOPERINCLUSAO'] = $cdOperador;
        $parcNegocioRow['NMSECUNDARIPARC'] = $nome;

        $parcNegocioRow['DTNASCIFUNDPARC'] =
            $dadosPessoais['DTNASCPESSOA'] ??
            $dadosPessoais['DTNASCIFUNDPARC'] ??
            null;

        $parcNegocioRow['NRINSCRICAOPARC'] = isset($dadosPessoais['NRCPFPESSOA']) ?
            Utils::removePunctuation($dadosPessoais['NRCPFPESSOA']) :
            null;

        $parcNegocioRow['CDTIPOINSCRICAO'] = 'CPF';
        $parcNegocioRow['CDTIPOPARCPRINCIPAL'] = 'PESSOA';
        $parcNegocioRow['IDPESSOAFISICA'] = 'S';
        $parcNegocioRow['IDATIVO'] = 'S';
        $parcNegocioRow['IDINSTITUICAO'] = 'N';
        $parcNegocioRow['IDPARCFUNDIDO'] = 'N';

        $nrParcNegocio = $this->parceiroNegocioService->setParceiroNegocio($parcNegocioRow);

        return $nrParcNegocio;
    }

    /**
     * Cria registros nas tabela GPE_PESSOA e GPE_PESSOAH com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio do resgistro criado
     *  na tabela PARCNEGOCIO.
     *
     * @return int[] Array com os números dos registros criados nas tabelas GPE_PESSOA(0) e GPE_PESSOAH(1).
     */
    public function setPessoaAdmissao(
        array  $dadosPessoais,
        int    $nrParcNegocio,
        string $cdOperador
    ): array {
        $dadosPessoais['CDOPERINCLUSAO'] = $cdOperador;

        $nrPessoa = $this->setPessoaMasterAdmissao($nrParcNegocio, $cdOperador);
        $nrPessoaH = $this->setPessoaHistoryAdmissao($dadosPessoais, $nrPessoa);

        return [$nrPessoa, $nrPessoaH];
    }

    /**
     * Cria um registro na tabela GPE_PESSOA com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio do resgistro criado
     *  na tabela PARCNEGOCIO.
     *
     * @return int Número do registro criado na tabela GPE_PESSOA.
     */
    public function setPessoaMasterAdmissao(
        int $nrParcNegocio,
        string $cdOperador
    ): int {
        $rowPessoa = [
            'NRPARCNEGOCIO' => $nrParcNegocio,
            'CDOPERINCLUSAO' => $cdOperador,
        ];

        $nrPessoa = $this->pessoaService->setPessoa($rowPessoa);

        return $nrPessoa;
    }

    /**
     * Cria um registro na tabela GPE_PESSOAH com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrPessoa Número do registro criado na tabela GPE_PESSOA.
     *
     * @return int Número do registro criado na tabela GPE_PESSOAH.
     */
    public function setPessoaHistoryAdmissao(
        array $dadosPessoais,
        int $nrPessoa
    ): int {
        $dadosPessoais['NRPESSOA'] = $nrPessoa;
        $dadosPessoais['DTMESCOMPETENC'] = $dadosPessoais['DTMESCOMPETENC'] ?? '01/01/3000';

        $nomeCompleto = Utils::fixName($dadosPessoais['NMPESSOA']);
        $nomeCompleto = strtoupper($nomeCompleto);

        $nomesPessoa = PessoaService::separarNomePessoa($nomeCompleto);

        $dadosPessoais['NMPESSOA'] = $nomeCompleto;
        $dadosPessoais['DSPRINOMEPESS']  = $nomesPessoa[0];
        $dadosPessoais['DSNOMEMEIOPESS'] = $nomesPessoa[1];
        $dadosPessoais['DSULTNOMEPESS']  = $nomesPessoa[2];

        $dadosPessoais['NRCPFPESSOA'] = isset($dadosPessoais['NRCPFPESSOA']) ?
            Utils::removePunctuation($dadosPessoais['NRCPFPESSOA']) :
            null;

        $dadosPessoais['NRPISPASEPPES'] = isset($dadosPessoais['NRPISPASEPPES']) ?
            Utils::removePunctuation($dadosPessoais['NRPISPASEPPES']) :
            null;

        $dadosPessoais['CDPAIS']  =
            $dadosPessoais['CDPAISNATURALI'] ??
            $dadosPessoais['CDPAIS'] ??
            null;

        $dadosPessoais['SGESTADO']  =
            $dadosPessoais['SGESTADONATURALI'] ??
            $dadosPessoais['SGESTADO'] ??
            null;

        $dadosPessoais['CDMUNICIPIO']  =
            $dadosPessoais['CDMUNICIPIONATURALI'] ??
            $dadosPessoais['CDMUNICIPIO'] ??
            null;

        if (isset($dadosPessoais['DTCHEGADAPAIS'])) {
            $anoChegada = explode('/', $dadosPessoais['DTCHEGADAPAIS'])[2];
            $dadosPessoais['NRANOCHEGAPAIS'] = $anoChegada;
        }

        $dadosPessoais['DTNASCPESSOA'] =
            $dadosPessoais['DTNASCPESSOA'] ??
            $dadosPessoais['DTNASCIFUNDPARC'] ??
            null;

        $nrPessoaH = $this->pessoaService->setPessoaH($dadosPessoais);

        return $nrPessoaH;
    }

    /**
     * Cria um registro na tabela ENDERECOPARC com os dados enviados.
     *
     * @param array $enderecoRow Row com os dados de endereço da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     *
     * @return int Número do registro criado na tabela ENDERECOPARC.
     */
    public function setEnderecoAdmissao(
        array  $enderecoRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        $canSet = (isset($enderecoRow['CDPAIS']) && isset($enderecoRow['CDLOGRADOURO']));

        if ($canSet) {
            $enderecoRow['NRPARCNEGOCIO'] = $nrParcNegocio;
            $enderecoRow['CDOPERINCLUSAO'] = $cdOperador;
            $this->parceiroNegocioService->setEnderecoParc($enderecoRow);
        }
    }

    /**
     * Cria um registro na tabela CONTCORRPARC com os dados enviados.
     *
     * @param array $contaRow Row com os dados da conta corrente da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setContaCorrAdmissao(
        array  $contaRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        $canSet =
            isset($contaRow['CDBANCO']) &&
            isset($contaRow['CDAGENCIA']) &&
            isset($contaRow['NRTPCONTCORR']) &&
            isset($contaRow['CDCONTCORR']);

        if ($canSet) {
            $contaRow['NRPARCNEGOCIO'] = $nrParcNegocio;
            $contaRow['CDOPERINCLUSAO'] = $cdOperador;
            $this->parceiroNegocioService->setContaCorrente($contaRow);
        }
    }

    /**
     * Cria na tabela COMUNICAPARC os registros de formas de comunicação da pessoa.
     *
     * @param array $comuRow Row com os dados das formas de comunicação da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setComunicaParcAdmissao(
        array  $comuRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        // Telefone principal
        if (isset($comuRow['TELPRINCIPAL']) && $comuRow['TELPRINCIPAL'] !== '') {
            $telefonePrincipal = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '01',
                'CDCOMUNICAPARC' => $comuRow['TELPRINCIPAL'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELPRINCIPAL'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefonePrincipal);
        }

        // Telefone residencial
        if (isset($comuRow['TELRESIDEN']) && $comuRow['TELRESIDEN'] !== '') {
            $telefoneResidencial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '03',
                'CDCOMUNICAPARC' => $comuRow['TELRESIDEN'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELRESIDEN'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefoneResidencial);
        }

        // Telefone comercial
        if (isset($comuRow['TELCOMERC']) && $comuRow['TELCOMERC'] !== '') {
            $telefoneComercial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '04',
                'CDCOMUNICAPARC' => $comuRow['TELCOMERC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELCOMERC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefoneComercial);
        }

        // Celular particular
        if (isset($comuRow['CELPARTIC']) && $comuRow['CELPARTIC'] !== '') {
            $celularParticular = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '02',
                'CDCOMUNICAPARC' => $comuRow['CELPARTIC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFCELPARTIC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($celularParticular);
        }

        // Celular comercial
        if (isset($comuRow['CELCOMERC']) && $comuRow['CELCOMERC'] !== '') {
            $celularComercial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '06',
                'CDCOMUNICAPARC' => $comuRow['CELCOMERC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFCELCOMERC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($celularComercial);
        }

        // FAX
        if (isset($comuRow['FAX']) && $comuRow['FAX'] !== '') {
            $fax = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '10',
                'CDCOMUNICAPARC' => $comuRow['FAX'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFFAX'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($fax);
        }

        // E-mail
        if (isset($comuRow['EMAIL']) && $comuRow['EMAIL'] !== '') {
            $email = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '09',
                'CDCOMUNICAPARC' => $comuRow['EMAIL'],
            ];

            $this->parceiroNegocioService->setComunicaParc($email);
        }
    }

    /**
     * Salva os arquivos enviados do file server e guarda sua informações
     * na tabela GPE_ANEXOS.
     *
     * @param array $documentos Array com os documentos enviados pelo usuário.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setDocumentosAdmissao(
        array $documentos,
        int   $nrParcNegocio
    ) {
        foreach ($documentos as $documento) {
            $documento['NRPARCNEGOCIO'] = $nrParcNegocio;

            $this->documentoService->saveDocumento($documento, false);
        }
    }

    public function getTipoFormacaoParc(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposFormacao = $this->admissaoRepository->getTipoFormacaoParc(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposFormacao;
    }

    public function getCursoAcademico(): array {
        $cursos = $this->admissaoRepository->getCursoAcademico();

        return $cursos;
    }

    public function getInstituicaoEstrutura(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
        ];

        $instituicoes = $this->admissaoRepository->getInstituicaoEstrutura($params);

        return $instituicoes;
    }

    public function getAreaFormacao(): array {
        $area = $this->admissaoRepository->getAreaFormacao();

        return $area;
    }

    public function getBeneficio(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $params = [
            'NRORG'             => $nrOrg,
            'NRORG_PADRAO'      => $nrOrgPadrao,
            'DTMESCOMPETENC'    => $dtMesCompetencia,
            'IDATIVO'           => 'S',
        ];

        $beneficios = $this->admissaoRepository->getBeneficio($params);

        return $beneficios;
    }

    public function getTipoVinculo(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $params = [
            'P_NRORG'             => $nrOrg,
            'P_NRORG_PADRAO'      => $nrOrgPadrao,
            'P_DTMESCOMPETENC'    => $dtMesCompetencia,
        ];

        $tipos = $this->admissaoRepository->getTipoVinculo($params);

        return $tipos;
    }

    public function getTipoAdmissao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->admissaoRepository->getTipoAdmissao(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getVinculoEmpregaticio(): array {
        $vinculosEmp = $this->admissaoRepository->getVinculoEmpregaticio();

        return $vinculosEmp;
    }

    public function getTipoOcupacao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->admissaoRepository->getTipoOcupacao(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getOcupacao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $params = [
            'P_NRORG'             => $nrOrg,
            'P_NRORG_PADRAO'      => $nrOrgPadrao,
            'P_DTMESCOMPETENC'    => $dtMesCompetencia,
        ];

        $ocupacoes = $this->admissaoRepository->getOcupacao($params);

        return $ocupacoes;
    }

    public function getEstruturaGerLeg(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);
        $nrTipoEstrutura = $paramOrg['NRTPESTRUTGEREN'];
        $nrEstrutLegal = $paramOrg['NRTPESTRUTLEGAL'];

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $params = [
            'NRORG'           => $nrOrg,
            'NRORG_PADRAO'    => $nrOrgPadrao,
            'DTMESCOMPETENC'  => $dtMesCompetencia,
            'NRTIPOESTRUTURA'   => $nrTipoEstrutura,
            'NRTPESTRUTLEGAL'   => $$nrEstrutLegal,
        ];

        $estruturas = $this->admissaoRepository->getEstruturaGerLeg($params);

        return $estruturas;
    }

    public function getEstruturaLegal(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $params = [
            'NRORG'           => $nrOrg,
            'NRORG_PADRAO'    => $nrOrgPadrao,
            'DTMESCOMPETENC'  => $dtMesCompetencia,
        ];

        $estruturas = $this->admissaoRepository->getEstruturaLegal($params);

        return $estruturas;
    }

    public function getTipoSalario(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->admissaoRepository->getTipoSalario(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function validateRequiredFieldsSimplificada(
        array $row
    ): void {
        $naoExisteRegistroPessoa = empty($this->dadosPessoaAdmissao);
        $validations = [];

        if ($naoExisteRegistroPessoa) {
            $dadosPessoais = (array) $row['dadosPessoais'];
            $validations['dadosPessoais'] = Utils::checkRequiredFields($dadosPessoais, self::ADMISSAO_SIMPLIFICADA_DADOS_PESSOAIS_REQUIRED_FIELDS, false);
        }

        $dadosVinculo = (array) $row['dadosVinculo'];
        $validations['dadosVinculo'] = Utils::checkRequiredFields($dadosVinculo, self::ADMISSAO_SIMPLIFICADA_DADOS_VINCULO_REQUIRED_FIELDS, false);

        $this->throwExpectedExceptionIfNotValid($validations);
    }

    public function validateRequiredFieldsCompleta(
        array $row
    ): void {
        $naoExisteRegistroPessoa = empty($this->dadosPessoaAdmissao);
        $validations = [];

        $dadosVinculo = (array) $row['dadosVinculo'];
        $validations['dadosVinculo'] = Utils::checkRequiredFields($dadosVinculo, self::ADMISSAO_COMPLETA_DADOS_VINCULO_REQUIRED_FIELDS, false);

        if ($naoExisteRegistroPessoa) {
            $dadosPessoais = (array) $row['dadosPessoais'];
            $validations['dadosPessoais'] = Utils::checkRequiredFields($dadosPessoais, self::ADMISSAO_COMPLETA_DADOS_PESSOAIS_REQUIRED_FIELDS, false);

            $dadosFiliacao = $dadosPessoais['filiacao'];
            $validations['filiacao'] = Utils::checkRequiredFields($dadosFiliacao, self::ADMISSAO_COMPLETA_FILIACAO_REQUIRED_FIELDS, false);

            $endereco = (array) $row['endereco'];
            $validations['endereco'] = Utils::checkRequiredFields($endereco, self::ADMISSAO_COMPLETA_ENDERECO_REQUIRED_FIELDS, false);
        }

        $this->throwExpectedExceptionIfNotValid($validations);
    }

    public function throwExpectedExceptionIfNotValid(
        array $validations
    ): void {
        $fieldsNotFilled = [];
        foreach ($validations as $validation) {
            if (empty($validation['isValid'])) {
                $fieldsNotFilled = array_merge($fieldsNotFilled, $validation['fields']);
            }
        }

        if (empty($fieldsNotFilled)) {
            return;
        }

        throw new ExpectedException(Utils::FIELDS_NOT_FILLED_BASE_MESSAGE . (implode(', ', $fieldsNotFilled)));
    }

    public function setAdmissao(
        array $row
    ): array {
        $cdOperador = (string) $row['CDOPERADOR'];
        $dadosVinculo = (array) $row['dadosVinculo'];
        $dadosPessoais = (array) $row['dadosPessoais'];
        $dependentes = (array) ($row['dependentes'] ?? []);
        $vt = (array) ($row['vt'] ?? []);
        $beneficios = (array) ($row['beneficios'] ?? []);
        $formacao = (array) ($row['formacao'] ?? []);

        $nrTipoVinculo = (int) ($dadosVinculo['NRTIPOVINCULOM'] ?? null);
        $dtAdmissao = (string) ($dadosVinculo['DTADMISSAOVINC'] ?? null);
        $nmPessoa = (string) ($dadosPessoais['NMPESSOA'] ?? null);
        $solicitacao = (array) ($dadosPessoais['solicitacao'] ?? []);

        $nrOrg = (int) $this->environment->getNrOrg();

        $processo = $this->verificaLimiteVagasProcesso($nrOrg, $solicitacao);

        $dtMesCompetencia = DateUtil::getDtCompetencia($dtAdmissao);

        $row['dadosPessoais']['DTMESCOMPETENC'] = $dtMesCompetencia;


        try {
            $this->connection->beginTransaction();

            // PARCNEGOCIO, GPE_PESSOA, GPE_PESSOAH, ENDERECOPARC
            // CONTCORRPARC, COMUNICAPARC, GPE_ANEXOS
            if($dadosVinculo['IDMULTVINC'] == 'UNICO' && empty($this->dadosPessoaAdmissao)){
                [$nrParcNegocio, $nrPessoa] = $this->setDadosPessoaisAdmissao($row);
            } else if (!empty($this->dadosPessoaAdmissao)) {
                $nrParcNegocio = $this->dadosPessoaAdmissao['NRPARCNEGOCIO'];
                $nrPessoa = $this->dadosPessoaAdmissao['NRPESSOA'];
            } else {
                throw new ExpectedException(
                    'Registro de Pessoa não encontrado. Certifique-se que o CPF já esta cadastrado ou que as informações de Vínculo estão como "Unico".'
                );
            }

            // Validação das movimentações adicionais
            $message = $this->validaEstruturasAdicionais($dadosVinculo);
            
            if(!empty($message)) {
                throw new ExpectedException($message);
            }
            
            // GPE_VINCULOM, GPE_VINCULOH
            [$nrVinculo, $nrVinculoH] = $this->setVinculoAdmissao(
                $dadosVinculo, $nrPessoa, $cdOperador, $nmPessoa
            );

            // DEPENDENTES (PARCNEGOCIO, GPE_PESSOA, GPE_PESSOAH, RELACIONAPARC, FPA_DEPVINCULO)
            $dependentesData = $this->setDependentesAdmissao(
                $dependentes, $cdOperador, $nrParcNegocio,
                $nrVinculo, $dtMesCompetencia
            );

            // VAB_LINHAVINC
            $this->setVTAdmissao($vt, $nrVinculo, $cdOperador);

            // VAB_BENEFICIARIO
            $this->setBeneficiosAdmissao(
                $beneficios, $nrVinculo, $nrPessoa,
                $cdOperador, $dependentesData
            );

            // PTO_PROGOCORRENCIA
            $this->setProgOcorrenciaAdmissao(
                $nrVinculo, $nrTipoVinculo,
                $cdOperador, $dtAdmissao
            );

            // PTO_ALTEBANCOHORAS
            $this->setBancoHorasAdmissao(
                $nrVinculo, $cdOperador, $nrTipoVinculo,
                $dtAdmissao
            );

            // GPE_MOVIMENTACAO
            $this->setMovimentacaoAdmissao(
                $nrVinculo, $cdOperador, $dadosVinculo,
                $dtAdmissao
            );

            // GPE_ALTESITUFUNC
            $this->setAlteSituacaoFuncionalAdmissao(
                $nrVinculo, $cdOperador, $dtAdmissao
            );

            // GPE_ALTEESCALA
            $this->setAlteEscalaAdmissao(
                $nrVinculo, $cdOperador, $dadosVinculo
            );

            // GPE_ALTESALARIO
            $this->setAlteSalarioAdmissao(
                $nrVinculo, $cdOperador, $dadosVinculo,
                $dtAdmissao
            );

            // GPE_ALTEOCUPACAO
            $this->setAlteOcupacaoAdmissao(
                $nrVinculo, $cdOperador, $dadosVinculo,
                $dtAdmissao
            );

            // DES_FORMACAO
            $this->setFormacaoParcAdmissa(
                $nrParcNegocio, $cdOperador, $formacao
            );

            if($processo && isset($processo['NRPROCESSO'])) {
               $this->incrementaVagasAdmitidasProcesso($nrOrg, $processo['NRPROCESSO']); 
            }

            $success = $this->connection->commit();

            $response = [
                "NRVINCULOM" => $nrVinculo,
                "SUCCESS" => $success
            ];

            return $response;
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function setVinculoAdmissao(
        array $dadosVinculo,
        int $nrPessoa,
        string $cdOperador,
        string $nmPessoa
    ): array {
        try {
            $dadosVinculo['CDOPERINCLUSAO'] = $cdOperador;
            $dadosVinculo['NRPESSOA'] = $nrPessoa;
            $dadosVinculo['NMVINCULOM'] = Utils::fixName($nmPessoa);

            $nrVinculo = $this->setVinculoMasterAdmissao($dadosVinculo);
            $nrVinculoH = $this->setVinculoHistoryAdmissao($dadosVinculo, $nrVinculo);

            return [$nrVinculo, $nrVinculoH];
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção dos dados do vínculo: ' . $e->getMessage());
        }
    }

    public function setVinculoMasterAdmissao(
        array $dadosVinculo
    ): int {
        $dadosVinculo['DTOPCAOFGTS'] = $dadosVinculo['DTADMISSAOVINC'];
        $dadosVinculo['DTPRIMADMISSAO'] = $dadosVinculo['DTADMISSAOVINC'];

        $nrVinculo = $this->vinculoService->setVinculo($dadosVinculo);

        return $nrVinculo;
    }

    public function setVinculoHistoryAdmissao(
        array $dadosVinculo,
        int $nrVinculo
    ): int {
        $dadosVinculo['CONFIDENCIAL'] = 'N';
        $dadosVinculo['NRSITUFUNCM'] = 1;
        $dadosVinculo['NRTPREGIMEPREV'] = 1;
        $dadosVinculo['IDMULTVINC'] = $dadosVinculo['IDMULTVINC'] ?? 'UNICO';
        $dadosVinculo['DTMESCOMPETENC'] = '01/01/3000';
        $dadosVinculo['NRVINCULOM'] = $nrVinculo;
        $dadosVinculo['NRCARGO'] = $dadosVinculo['NROCUPACAOM'] ?? null;
        $dadosVinculo['NRFUNCAO'] = $dadosVinculo['NRCARGO'];

        $nrVinculoH = $this->vinculoService->setVinculoH($dadosVinculo);

        return $nrVinculoH;
    }

    public function setVTAdmissao(
        array $valesTransporte,
        int $nrVinculo,
        string $cdOperador
    ): array {
        try {
            $nrsVts = [];

            foreach ($valesTransporte as $vt) {
                $vt['NRVINCULOM'] = $nrVinculo;
                $vt['CDOPERINCLUSAO'] = $cdOperador;

                $nrVT = $this->valeTransporteService->setValeTransporte($vt);

                array_push($nrsVts, $nrVT);
            }

            return $nrsVts;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção dos Vales-Transporte do vínculo: ' . $e->getMessage());
        }
    }

    public function setDependentesAdmissao(
        array $dependentes,
        string $cdOperador,
        int $nrParcNegocio,
        $nrVinculo,
        string $dtMesCompetencia
    ): array {
        try {
            $dependentesData = [];

            foreach ($dependentes as $dep) {
                $dep['DTMESCOMPETENC'] = $dtMesCompetencia;
                $cpf = $dep['NRCPFPESSOA'] ?? null;
                $pessoaDep = null;

                if ($cpf) {
                    $pessoaDep = $this->pessoaService->getPessoaByCpf($cpf);

                    if ($pessoaDep) {
                        $nrParcDep = $pessoaDep['NRPARCNEGOCIO'];
                        $nrPessoaDep = $pessoaDep['NRPESSOA'];
                        $nrPessoaHDep = $pessoaDep['NRPESSOAH'];
                    }
                }

                if ($pessoaDep) {
                    $nrParcDep = $pessoaDep['NRPARCNEGOCIO'];
                    $nrPessoaDep = $pessoaDep['NRPESSOA'];
                    $nrPessoaHDep = $pessoaDep['NRPESSOAH'];
                } else {
                    $nrParcDep = $this->setPessoaParcNegocio($dep, $cdOperador);
                    [$nrPessoaDep, $nrPessoaHDep] = $this->setPessoaAdmissao($dep, $nrParcDep, $cdOperador);
                }

                $dep['NRPARCNEGOCIO'] = $nrParcNegocio;
                $dep['NRPARCNEGRELAC'] = $nrParcDep;
                $dep['CDOPERINCLUSAO'] = $cdOperador;

                $nrRelacionaParc = $this->dependenteService->setRelacionaParc($dep);

                $nrDepVinculo = null;

                if (isset($dep['NRTIPODEPENDE'])) {
                    $dep['NRVINCULOM'] = $nrVinculo;
                    $dep['NRPESSOA'] = $nrPessoaDep;

                    $nrDepVinculo = $this->dependenteService->setDepVinculo($dep);
                }

                $depData = [$nrPessoaDep, $nrPessoaHDep, $nrRelacionaParc, $nrDepVinculo];
                array_push($dependentesData, $depData);
            }

            return $dependentesData;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção dos dependentes: ' . $e->getMessage());
        }
    }

    public function setBeneficiosAdmissao(
        array $beneficios,
        int $nrVinculo,
        int $nrPessoa,
        string $cdOperador,
        array $dependentes
    ) {
        try {
            /**
             * @var array
             *
             *  [
             *      "NRBENEFICIOM" => (int),
             *      "NRBENEFICIARIO" => (int),
             *      "dependentes" => [
             *          'NRPESSOA' => (int),
             *          'NRBENEFICIARIO' => (int)
             *      ][]
             *  ][]
             */
            $beneficiosTitularDependentes = [];

            foreach ($beneficios as $beneficio) {
                /**
                 * @var array
                 *
                 *  [
                 *      'NRPESSOA' => (int),
                 *      'NRBENEFICIARIO' => (int)
                 *  ][]
                 */
                $beneficiariosDeps = [];

                $beneficio['NRVINCULOM'] = $nrVinculo;
                $beneficio['CDOPERINCLUSAO'] = $cdOperador;
                $beneficio['NRPESSOA'] = $nrPessoa;
                $beneficio['IDTPBENEFICIARIO'] = 'T';

                $nrBeneficiario = $this->beneficioService->setBeneficiario($beneficio);

                $nrBeneficio = (int) $beneficio['NRBENEFICIOM'];
                $registroBeneficio = $this->beneficioService->getBeneficio($nrBeneficio);

                if (isset($registroBeneficio['NREVENTOMENSADEP'])) {
                    foreach ($dependentes as $dep) {
                        [$nrPessoaDep, $nrPessoaHDep, $nrRelacionaParc, $nrDepVinculo] = $dep;

                        if ($nrDepVinculo !== null) {
                            $beneficio['NRPESSOA'] = $nrPessoaDep;
                            $beneficio['IDTPBENEFICIARIO'] = 'D';

                            $nrBeneficiarioDep = $this->beneficioService->setBeneficiario($beneficio);

                            $beneficiarioDep = [
                                'NRPESSOA'          => $nrPessoaDep,
                                'NRBENEFICIARIO'    => $nrBeneficiarioDep,
                            ];

                            array_push($beneficiariosDeps, $beneficiarioDep);
                        }
                    }
                }

                $beneficioTitularDependente = [
                    'NRBENEFICIO' => $nrBeneficio,
                    'NRBENEFICIARIO' => $nrBeneficiario,
                    'dependentes' => $beneficiariosDeps,
                ];

                array_push($beneficiosTitularDependentes, $beneficioTitularDependente);
            }

            return $beneficiosTitularDependentes;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção dos benefícios do vínculo: ' . $e->getMessage());
        }
    }

    public function setProgOcorrenciaAdmissao(
        int $nrVinculo,
        int $nrTipoVinculo,
        string $cdOperador,
        string $dtAdmissao
    ) {
        try {
            $nrOrg = (int) $this->environment->getNrOrg();
            $ocorrenciasAdmissao = $this->ocorrenciaService->getOcorrenciasAdmissao($nrTipoVinculo);
            
            foreach ($ocorrenciasAdmissao as $ocorAdmissao) {
                $rowOcorrencia = [
                    'NRORG' => $nrOrg,
                    'NRVINCULOM' => $nrVinculo,
                    'NRTIPOPROGOCORRENCIA' => $ocorAdmissao['NRTIPOPROGOCORRENCIA'],
                    'DTINICIOPROGOCOR' => $dtAdmissao,
                    'IDPROGOCORCONFIRMADA' => 'S',
                    'DSOBSERVACAO' => '',
                    'CDOPERINCLUSAO' => $cdOperador,
                    'NRORGINCLUSAO' => $nrOrg,
                    'DTINCLUSAO' => date('d/m/Y'),
                    'CDOPERULTATU' => $cdOperador,
                    'NRORGULTATU' => $nrOrg,
                    'DTULTATU' => date('d/m/Y'),
                ];
    
                $this->ocorrenciaRepository->newOcorrenciaFreq($rowOcorrencia);
            }
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção das ocorrências: ' . $e->getMessage());
        }
    }

    public function setBancoHorasAdmissao(
        int $nrVinculo,
        string $cdOperador,
        int $nrTipoVinculo,
        string $dtAdmissao
    ) {
        try {
            $bancoHorasAdmi = $this->bancoHorasService->getBancoHorasAdmi($nrTipoVinculo);

            $nrsAlteBancoHoras = [];

            foreach ($bancoHorasAdmi as $banco) {
                $rowBancoHoras = [
                    'NRVINCULOM' => $nrVinculo,
                    'NRBANCOHORAS' => $banco['NRBANCOHORAS'],
                    'DTINICIOALTEBHORAS' => $dtAdmissao,
                    'CDOPERINCLUSAO' => $cdOperador,
                ];

                $nrAlteBancoHoras = $this->bancoHorasService->setAlteBancoHoras($rowBancoHoras);

                array_push($nrsAlteBancoHoras, $nrAlteBancoHoras);
            }

            return $nrsAlteBancoHoras;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção do banco de horas do vínculo: ' . $e->getMessage());
        }
    }

    public function setMovimentacaoAdmissao(
        int $nrVinculo,
        string $cdOperador,
        array $dadosVinculo,
        string $dtAdmissao
    ): array {
        try {
            $nrOrg = (int) $this->environment->getNrOrg();

            $nrMovSind = null;
            $nrMovLegal = null;
            $nrMovGeren = null;

            $row = [
                'NRVINCULOM'        => $nrVinculo,
                'DTINIMOVIMENT'     => $dtAdmissao,
                'CDOPERINCLUSAO'    => $cdOperador,
                'NRTPMOVTRANSFM'    => $this->getNrTpMovTransfer(),
            ];

            if (isset($dadosVinculo['NRESTRUTSIND'])) {
                $row['NRTIPOTRANSFER'] = 1;
                $row['NRESTRUTURAM'] = (int) $dadosVinculo['NRESTRUTSIND'];
                $row['NRTIPOESTRUTURA'] = 10;

                $nrMovSind = $this->movimentacaoService->setMovimentacao($row);
            }

            if (isset($dadosVinculo['NRESTRUTGEREN']) || isset($dadosVinculo['NRESTRUTLEGAL'])) {
                $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);

                if (isset($dadosVinculo['NRESTRUTGEREN'])) {
                    $row['NRTIPOTRANSFER'] = 2;
                    $row['NRESTRUTURAM'] = (int) $dadosVinculo['NRESTRUTGEREN'];
                    $row['NRTIPOESTRUTURA'] = (int) $paramOrg['NRTPESTRUTGEREN'];

                    $nrMovGeren = $this->movimentacaoService->setMovimentacao($row);
                }

                if (isset($dadosVinculo['NRESTRUTLEGAL'])) {
                    $row['NRTIPOTRANSFER'] = 1;
                    $row['NRESTRUTURAM'] = (int) $dadosVinculo['NRESTRUTLEGAL'];
                    $row['NRTIPOESTRUTURA'] = (int) $paramOrg['NRTPESTRUTLEGAL'];

                    $nrMovLegal = $this->movimentacaoService->setMovimentacao($row);
                }
            }

            if(isset($dadosVinculo['NRESTRUTADICIONAL'])) {
                foreach($dadosVinculo['NRESTRUTADICIONAL'] as $nrEstrutAdicional) {
                    if($dadosVinculo['NRTIPOTRANSFERADICIONAL']) {
                        Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
                            'nrorg' => $nrOrg, 'nrestruturam' => $nrEstrutAdicional
                        ]);

                        $params = array(
                            'NRORG'             => $nrOrg,
                            'NRVINCULOM'        => $nrVinculo,
                            'DTINIMOVIMENT'     => $dtAdmissao,
                            'CDOPERINCLUSAO'    => $cdOperador,
                            'NRTPMOVTRANSFM'    => $this->getNrTpMovTransfer(),
                            'NRESTRUTURAM'      => (int) $nrEstrutAdicional,
                            'NRTIPOESTRUTURA'   => (int) $paramOrg['NRTPESTRUTGEREN'],
                            'NRTIPOTRANSFER'    => $dadosVinculo['NRTIPOTRANSFERADICIONAL'],
                        );

                        $this->movimentacaoService->setMovimentacao($params);
                    }
                }
            }

            return [$nrMovSind, $nrMovLegal, $nrMovGeren];
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção das transferências/movimentaçôes do vínculo: ' . $e->getMessage());
        }
    }

    public function setAlteSituacaoFuncionalAdmissao(
        int $nrVinculo,
        string $cdOperador,
        string $dtAdmissao
    ): int {
        try {
            $row = [
                'NRVINCULOM' => $nrVinculo,
                'CDOPERINCLUSAO' => $cdOperador,
                'NRSITUFUNCM' => 1,
                'DTINISITUFUNC' => $dtAdmissao,
            ];

            $nrAlteSituacaoFuncional = $this->situacaoFuncionalService->setAlteSituacaoFuncional($row);

            return $nrAlteSituacaoFuncional;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção da situação funcional vínculo: ' . $e->getMessage());
        }
    }

    public function setAlteEscalaAdmissao(
        int $nrVinculo,
        string $cdOperador,
        array $dadosVinculo
    ): int {
        try {
            $dadosVinculo['NRVINCULOM'] = $nrVinculo;
            $dadosVinculo['CDOPERINCLUSAO'] = $cdOperador;

            if (empty($dadosVinculo['NRESCALATRABM'])) {
                return 0;
            }

            $nrAlteEscala = $this->situacaoFuncionalService->setAlteEscala($dadosVinculo);

            return $nrAlteEscala;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção da escala de trabalho do vínculo: ' . $e->getMessage());
        }
    }

    public function setAlteSalarioAdmissao(
        int $nrVinculo,
        string $cdOperador,
        array $dadosVinculo,
        string $dtAdmissao
    ): int {
        try {
            $dadosVinculo['NRVINCULOM'] = $nrVinculo;
            $dadosVinculo['CDOPERINCLUSAO'] = $cdOperador;
            $dadosVinculo['NRTPMODALIDSAL'] = 1;
            $dadosVinculo['DTALTESALARIO'] = $dtAdmissao;
            $dadosVinculo['NRTIPOALTERA'] = 1;

            try {
                $dadosVinculo['VRSALARIO'] = Utils::currencyStringToNumber(
                    $dadosVinculo['VRSALARIO']
                );
            } catch (\Exception $e) {
                $message = 'Formato de salário inválido! Utilize apenas números e vírgula.';
                throw new ExpectedException($message);
            }

            $nrAlteSalario = $this->alteSalarioService->setAlteSalario($dadosVinculo);

            return $nrAlteSalario;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção do salário do vínculo: ' . $e->getMessage());
        }
    }

    public function setAlteOcupacaoAdmissao(
        int $nrVinculo,
        string $cdOperador,
        array $dadosVinculo,
        string $dtAdmissao
    ): int {
        try {
            $dadosVinculo['NRVINCULOM'] = $nrVinculo;
            $dadosVinculo['CDOPERINCLUSAO'] = $cdOperador;
            $dadosVinculo['NRTPMODALIDSAL'] = 1;

            $nrAlteOcupacao = $this->ocupacaoService->setAlteOcupacao($dadosVinculo);

            return $nrAlteOcupacao;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção da ocupação do vínculo: ' . $e->getMessage());
        }
    }

    public function setFormacaoParcAdmissa(
        int $nrParcNegocio,
        string $cdOperador,
        array $formacao
    ): array {
        try {
            $nrsFormacoes = [];

            foreach ($formacao as $form) {
                $form['NRPARCNEGOCIO'] = $nrParcNegocio;
                $form['CDOPERINCLUSAO'] = $cdOperador;

                $nrFormacao = $this->formacaoService->setFormacaoParc($form);

                array_push($nrsFormacoes, $nrFormacao);
            }

            return $nrsFormacoes;
        }
        catch (\Exception $e) {
            throw new ExpectedException('Não foi possível realizar a inserção da formação do vínculo: ' . $e->getMessage());
        }
    }

    public function getMotivosAdmissao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $motivosAdmissao = $this->admissaoRepository->getMotivosAdmissao(
            $nrOrg, $nrOrgPadrao
        );

        return $motivosAdmissao;
    }

    public function getTodosVinculosAdmissao(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $vinculos = $this->admissaoRepository->getTodosVinculosAdmissao(
            $nrOrg
        );

        return $vinculos;
    }

    public function insertOperador(
        array $dadosPessoais,
        int $nrParcNegocio
    ): string {
        $criarOperador = empty($dadosPessoais['INSERE_OPERADOR']) ?
            false :
            $dadosPessoais['INSERE_OPERADOR'] === 'S';
        
        if (empty($dadosPessoais['EMAIL'])) {
            $criarOperador = false;
        }

        if (!$criarOperador) {
            return '';
        }

        $nrOrg = (int) $this->environment->getNrOrg();

        $nmPessoa = $dadosPessoais['NMPESSOA'];
        $email = $dadosPessoais['EMAIL'];
        $senha = OperadorService::generatePassword($nmPessoa);
        $senhaCriptografada = OperadorService::encryptPassword($senha);

        $rowOperador = [
            'NRORG' => $nrOrg,
            'NMOPERADOR' => $nmPessoa,
            'NRPARCNEGOCIO' => $nrParcNegocio,
            'DSEMAILOPER' => $email,
            'DTNASCIMENTO' => $dadosPessoais['DTNASCPESSOA'],
            'CDSENHAOPERWEB' => $senhaCriptografada,
            'IDOPERATIVO' => 'A',
            'IDSTATUSENHA' => 'A',
            'IDATUOPERADOR' => 'S',
            'IDCADPRAPROVORE' => 'N',
            'CDFUNC' => '003',
            'IDCADPRPROCEORE' => 'N',
            'IDLIGEXTOPER' => 'N',
            'IDSENHVARDIA' => 'N',
            'IDOPERMASTER' => 'N',
        ];

        $cdOperador = $this->operadorService->insert($rowOperador, false);

        $this->emailService->sendEmailNovoOperador(
            $nmPessoa,
            $email,
            $senha
        );

        return $cdOperador;
    }

    /**
     * Define as informacoes da pessoa que esta sendo admitida caso exista
     * @param array $row
     * @return array
     */
    public function setPessoaExistente(array $row): array {
        $cpf = Utils::sanitizeCPF($row['dadosPessoais']['NRCPFPESSOA']);
        $infoPessoaCPF = $this->buscarPessoaCPF($cpf);
        return $this->dadosPessoaAdmissao = $infoPessoaCPF['pessoa'] ?? [];
    }

    public function buscarPessoaCPF(string $nrcpfpessoa): array {
        $nrorg = $this->environment->getNrorg();
        $nrorg_padrao = $this->environment->getNrOrgPadrao();
        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        
        $pessoa = Utils::getEntityManager()->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtmescompetenc, $nrcpfpessoa);
        $pessoa = $pessoa[0] ?? null;

        if (empty($pessoa)) {
            return [];
        }
        
        $pessoa['DEPENDENTES'] = Utils::getEntityManager()->getRepository(Repositories::GPE_PESSOA)->retornaRelacionamentosPorParcnegocio($nrorg, $nrorg_padrao, $pessoa['NRPARCNEGOCIO'], $dtmescompetenc);
        
        $dadosFaltantesPessoa = [];
        $dadosFaltantesPessoaEndereco = [];
        
        if(empty($pessoa['NRRGPESSOA'])){
            $dadosFaltantesPessoa[] = "RG<br>";
        }
        
        if(empty($pessoa['NRPISPASEPPES'])){
            $dadosFaltantesPessoa[] = "PIS<br>";
        }
        
        if(empty($pessoa['NRCTPSPESSOA'])){
            $dadosFaltantesPessoa[] = "CTPS<br>";
        }
        
        if(empty($pessoa['NMBAIRROENDERECO'])){
            $dadosFaltantesPessoaEndereco[] = "BAIRRO<br>";
        }
        
        if(empty($pessoa['SGESTADO'])){
            $dadosFaltantesPessoaEndereco[] = "ESTADO<br>";
        }
        
        if(empty($pessoa['CDLOGRADOURO'])){
            $dadosFaltantesPessoaEndereco[] = "LOGRADOURO<br>";
        }
        
        if(empty($pessoa['NRCEPENDERECO'])){
            $dadosFaltantesPessoaEndereco[] = "CEP<br>";
        }
        
        if(empty($pessoa['CDMUNICIPIO'])){
            $dadosFaltantesPessoaEndereco[] = "MUNICÍPIO<br>";
        }
        
        if(empty($pessoa['DSENDERECO'])){
            $dadosFaltantesPessoaEndereco[] = "ENDEREÇO<br>";
        }
        
        if(empty($pessoa['CDPAIS'])){
            $dadosFaltantesPessoaEndereco[] = "PAIS<br>";
        }

        if(!empty($dadosFaltantesPessoa) || !empty($dadosFaltantesPessoaEndereco)){
            if(!empty($dadosFaltantesPessoa)){
                $dadosFaltantesImplodePessoa = implode(' ',$dadosFaltantesPessoa);
                $dadosFaltantesImplodePessoa = "<br><br> Dados pessoais necessários:<br> $dadosFaltantesImplodePessoa";
            }
            
            if(!empty($dadosFaltantesPessoaEndereco)){
                $dadosFaltantesImplodePessoaEndereco = implode(' ',$dadosFaltantesPessoaEndereco);
                $dadosFaltantesImplodePessoaEndereco = "<br> Dados de endereço necessários:<br> $dadosFaltantesImplodePessoaEndereco";
            }
        
            $msg = 'Existem dados obrigatórios não preenchidos para a pessoa: '.$pessoa['NMPESSOA'].' '.($dadosFaltantesImplodePessoa ?? '').' '.($dadosFaltantesImplodePessoaEndereco ?? '').'<br> Favor fazer o preenchimento antes de prosseguir com a admissão.';
        }
        
        return [
            'pessoa' => $pessoa,
            'msg' => $msg ?? null,
        ];
    }

    public function validaEstruturasAdicionais($row): ?string {
        if(empty($row['NRESTRUTADICIONAL'])){
            return null;
        }
        $nrorg = $this->environment->getNrorg();

        $estruturaGerencial = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
            'nrorg' => $nrorg, 'nrestruturam' => $row['NRESTRUTGEREN']
        ]);

        $estruturaLegal = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
            'nrorg' => $nrorg, 'nrestruturam' => $row['NRESTRUTLEGAL']
        ]);

        $estruturaSindical = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
            'nrorg' => $nrorg, 'nrestruturam' => $row['NRESTRUTSIND']
        ]);

        if((empty($row['NRESTRUTGEREN']) && empty($row['NRESTRUTLEGAL'])) || (empty($row['NRESTRUTGEREN']) || empty($row['NRESTRUTLEGAL']))) {
            return "Preencha primeiro as movimentações padrões, Legal e Gerencial para depois realizar o preenchimento de movimentação Adicional!!";
        }

        $nrtipoestruturaLegal = $estruturaLegal->getNrtipoestrutura();
        $nrtipoestruturaGeren = $estruturaGerencial->getNrtipoestrutura();
        $nrtipoestruturaSindi = $estruturaSindical->getNrtipoestrutura();

        // Verifica tipo de estrutura repetido no array de estruturas adicionais
        if($this->verificaTipoEstruturaDuplicado($row['NRESTRUTADICIONAL'], $nrorg)){
            return 'Existe "Tipo de Estrutura" repetido em Movimentação Adicional, favor verificar.';
        }
        
        foreach($row['NRESTRUTADICIONAL'] as $index => $nrEstrutAdicional) {
            $estruturaAdicional = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
                'nrorg' => $nrorg, 'nrestruturam' => $nrEstrutAdicional
            ]);

            $nrtipoestruturaAdicional = $estruturaAdicional->getNrtipoestrutura();

            if($nrtipoestruturaLegal == $nrtipoestruturaAdicional) {
                return Messages::MOV_ADI_IGUAL_LEGAL;
            }
            
            if($nrtipoestruturaGeren == $nrtipoestruturaAdicional) {
                return Messages::MOV_ADI_IGUAL_GEREN;
            }
            
            if($nrtipoestruturaSindi == $nrtipoestruturaAdicional){
                return Messages::MOV_ADI_IGUAL_SINDI;
            }
        }

        return null;
    }

    public function verificaTipoEstruturaDuplicado($estruturas, $nrorg) {

        $tiposEstruturas = [];
        
        foreach($estruturas as $estrutura) {
            $estruturaAdicional = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy([
                'nrorg' => $nrorg, 'nrestruturam' => $estrutura
            ]);
            
            $tiposEstruturas[] = $estruturaAdicional->getNrtipoestrutura();

        } 
        
        $contagem = array_count_values($tiposEstruturas);
        
        foreach ($contagem as $quantidade) {
            
            if ($quantidade > 1) {
                return true; // Tem uma duplicata
            }
        }
        
        return false; // Não há duplicatas
    }

    /*
     *  Funções criadas para passar por padrão pro campo Tipo Movimentação Transferência:
     *  1 - Admissão no primeiro emprego - caso a pessoa não exista na base de dados
     *  2 - Admissão com emprego anterior - caso a pessoa exista na base de dados
     */
    public function isDadosPessoaEmpty(): bool {
        return empty($this->dadosPessoaAdmissao);
    }

    public function getNrTpMovTransfer(): int {
        if($this->isDadosPessoaEmpty()) {
            return 1;
        }

        return 2;
    }


    public function getLoteVinculos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = $this->environment->getCdOperador();
        $dtMesCompetencia = $this->environment->getDtmescomptencDiaCorrente();

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_CDOPERADOR'      => $cdOperador,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia
        ];

        $loteVinculos = $this->admissaoRepository->getLoteVinculos(
            $params
        );

        return $loteVinculos;
    }

    public function verificaLimiteVagasProcesso($nrOrg, $solicitacao){
        $isSolicitacao = $solicitacao['isSolicitacao'] ?? false;
        $nrProcesso = $solicitacao['NRPROCESSO'] ?? null;
        
        if ($isSolicitacao && $nrProcesso) {
            $processo = $this->admissaoRepository->getProcesso($nrOrg, $nrProcesso);
        
            if ($processo && $processo['NRVAGAS'] <= $processo['NRVAGASADMITIDAS']) {
                throw new ExpectedException(
                    'Essa Solicitação/Processo já atingiu o número máximo de vagas admitidas.'
                );
            }

            return $processo;
        }
        return false;
    }

    public function incrementaVagasAdmitidasProcesso($nrOrg, $nrProcesso){
        if ($nrProcesso) {
            $this->admissaoRepository->incrementaVagasAdmitidasProcesso($nrOrg, $nrProcesso);
        }
    }
}