<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;
use Ems\Repository\{  MarcacaoRepository,  VinculoRepository, ApuracaoAdcNoturnoRepository };
use Ems\Util\Time;
use Teknisa\Model\Gestaopessoas\Entities\GpeMarcacaoponto;
use Folha\Entities\FpaLotelancevento;
use Ems\Util\Utils;
use Util\Repositories;
use Ems\Services\OcorrenciaService;
use Util\MsgProcesso;
use Ems\Services\ApuracaoService;


use Util\{ Environment, Messages, DateUtil, NovoCodigo };
use Folha\Logic\ControlePonto\Apuracao;

class ApuracaoAdcNoturnoService {

    /** @var Environment $environment */
    protected $environment;

    protected $periodoRepository;

    /** @var EntityManager $entityManager */
    protected $entityManager;

    /** @var VinculoRepository */
    protected $vinculoRepository;

    /** @var VinculoService */
    protected $vinculoService;

    /** @var GpeMarcacaoponto */
    protected $marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco;

    /** @var ApuracaoService */
    protected $apuracaoService;

    /** @var ApuracaoAdcNoturnoRepository */
    protected $apuracaoAdcNoturnoRepository;

    const ORIGEM_PTO = 'PTO';
    
    // Constants usadas para validação
    const APUR_TEMPO_MAXIMO_SEM_ALMOCO          = 6;
    const APUR_TEMPO_MINIMO_ALMOCO              = 1;
    const APUR_TEMPO_MIN_SEM_ALMOCO             = 4;
    
    const TEMPO_JORNADA_INTERVALO_15_TEMPO_MAX  = 6;
    const TEMPO_JORNADA_INTERVALO_15_TEMPO_MIN  = 4;
    
    const TEMPO_MINIMO_INTERVALO                = 0.25;
    
    const APUR_JORNADA_EST                      = 6;
    const APUR_INTERVALO_JORNADA_EST            = 15 / 60;
    
    // Tipos de vínculo cujo processo de apuração é realizado dia a dia
    const TIPOS_VINC_APUR_DIARIA = [VinculoService::TIPO_VINC_EST];


    // Constantes usadas para a determinação de intervalos
    const IDTIPOINTERVALO_TRABALHO          = 'TRABALHO';
    const LABEL_IDTIPOINTERVALO_TRABALHO    = 'Trabalho';
    const IDTIPOINTERVALO_AUSENCIA          = 'AUSENCIA';
    const LABEL_IDTIPOINTERVALO_AUSENCIA    = 'Ausência';
    
    // Tipos de Marcação (Coletores)
    const NRCOLETOR_MARCACAO_DIGITADA               = 24;
    const NRCOLETOR_MARCACAO_IMPORTADA_REP          = 26;
    const CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT     = self::NRCOLETOR_MARCACAO_DIGITADA;
    const IMPORTACAOREP_DEFAULT_NRCOLETOR_DEFAULT   = self::NRCOLETOR_MARCACAO_IMPORTADA_REP;
    
    // Tipo de dia de ocorrência das marcações
    const IDDIAOCORRENCIA_DIA_ANTERIOR          = 'DA';
    const IDDIAOCORRENCIA_DIA_CORRENTE          = 'DC';
    const IDDIAOCORRENCIA_DIA_SEGUINTE          = 'DS';
    const IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR    = 'FA';
    const IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE    = 'FS';
    const IDDIAOCORRENCIA_FORCA_DIA_CORRENTE    = 'FC';
    
    const IMPORTACAO_TEMPO_TOLERANCIA = '02:00';
    const IMPORTACAO_TEMPO_TOLERANCIA_ANTERIOR = self::IMPORTACAO_TEMPO_TOLERANCIA;
    const IMPORTACAO_TEMPO_TOLERANCIA_SEGUINTE = self::IMPORTACAO_TEMPO_TOLERANCIA;
    
    const IMPORTACAO_DEFAULT_DIA = self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
    const IMPORTACAO_DIA_CORRENTE = self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
    const IMPORTACAO_DIA_SEGUINTE = self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE;
    const IMPORTACAO_DIA_ANTERIOR = self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR;
    
    // Agrupações de dias de ocorrencia
    const IDDIAOCORRENCIA_ANTERIORES    = [self::IDDIAOCORRENCIA_DIA_ANTERIOR, self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR];
    const IDDIAOCORRENCIA_CORRENTES     = [self::IDDIAOCORRENCIA_DIA_CORRENTE, self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE];
    const IDDIAOCORRENCIA_SEGUINTES     = [self::IDDIAOCORRENCIA_DIA_SEGUINTE, self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE];
    const CADAUTOMARC_DEFAULT_IDDIAOCORRENCIA_DIA_CORRENTE = self::IDDIAOCORRENCIA_DIA_CORRENTE;
    
    // ID Enquadra Horário
    const IDENQUADRAHORARIO_DEFAULT_ONCREATION  = 'N';
    const MARCDESPREZADA_DEFAULT_ONCREATION     = 'N';

    // Controla marcacao
    const IDCTRLMARC_NORMAL = 'N';
    const IDCTRLMARC_AUTOMATICO = 'A';

    public function __construct(
        Environment                   $environment,
        EntityManager                 $entityManager,
        MarcacaoRepository            $marcacaoRepository,
        VinculoRepository             $vinculoRepository,
        OcorrenciaService             $ocorrenciaService,
        ApuracaoService               $apuracaoService,
        ApuracaoAdcNoturnoRepository  $apuracaoAdcNoturnoRepository

    ) {

        $this->environment                    = $environment;
        $this->entityManager                  = $entityManager;
        $this->marcacaoRepository             = $marcacaoRepository;
        $this->vinculoRepository              = $vinculoRepository;
        $this->ocorrenciaService              = $ocorrenciaService;
        $this->apuracaoService                = $apuracaoService;
        $this->apuracaoAdcNoturnoRepository   = $apuracaoAdcNoturnoRepository;
        $this->itemLoteLanc                   = Repositories::FPA_ITEMLOTELANC;
        $this->loteLanc                       = Repositories::FPA_LOTELANCEVENTO;
        $this->msgProcesso                    = MsgProcesso::getInstance(); 
    }

    public function apurarAutoAdcNoturno($row) 
    {   
        $diaInicio = $row['DTINICIOPERIODO'] ?? null;
        $diaFim = $row['DTFIMPERIODO'] ?? null;
        $nrvinculomList = $row['NRVINCULOM_LIST'] ?? array();
        $nrvinculom = $row['NRVINCULOM'] ?? null;
        $idApuraAutoAdcNoturno = $row['IDAPURAUTOADCNOTUR'] ?? 'UNID';
        $nrPeriodo = $row['NRPERIODO'] ?? null;
        $vinculoLista = array();

        foreach ($nrvinculomList as $item) {
            $vinculoLista[] = $item['NRVINCULOM'];
        }
    
        $resultadoMarcacao = $this->setMarcacoesAutomatica($diaInicio, $diaFim, $vinculoLista, $nrvinculom, $idApuraAutoAdcNoturno);

        if (!$resultadoMarcacao) {
            return;
        }

        // Apurar os períodos em lote
        $resultadosApuracao = [];
        if ($idApuraAutoAdcNoturno === 'VINC') {
            $nrvinculomInt = (int)$nrvinculom;
            $resultadosApuracao[] = $this->apuracaoService->apurarPeriodo($nrvinculomInt, $nrPeriodo);
        } else {
            foreach ($vinculoLista as $nrvinc) {
                $resultadosApuracao[] = $this->apuracaoService->apurarPeriodo($nrvinc, $nrPeriodo);
            }
        }

        if (empty($resultadosApuracao)) {
            return;
        }

        // Obter dados do adicional noturno em lote
        $resultadosAdcNoturno = [];
        if ($idApuraAutoAdcNoturno === 'VINC') {
            $nrvinculomInt = (int)$nrvinculom;
            $resultadosAdcNoturno[] = $this->apuracaoAdcNoturnoRepository->getQtdAdcNoturno($diaInicio, $diaFim, $nrvinculomInt, $this->environment->getNrOrg());
        } else {
            foreach ($vinculoLista as $nrvinc) {
                $resultadosAdcNoturno[] = $this->apuracaoAdcNoturnoRepository->getQtdAdcNoturno($diaInicio, $diaFim, $nrvinc, $this->environment->getNrOrg());
            }
        }

        // Gerar itens do lote
        $itensLote = [];
        if ($resultadosAdcNoturno) {
            foreach ($resultadosAdcNoturno as $adcNoturno) {
                $itensLote[] = [
                    'NRVINCULOM' => $adcNoturno[0]['NRVINCULOM'],
                    'NMVINCULOM' => '',
                    'NREVENTOM' =>  $row['NREVENTOM'],
                    'NMEVENTOH' => '',
                    'VRREFFOLHALOTE' => $adcNoturno[0]['QTDE_HORAS'],
                    'VREVENTOLOTE' => null,
                ];
            }
        }


        $descricaoLote = 'EXPORTAÇÃO DO ADICIONAL NOTURNO';
        // Gerar o lote de eventos
        $lote = $this->geraLoteLancEvento(
            $row['NRORG'], 
            Utils::getParamters()->getDtmescomptencDiaCorrente(), 
            'PTO', 
            $descricaoLote, 
            'PENDENTE', 
            '', 
            1,  
            $itensLote
        );
    }

    /**
    *  Rotina responsável por realizar o cadastro de várias marcações para vários vínculos e dias
    * 
    *  @param  string  $diaInicio
    *  @param  string  $diaFim
    *  @param  Array   $nrvinculomList
    *  @param  int     $nrvinculom
    *  @param  string  $idApuraAutoAdcNoturno
    *  @return bool
    *  @throws \Exception
    */
   public function setMarcacoesAutomatica(
       string $diaInicio,
       string $diaFim,
       $nrvinculomList,
       $nrvinculom,
       $idApuraAutoAdcNoturno
   ): bool {
       $nrorg = (int) $this->environment->getNrOrg();
       $dtmescompetenc = Utils::getParamters()->getDtmescomptencDiaCorrente();
       $cdoperador     = (string)$this->environment->getCdOperador();
       
       
       $dataInicio     = DateUtil::getDataDeString($diaInicio, DateUtil::FORMATO_BRASILEIRO, true);
       $dataFim        = DateUtil::getDataDeString($diaFim, DateUtil::FORMATO_BRASILEIRO, true);
       $dataCorrente   = $dataInicio;
       
       $nrMarcacaoPontoSaidaInseridas = [];
       $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco = [];

       $this->entityManager->beginTransaction();

       try {

           // Passa dia a dia, inserindo as marcações para os vínculos filtrados
           while ($dataCorrente <= $dataFim) {
               // $dataString = $dataCorrente->format(DateUtil::FORMATO_BRASILEIRO);

                if( $idApuraAutoAdcNoturno === 'VINC') {
                    $nrvinculomList = array();
                    $nrvinculomList[] = $nrvinculom;
                    $vinculos = $this->vinculoRepository->getVinculosByEstruturas($nrorg, $dtmescompetenc, null, null, $nrvinculomList);
                } else {
                    $vinculos = $this->vinculoRepository->getVinculosByEstruturas($nrorg, $dtmescompetenc, null, null, $nrvinculomList);
                }

               if (empty($vinculos) || !is_array($vinculos) || count($vinculos) < 1) {
                   throw new \Exception(Messages::HCM00000_GER_VINC_N_ENCONTR_FILTRO_INF_ERROR, Messages::ALERT_MESSAGE_CODE);
               }
   
               $nrVinculos = array_map(function ($item) { return $item['NRVINCULOM']; }, $vinculos);

                $this->setMarcacoesCadAutoPorHorario(
                    $nrVinculos, $dataCorrente
                );

               
               $dataCorrente = DateUtil::adicionaIntervalo($dataCorrente);
           }

           $this->entityManager->flush();
           $this->entityManager->commit();
       } catch (\Exception $e) {
           $this->entityManager->rollback();
           throw $e;
       }

       /* Inclui as marcacoes de almoco conforme as informacoes armazenadas na criacao das marcacoes */
       $dadosMarcacaoesSaida = array_map(
           function ($item) {
               return [
                   'NRVINCULOM' => $item->getNrvinculom(),
                   'NRMARCACAOPONTO' => $item->getNrMarcacaoponto(),
               ];
           },
           $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco
       );


       if ($dadosMarcacaoesSaida) {
    
        foreach ($dadosMarcacaoesSaida as $dados) {
            $nrvinculom = $dados['NRVINCULOM'];
            $nrmarcacaoponto = $dados['NRMARCACAOPONTO'];

            $this->ocorrenciaService->setOcorrenciaAlmocoByNrmarcacao(
                $nrorg,
                $nrmarcacaoponto,
                $cdoperador
                );
            }
      
       }
       
       return true;
   }

    private function setMarcacoesCadAutoPorHorario(
    array $nrVinculos,
    \DateTime $data
    ): void {
        $dataString = $data->format(DateUtil::FORMATO_BRASILEIRO);
        $nrorg = (int) $this->environment->getNrOrg();
        $dtmescompetenc = Utils::getParamters()->getDtmescomptencDiaCorrente();

        foreach($nrVinculos as $nrVinculo){
            $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVincMestreHistorico($nrorg, $nrVinculo, DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y'));
            if(is_null($vinculo)) {
                continue;
            }
            $situacao = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoPorDatainicio($nrorg, $nrVinculo, $data->format('d/m/Y'));            

            $situacaoh = null;
            /* Verifica se o vínculo está afastado ou de atestado*/
            if(!empty($situacao)) {
                $situacaoh = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->getAlteSitufunc(DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y'), $nrorg, $situacao->getNrsitufuncm());
                $idafastamento = $situacaoh[0]['IDAFASTSITUFUNC'];
                $nrsitufunc = $situacaoh[0]['NRSITUFUNCM'];
            }
            if(!empty($situacaoh) && ($idafastamento == 'S' || $nrsitufunc == '12' || $nrsitufunc == '15' ||$nrsitufunc == '16' ||$nrsitufunc == '17')) {
                continue;
            }

            /* Verifica se o vínculo se enquadra no artigo 62 da CLT */
            $escala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->getAlteEscalaByCompetencia($nrVinculo, $nrorg, DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y')); 
            if($escala['IDCONTJORNADA'] == 'S') {
                continue;
            }
            /* Verifica se o vínculo está de Férias */
            $ferias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaItem($nrorg, $nrVinculo);
            $emFerias = false;
            foreach($ferias as $gozo) {
                // Verifica se a data da marcação está entre um período de gozo ou é igual ao começo ou final de gozo
                if((DateUtil::comparaDatas($data, $gozo->getDtinigozoferias(), 2) && DateUtil::comparaDatas($data, $gozo->getDtfimgozoferias(), 1)) 
                || (DateUtil::comparaDatas($data, $gozo->getDtinigozoferias(), 3) || DateUtil::comparaDatas($data, $gozo->getDtfimgozoferias(), 3))) {
                    $emFerias = true;
                }
            }
            if($emFerias) {
                continue;
            }
            /* Horario do dia em questao */
            $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrorg, $nrVinculo, $dataString, $dtmescompetenc );
            if(empty($dadosHorarioVinculo)){
                continue;
            }
            /* Verifica se o dia eh feriado 
            Caso o tipo do horário seja turno, o processo deve continuar */
            if ($this->isFeriadoVinculo($nrVinculo, $dataString) && $dadosHorarioVinculo['IDTIPOHORARIO'] != "T") {
                continue;
            }
            /* Busca os intervalos do horario */
            $intervalos = $this->getIntervalosHorario($nrorg, $dadosHorarioVinculo['NRHORDIAM'] ?? null);
            
            /* Busca pela ocorrência de abono de falta no dia em questão e o exclui */
            if($dadosHorarioVinculo['IDTIPOHORARIO'] == "T") {
                $ocorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array("nrorg" => $nrorg, "nrvinculom" => $nrVinculo, "dtinicioprogocor" => $data, "dtfimprogocor" => $data));
                if(!empty($ocorrencia)) {
                    $this->entityManager->remove($ocorrencia);
                }
            }
            
            
            /* Cria as marcacoes de acordo com os intervalos */
            foreach($intervalos as $intervalo){
                $dataHoraEntrada = $this->getMarcacaoFromHrIntervalo($data, DateUtil::getHoraFloatDeString($intervalo->getHrentrada()));
                $iddiaocorrenciaEntrada = $this->translateIdDiaOcorrenciaFromHorario($intervalo->getIddiaocorreentrada());

                /* Somente intervalos com controle 'Normal' sao considerados */
                if ($intervalo->getIdctrlmarcentrada() == self::IDCTRLMARC_NORMAL) {
                    $this->criarMarcacao(
                        $nrVinculo, $dataHoraEntrada, self::CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT, $iddiaocorrenciaEntrada
                    );
                }

                $dataHoraSaida = $this->getMarcacaoFromHrIntervalo($data, DateUtil::getHoraFloatDeString($intervalo->getHrsaida()));
                $iddiaocorrenciaSaida = $this->translateIdDiaOcorrenciaFromHorario($intervalo->getIddiaocorresaida());

                /* Somente intervalos com controle 'Normal' sao considerados */
                if ($intervalo->getIdctrlmarcsaida() == self::IDCTRLMARC_NORMAL) {
                    $marcacaoSaida = $this->criarMarcacao(
                        $nrVinculo, $dataHoraSaida, self::CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT, $iddiaocorrenciaSaida
                    );
    
                    /* Armazena a primeira marcacao de saida para manipulacoes posteriores */
                    if (!empty($marcacaoSaida) && $intervalo->getNrseqintervhorario() == 1 && count($intervalos) > 1) {
                        $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco[] = $marcacaoSaida;
                    }
                }
            }
        }
    }

    private function isFeriadoVinculo($vinculo, $date)
    {   
        $feriadosVinc = $this->vinculoRepository->getFeriadosVinculo($vinculo,  $this->environment->getNrOrg(),  Utils::getParamters()->getDtmescomptencDiaCorrente());
        
        foreach($feriadosVinc as $feriadoVinc) {
            if($feriadoVinc['DTCALENDARIO'] == $date) {
                return true;
            }
        }
        return false;
    }

    private function getIntervalosHorario(string $nrOrg, string $nrHordiam): ?Array {
        return $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(
            array('nrorg' => $nrOrg, 'nrhordiam' => $nrHordiam),
            array('nrseqintervhorario' => 'ASC')
        );
    }

    /**
     * Soma a hora a data informada
     * 
     * @param \DateTime $dtHrMarc
     * @param ?float    $hora
     * 
     * @return \DateTime
     */
    private function getMarcacaoFromHrIntervalo(\DateTime $dtHrMarc, ?float $hora): \DateTime {
        $dataFinal = DateUtil::adicionaIntervalo(
            DateUtil::truncateData($dtHrMarc),
            floor($hora),
            DateUtil::HORAS
        );
        
        $dataFinal = DateUtil::adicionaIntervalo(
            $dataFinal,
            round(($hora - floor($hora)) * 60), // Decimal para minutos
            DateUtil::MINUTOS
        );
        
        return $dataFinal;
    }


    /**
     * Traduz o iddiaocorrencia vindo do intervalo para o utilizado nas marcacoes
     */
    public function translateIdDiaOcorrenciaFromHorario(
        string $idDiaOcorrencia
    ): string {

        switch ($idDiaOcorrencia){
            case self::IDDIAOCORRENCIA_DIA_CORRENTE:
                return self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
            case self::IDDIAOCORRENCIA_DIA_ANTERIOR:
                return self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR;
            case self::IDDIAOCORRENCIA_DIA_SEGUINTE:
                return self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE;
        }
    }


    /**
     * Cria uma entidade GpeMarcacaoponto
     */
    public function criarMarcacao(
        string $nrvinculom,
        \DateTime $dataHora,
        string $nrcoletor,
        string $iddiaocorrencia
    ): ?GpeMarcacaoponto {
        $nrorg      = (int) $this->environment->getNrOrg();
        $cdoperador = (int) $this->environment->getCdOperador();
        /* Altera a data da marcacao de acordo com a indicacao do dia da ocorrencia, para que a marcacao faca referencia a data original */
        $dataHora = $this->alteraDataDeAcordoComDiaOcorrencia($dataHora, $iddiaocorrencia);

        /* Validacao para impedir a criacao de marcacoes duplicadas */
        $marcacaoExistente = $this->entityManager->getRepository(Repositories::GPE_MARCACAOPONTO)->findOneBy(array(
            "nrorg" => $nrorg, "nrvinculom" => $nrvinculom, "dthrajustada" => $dataHora
        ));
        if(!empty($marcacaoExistente)) {
            $this->msgProcesso->addMessage(
                "Marcação: '{$dataHora->format(DateUtil::FORMATO_DATAHORA)}' não criada pois já existe uma marcação com mesma data e horário. - Vínculo: $nrvinculom"
            );
            return null;
        }
            
        $nrmarcacaoponto = NovoCodigo::geraCodigo($nrorg, 'GPE_MARCACAOPONTO', 1, 12, true);
        
        $novaMarcacaoPonto = new GpeMarcacaoponto;
        $novaMarcacaoPonto->setNrvinculom($nrvinculom);
        $novaMarcacaoPonto->setNrmarcacaoponto($nrmarcacaoponto);
        $novaMarcacaoPonto->setNrorg($nrorg);
        $novaMarcacaoPonto->setDthrmarcacaoponto($dataHora);
        $novaMarcacaoPonto->setDthrajustada($dataHora);
        $novaMarcacaoPonto->setNrcoletor($nrcoletor);
        $novaMarcacaoPonto->setIddiaocorrencia($iddiaocorrencia);
        $novaMarcacaoPonto->setIdenquadrahorario(self::IDENQUADRAHORARIO_DEFAULT_ONCREATION);
        $novaMarcacaoPonto->setIdmarcdesprezada(self::MARCDESPREZADA_DEFAULT_ONCREATION);
        $novaMarcacaoPonto->setIdativo('S');
        $novaMarcacaoPonto->setDtinclusao(DateUtil::getDataAtual());
        $novaMarcacaoPonto->setCdoperinclusao($cdoperador);
        $this->entityManager->persist($novaMarcacaoPonto);
        
        return $novaMarcacaoPonto;
    }
    
    /**
     * De acordo com a ocorrencia retorna o dia ajustado correspondente
     */
    public function alteraDataDeAcordoComDiaOcorrencia(
        \DateTime $dataMarcacao,
        string $iddiaocorrencia
    ): \DateTime {

        if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_ANTERIORES)) {
            return DateUtil::adicionaIntervalo($dataMarcacao, 1);
        } else if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_SEGUINTES)) {
            return DateUtil::subtraiIntervalo($dataMarcacao, 1);
        }
        return $dataMarcacao;
    }


    public function setOrigemLoteLanc($origem) {
        try {
            switch ($origem) {
                case self::ORIGEM_PTO:
                    $this->idoriglotelanc = self::ORIGEM_PTO;
                    break;
            }

            if ($this->idoriglotelanc === 'undefined') {
                throw new \Exception(Messages::HCM01000_FPA_LOTELANCEVEN_ORIG_UNDEFINED, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $dtmescompetencia
     * @param $origem
     * @param $dslotelanc
     * @param $idstatuslote
     * @param $dsobslote
     * @param $nrtipomoviment
     * @param $itens
     * @return $nrlote
     * @throws \Exception
     */
    public function geraLoteLancEvento($nrorg, $dtmescompetencia, $origem, $dslotelanc, $idstatuslote = 'PENDENTE', $dsobslote = null, $nrtipomoviment, $itens) {
        try {
            $this->entityManager->beginTransaction();
           // $this->setOrigemLoteLanc($origem);
            $nrlote = $this->insereItensLoteLancEvento($nrorg, $itens);
            if (!is_null($nrlote)) {
                $this->insereLoteLancEvento($nrorg, $dtmescompetencia, $nrlote, $dslotelanc, $idstatuslote, $dsobslote, $nrtipomoviment);
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
            return $nrlote;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /**
     * @param int $nrorg
     * @param array $itens
     * @param int|null $nrlote Se for passado como nulo é gerado um novo número de lote
     * @return int|null $nrlote
     * @throws \Exception
     */
    public function insereItensLoteLancEvento($nrorg, $itens, $nrlote=null)
    {
        try {
            if (!empty($itens)) {
                foreach ($itens as $info) {
                    if($info['NREVENTOM'] == 480){
                        $info['VRREFFOLHALOTE'] = null;
                    }
                    $dsobsItemlote = isset($info['DSOBSERVACAO']) ? $info['DSOBSERVACAO'] : null;
                    $dtocorrencia  = isset($info['DTOCORRENCIA']) ? DateUtil::getDataDeString($info['DTOCORRENCIA'], DateUtil::FORMATO_BRASILEIRO, true) : null;
                    
                    $obj = new $this->itemLoteLanc;
                    $obj->setNrvinculom($info['NRVINCULOM']);
                    $obj->setNreventom($info['NREVENTOM']);
                    $obj->setVrreffolhalote($info['VRREFFOLHALOTE']);
                    $obj->setVreventolote($info['VREVENTOLOTE']);
                    $obj->setIdativo('S');
                    $obj->setNrorg($nrorg);
                    $obj->setNrorginclusao($nrorg);
                    $obj->setDtinclusao(DateUtil::getDataAtual());
                    $obj->setCdoperinclusao($this->environment->getCdOperador());
                    $obj->setNrorgultatu($nrorg);
                    $obj->setDtultatu(DateUtil::getDataAtual());
                    $obj->setCdoperultatu($this->environment->getCdOperador());
                    $obj->setDsobservacao($dsobsItemlote);
                    $obj->setDtocorrencia($dtocorrencia);
                    if (($obj->getVreventolote()) || ($obj->getVrreffolhalote())) {
                        if (is_null($nrlote)) {
                            $nrlote = NovoCodigo::geraCodigo($nrorg, 'FPA_LOTELANCEVENTO', 1, 22, true, false);
                        }
                        $obj->setNrlotelancevento($nrlote);
                        $obj->setNritlotelanc(Novocodigo::geraCodigo($nrorg, 'FPA_ITEMLOTELANC', 1, 22, true, false));
                        $this->entityManager->persist($obj);
                    }
                }
            }
            return $nrlote;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $dtmescompetencia
     * @param $nrlote
     * @param $dslotelanc
     * @param $idstatuslote
     * @param $dsobslote
     * @param $idoriglote
     * @param $nrtipomoviment
     * @throws \Exception
     */
    public function insereLoteLancEvento($nrorg, $dtmescompetencia, $nrlote, $dslotelanc, $idstatuslote, $dsobslote, $nrtipomoviment, $nmestrutlegal = null, $nmestrutgeren = null) {
        try {

            if (!$nrlote) {
                $nrlote = NovoCodigo::geraCodigo($nrorg, 'FPA_LOTELANCEVENTO', 1, 22, true, false);
            }
            if(empty($dsobslote)){
                if(!empty($nmestrutlegal)){
                    $dsobslote = 'Estrutura Legal: '.$nmestrutlegal.'
';
                }
                if(!empty($nmestrutgeren)){
                    $dsobslote .= 'Estrutura Gerencial: ' .$nmestrutgeren;
                }
                
            }
            $obj = new $this->loteLanc;
            $obj->setNrlotelancevento($nrlote);
            $obj->setDtlotelancevento(DateUtil::getDataAtual());
            $obj->setDtcompetloteeve(DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
            $obj->setDslotelancevento($dslotelanc);
            $obj->setIdstatuslotelanc($idstatuslote);
            $obj->setIdoriglotelanc('PTO');
            $obj->setNrtipomoviment($nrtipomoviment);
            $obj->setDsobslotelanc($dsobslote);
            $obj->setIdativo('S');
            $obj->setNrorginclusao($nrorg);
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->environment->getCdOperador());
            $obj->setNrorgultatu($nrorg);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->environment->getCdOperador());
            $obj->setNrorg($nrorg);
            $this->entityManager->persist($obj);
            $this->entityManager->flush($obj);
            return $obj;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosHorarioVinculo($nrorg, $nrvinculom, $dtapuracaoponto, $dtmescompetenc = null) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT HT.NRHORDIAM, HD.IDTIPOHORARIO, IH.NRINTERVALOHORARIO, HH.NMHORDIAH, AE.NRESCALATRABM, EH.QTHRESCALATRABH AS QTJORNADADIARIA
		  FROM (SELECT NRORG, MAX(NRTURNO) NRTURNO, NRESCALATRABM
					 FROM GPE_ALTEESCALA
					WHERE NRORG = :P_NRORG
					  AND NRVINCULOM = :P_NRVINCULOM
					  --AND TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
                      AND DTINIESCALA = (SELECT MAX(DTINIESCALA)
                                 FROM GPE_ALTEESCALA ALT
                                WHERE ALT.NRORG        = GPE_ALTEESCALA.NRORG
                                  AND ALT.NRVINCULOM   = GPE_ALTEESCALA.NRVINCULOM
                                  AND ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
					GROUP BY NRORG, NRESCALATRABM) AE,
				 (SELECT NRORG, NRTURNO, TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')-DTDATABASE AS NRDIASENTRE
					 FROM GPE_TURNO
					WHERE NRORG = :P_NRORG) TU,
				 (SELECT NRORG, NRTURNO, NRHORDIAM,
							ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					ORDER BY NRORG, NRTURNO, NRSEQHORARIO) HT,
				 (SELECT NRORG, NRTURNO, COUNT(*) AS QTHORARIOS
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRTURNO) QH,
				 (SELECT NRORG, NRHORDIAM, MAX(DTMESCOMPETENC) AS DTMESCOMPETENC, NMHORDIAH
					 FROM GPE_HORDIAH
					WHERE NRORG = :P_NRORG
					  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
					GROUP BY NRORG, NRHORDIAM, NMHORDIAH) HH,
				 (SELECT NRORG, NRHORDIAM, MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO
					 FROM GPE_INTERVALOSHORARIO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRHORDIAM) IV, GPE_INTERVALOSHORARIO IH, GPE_HORDIAH HD, GPE_ESCALATRABH EH
		 WHERE AE.NRORG = TU.NRORG
			AND AE.NRTURNO = TU.NRTURNO
			AND AE.NRORG = HT.NRORG
			AND AE.NRTURNO = HT.NRTURNO
			AND HT.NRORG = HH.NRORG
			AND HT.NRHORDIAM = HH.NRHORDIAM
			AND HD.NRORG = HH.NRORG
			AND HD.NRHORDIAM = HH.NRHORDIAM
			AND HD.DTMESCOMPETENC = HH.DTMESCOMPETENC
			AND IV.NRORG(+) = HH.NRORG
			AND IV.NRHORDIAM(+) = HH.NRHORDIAM
			AND IH.NRORG(+) = IV.NRORG
			AND IH.NRHORDIAM(+) = IV.NRHORDIAM
			AND IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO
			AND AE.NRORG = QH.NRORG
			AND AE.NRTURNO = QH.NRTURNO
			AND HT.NRSEQHR-1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS)
      AND EH.NRESCALATRABM = AE.NRESCALATRABM
      AND EH.NRORG = AE.NRORG
      AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                 FROM GPE_ESCALATRABH
                                WHERE NRORG = EH.NRORG
                                  AND NRESCALATRABM = AE.NRESCALATRABM
                                  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
        ");

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTMESCOMPETENCIA" => is_null($dtmescompetenc) ? DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO) : $dtmescompetenc,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
            )
        );
        
        $dadosHorario = $query->fetchAll();

        if (isset($dadosHorario[0])) {
            return $dadosHorario[0];
        } else {
            return false;
        }
    }

    
    
}