<?php

namespace Ems\Services;

use Ems\Services\{
    AdmissaoService,
    EnderecoService,
    PessoaService
};
use Ems\Repository\GeralRepository;
use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };
use Util\Environment;
use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;

class GupyService {
    public const ORG_20_TOKEN = '50d22558-0c0f-4b62-97e2-e459e886a7c1';
    public const DEFAULT_ORG = 20;
    public const DEFAULT_OPERADOR = '000000000001';

    /** @var EnderecoService */
    protected $enderecoService;

    /** @var GeralRepository */
    protected $geralRepository;

    /** @var Environment */
    protected $environment;

    /** @var PessoaService */
    protected $pessoaService;

    /** @var AdmissaoService */
    protected $admissaoService;

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        GeralRepository $geralRepository,
        EnderecoService $enderecoService,
        Environment $environment,
        PessoaService $pessoaService,
        EntityManager $entityManager
    ) {
        $this->geralRepository = $geralRepository;
        $this->enderecoService = $enderecoService;
        $this->environment = $environment;
        $this->pessoaService = $pessoaService;
        $this->entityManager = $entityManager;
    }

    public function gupyPreEmployeeMoved(
        array $row
    ) {
        $this->verifyAuthentication();

        $this->environment->setNrOrg(self::DEFAULT_ORG);
        $this->environment->setCdOperador(self::DEFAULT_OPERADOR);

        $data = $row['data'];
        $this->verifyData($data);

        $candidate = $data['candidate'] ?? null;
        $documents = $data['admission']['documents']['preEmployee'] ?? null;
        $dependentsDocuments = $data['admission']['documents']['dependents'] ?? [];
        $dtAdmissaoPreliminar = $data['admission']['hiringDate'] ?? null;

        $this->admissaoService = Utils::getInstance('AdmissaoService');

        $admissaoRow = $this->createAdmissaoRow($candidate, $documents, $dtAdmissaoPreliminar);
        $nmMae = $admissaoRow['dadosPessoais']['NMMAE'] ?? null;
        $nmPai = $admissaoRow['dadosPessoais']['NMPAI'] ?? null;

        $this->entityManager->beginTransaction();

        try {
            [$nrParcNegocio, $nrPessoa, $nrPessoaH] = $this->admissaoService->setDadosPessoaisAdmissao($admissaoRow);
            $this->setDependentes($dependentsDocuments, $nrParcNegocio, $nmMae, $nmPai);

            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    private function verifyData(
        array $data
    ) {
        $candidate = $data['candidate'] ?? null;
        $status = $data['admission']['status'] ?? null;
        $documents = $data['admission']['documents']['preEmployee'] ?? null;
        $cpf = $candidate['identificationDocument'] ?? null;
        $hiringDate = $data['admission']['hiringDate'] ?? null;

        $validStati = [
            'SEND_DOCUMENTS',
            'SIGNING_CONTRACT',
        ];

        // Não trocar var_dump por Exception!!!
        if (!$candidate) {
            var_dump('Candidato não informado!'); die;
        }

        if (!in_array($status, $validStati)) {
            var_dump('Status inválido'); die;
        }

        if (!$cpf) {
            var_dump('CPF não informado!'); die;
        }

        if (!$documents) {
            var_dump('Não há documentos!'); die;
        }

        if (!$this->documentsHaveValues($documents)) {
            var_dump('Documentos não têm resposta!'); die;
        }

        if ($this->candidateIsRegistered($candidate, $documents, $hiringDate)) {
            var_dump('O candidato já está cadastrado!'); die;
        }
    }

    private function candidateIsRegistered(
        array $candidate,
        array $documents,
        string $hiringDateString
    ): bool {
        $cpf = $candidate['identificationDocument'];
        $pessoa = $this->pessoaService->getPessoaByCpf($cpf);
        $isRegistered = $pessoa !== false;
        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime($hiringDateString));
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        if($isRegistered) {
            $pessoaHistorico = $this->pessoaService->retornaHistoricoAtualPorPessoa($this->environment->getNrOrg(), $pessoa['NRPESSOA'], $dtMesCompetencia);
           
            if($pessoaHistorico !== false && $pessoaHistorico->getDtmescompetenc()->format('d/m/Y') === $dtMesCompetencia) {
                
                $dadosPessoaisRow = $this->getDadosPessoaisRow($candidate, $documents, $dtMesCompetencia);
                $dadosPessoais = $this->validaDadosPessoais($dadosPessoaisRow, $dtMesCompetencia);
               
                $updatePessoaHistorico = $this->updatePessoaHistorico($pessoaHistorico, $dadosPessoais);
            }
            else{             
                $dadosPessoaisRow = $this->getDadosPessoaisRow($candidate,$documents, $dtMesCompetencia);
                $dadosPessoais = $this->validaDadosPessoais($dadosPessoaisRow, $dtMesCompetencia);
                
                $setPessoaHistorico = $this->setPessoaHistorico($pessoa['NRPESSOA'], $dadosPessoais);
            }
        }

        return $isRegistered;
    }

    private function documentsHaveValues(
        array $documents
    ): bool {
        $firstValue = $this->getDocumentValues($documents, 3);

        return !!$firstValue;
    }

    private function verifyAuthentication() {
        $headers = getallheaders();
        $authenticationHeader =
            $headers['Authorization']
            ?? $headers['authorization']
            ?? $headers['AUTHORIZATION']
            ?? null;

        if ($authenticationHeader === null) {
            var_dump('Token de autenticação não está presente!', $headers);die;
        }

        $validToken = strpos($authenticationHeader, self::ORG_20_TOKEN) !== false;

        if (!$validToken) {
            var_dump('Invalid Gupy Token!');die;
        }
    }

    private function createAdmissaoRow(
        array $candidate,
        array $documents,
        $dtAdmissaoPreliminar
    ): array {
        $dadosPessoais = $this->getDadosPessoaisRow($candidate, $documents, $dtAdmissaoPreliminar);
        $comunicacao = $this->getComunicacaoRow($candidate);
        $endereco = $this->getEnderecoRow($documents);
        $conta = $this->getContaRow($documents);
        $documentos = [];

        $admissaoRow = [
            'CDOPERADOR' => self::DEFAULT_OPERADOR,
            'dadosPessoais' => $dadosPessoais,
            'comunicacao' => $comunicacao,
            'endereco' => $endereco,
            'conta' => $conta,
            'documentos' => $documentos,
        ];

        return $admissaoRow;
    }

    private function getDocumentValues(
        array $documents,
        int $docId
    ) {
        foreach ($documents as $doc) {
            if ($doc['id'] === $docId) {
                $docValues = $doc['values'] ?? null;
                return $docValues;
            }
        }

        return null;
    }

    private function getDadosPessoaisRow(
        array $candidate,
        array $documents,
        $dtAdmissaoPreliminar
    ): array {
        $rgValues = $this->getDocumentValues($documents, 3) ?? [];
        $pisValues = $this->getDocumentValues($documents, 40035) ?? [];
        $escolaridadeValues = $this->getDocumentValues($documents, 40134) ?? [];
        $condicaoFisValues = $this->getDocumentValues($documents, 40121) ?? [];
        $militarValues = $this->getDocumentValues($documents, 40133) ?? [];

        $candidateRow = $this->getDataFromCandidate($candidate);
        $rgRow = $this->getDataFromRG($rgValues);
        $pisRow = $this->getDataFromPis($pisValues);
        $escolaridadeRow = $this->getDataFromComprovanteEscolar($escolaridadeValues);
        $condicaoFisRow = $this->getDataFromCondicaoFisica($condicaoFisValues);
        $militarRow = $this->getDataFromCertReservista($militarValues);

        $dadosPessoais = array_merge(
            $candidateRow,
            $rgRow,
            $pisRow,
            $escolaridadeRow,
            $condicaoFisRow,
            $militarRow
        );

        // caso precise definir a data de admissão preliminar na row, basta
        // descomentar a linha abaixo
        // $dadosPessoais['DTADMISSAOPRE'] = $dtAdmissaoPreliminar;
        $dadosPessoais['DTMESCOMPETENC'] = date('01/m/Y');
        $dadosPessoais['INSERE_OPERADOR'] = 'S';

        return $dadosPessoais;
    }

    private function getDataFromCandidate(
        array $candidate
    ): array {
        $cpf = $candidate['identificationDocument'] ?? null;
        $email = $candidate['email'];

        $disabilities = $candidate['disabilities'] ?? null;
        $idCotaPCD = $disabilities ? 'S' : 'N';

        $row = [
            'NRCPFPESSOA' => $cpf,
            'IDCOTAPCD' => $idCotaPCD,
            'EMAIL' => $email,
        ];

        return $row;
    }

    private function getEnderecoRow(
        array $documents
    ): array {
        $enderecoValues = $this->getDocumentValues($documents, 2) ?? [];
        $enderecoRow = $this->getDataFromEndereco($enderecoValues);

        return $enderecoRow;
    }

    private function getContaRow(
        array $documents
    ): array {
        $contaValues = $this->getDocumentValues($documents, 40032) ?? [];
        $contaRow = $this->getDataFromDadosBancarios($contaValues);

        return $contaRow;
    }

    private function getComunicacaoRow(
        array $candidate
    ): array {
        $telefone = empty($candidate['phoneNumber']) ?
            null :
            $candidate['phoneNumber'];

        [$ddiTel, $dddTel, $phoneTel] = $this->getPhoneInfo($telefone);

        $celular = empty($candidate['mobileNumber']) ?
            null :
            $candidate['mobileNumber'];

        [$ddiCel, $dddCel, $phoneCel] = $this->getPhoneInfo($celular);

        $email = $candidate['email'];

        $comunicacao = [
            'TELPRINCIPAL' => $phoneTel,
            'PREFTELPRINCIPAL' => $dddTel,
            'CELPARTIC' => $phoneCel,
            'PREFCELPARTIC' => $dddCel,
            'EMAIL' => $email,
        ];

        return $comunicacao;
    }

    // Obs: funciona apenas para números de
    // telefone no formato brasileiro
    private function getPhoneInfo(
        $phoneNumber
    ): array {
        if (!$phoneNumber) {
            return [null, null, null];
        }

        $ddi = substr($phoneNumber, 0, 3);
        $ddd = substr($phoneNumber, 3, 2);
        $phone = substr($phoneNumber, 5);

        $phoneInfo = [$ddi, $ddd, $phone];
        return $phoneInfo;
    }

    private function getDataFromEndereco(
        array $docValues
     ): array {
        $cdPais = null;
        $sgEstado = null;
        $cdMunicipio = null;
        $cdBairro = null;
        $cdLogradouro = null;

        $nmPais         = $docValues['pais'] ?? null;
        $nmEstado       = $docValues['estado'] ?? null;
        $nmMunicipio    = $docValues['cidade'] ?? null;
        $cep            = $docValues['cep'] ?? null;
        $nmBairro       = $docValues['bairro'] ?? null;
        $endereco       = $docValues['logradouro'] ?? null;
        $numero         = $docValues['numero'] ?? null;
        $complemento    = $docValues['complemento'] ?? null;

        if ($nmPais) {
            $pais = $this->geralRepository->getPaisByNameOrSg($nmPais);

            if ($pais) {
                $cdPais = $pais['CDPAIS'];
            }
        }

        if ($cdPais && $nmEstado) {
            $estado = $this->geralRepository->getEstadoByNameOrSg(
                $cdPais,
                $nmEstado
            );

            if ($estado) {
                $sgEstado = $estado['SGESTADO'];
            }
        }

        if ($cdPais && $sgEstado && $nmMunicipio) {
            $municipio = $this->geralRepository->getCidadeByName(
                $cdPais,
                $sgEstado,
                $nmMunicipio
            );

            if ($municipio) {
                $cdMunicipio = $municipio['CDMUNICIPIO'];
            }
        }

        if ($cdPais && $sgEstado && $cdMunicipio && $nmBairro) {
            $bairro = $this->geralRepository->getBairroByName(
                $cdPais,
                $sgEstado,
                $cdMunicipio,
                $nmBairro
            );

            if ($bairro) {
                $cdBairro = $bairro['CDBAIRRO'];
            }
        }

        if ($endereco) {
            $logradouro = explode(' ', $endereco)[0] ?? null;

            if ($logradouro) {
                $cdLogradouro = $this->enderecoService->getCdLogradouroByName($logradouro);
            }
        }

        $row = [
            'CDTIPOENDERECO' => 'PRINCIPAL',
            'CDPAIS' => $cdPais,
            'SGESTADO' => $sgEstado,
            'CDMUNICIPIO' => $cdMunicipio,
            'NRCEPENDERECO' => $cep,
            'NMBAIRROENDERECO' => $nmBairro,
            'CDBAIRRO' => $cdBairro,
            'CDLOGRADOURO' => $cdLogradouro,
            'DSENDERECO' => $endereco,
            'NRIMOVELENDERECO' => $numero,
            'DSCOMPLEENDERECO' => $complemento,
        ];

        return $row;
    }

    private function getDataFromRG(
        array $docValues
     ): array {
        $cdEstadoCivil = null;
        $nrRaca = null;
        $nrNacionalidade = null;
        $cdPais = null;
        $sgEstado = null;
        $cdMunicipio = null;

        $rg = $docValues['numero-do-rg'] ?? null;
        $dtExpedicaoRg = $docValues['data-de-expedicao'] ?? null;
        $ufEmissor = $docValues['uf-emissor'] ?? null;
        // $orgaoEmissor = $docValues['orgao-emissor'] ?? null;

        $nmEstadoCivil = $docValues['estado-civil'] ?? null;

        $nome = $docValues['nome'] ?? null;
        $dtNacimento = $docValues['data-de-nascimento'] ?? null;
        $genero = $docValues['genero'] ?? null;
        $etnia = $docValues['etnia'] ?? null;
        $idSexo = $genero === 'Masculino' ? 'M' : 'F';

        $nomeMae = $docValues['nome-da-mae'] ?? null;
        $nomePai = $docValues['nome-do-pai'] ?? null;

        $nmPais = $docValues['nacionalidade'] ?? null;
        $nmEstado = $docValues['estado-onde-nasceu'] ?? null;
        $nmMunicipio = $docValues['cidade-onde-nasceu'] ?? null;


        if ($nmEstadoCivil) {
            $cdEstadoCivil = $this->pessoaService->getCdEstadoCivilByNm(
                $nmEstadoCivil
            );
        }

        if ($etnia) {
            $nrRaca = $this->pessoaService->getNrRacaByNm(
                $etnia
            );
        }

        if ($nmPais) {
            $pais = $this->geralRepository->getPaisByNameOrSg(
                $nmPais
            );

            $cdPais = $pais['CDPAIS'] ?? null;

            if ($cdPais) {
                $nacionalidade = $this->geralRepository->getNacionalidadeByCdPais(
                    $cdPais
                );

                if ($nacionalidade) {
                    $nrNacionalidade = $nacionalidade['NRNACIONALIDADE'];
                }
            }
        }

        if ($cdPais && $nmEstado) {
            $estado = $this->geralRepository->getEstadoByNameOrSg(
                $cdPais,
                $nmEstado
            );

            if ($pais) {
                $sgEstado = $estado['SGESTADO'];
            }
        }

        if ($cdPais && $sgEstado && $nmMunicipio) {
            $municipio = $this->geralRepository->getCidadeByName(
                $cdPais,
                $sgEstado,
                $nmMunicipio
            );

            if ($municipio) {
                $cdMunicipio = $municipio['CDMUNICIPIO'];
            }
        }

        if ($dtExpedicaoRg) {
            $dtExpedicaoRg = DateUtil::iso8601ToDDMMYYYY($dtExpedicaoRg);
        }

        if ($dtNacimento) {
            $dtNacimento = DateUtil::iso8601ToDDMMYYYY($dtNacimento);
        }

        $row = [
            'NMMAE' => $nomeMae,
            'NMPAI' => $nomePai,
            'NMPESSOA' => $nome,
            'NRRGPESSOA' => $rg,
            'DTEXRGPESSOA' => $dtExpedicaoRg,
            'SGUFRGPESSOA' => $ufEmissor,
            'NRRACAPESSOA' => $nrRaca,
            'CDESTACIVIL' => $cdEstadoCivil,
            'NRNACIONALID' => $nrNacionalidade,
            'DTNASCPESSOA' => $dtNacimento,
            'IDSEXOPESSOA' => $idSexo,
            'CDPAIS' => $cdPais,
            'SGESTADO' => $sgEstado,
            'CDMUNICIPIO' => $cdMunicipio,
        ];

        return $row;
    }

    private function getDataFromPis(
        array $docValues
     ): array {
        $pis = $docValues['2a368cec-f982-4549-883e-d4771d0a2bc1'] ?? null;
        $dtPis = $docValues['8a0c7600-6bc9-4eab-bf1a-c855318a1a26'] ?? null;

        if ($dtPis) {
            $dtPis = DateUtil::iso8601ToDDMMYYYY($dtPis);
        }

        $row = [
            'NRPISPASEPPES' => $pis,
            'DTINSCPISPASEP' => $dtPis,
        ];

        return $row;
    }

    private function getDataFromComprovanteEscolar(
        array $docValues
     ): array {
        $escolaridade = $docValues['32f6ed00-e429-40b6-a1d0-7e18f0fe0059'] ?? null;

        $escolaridadeMap = [
            'Ensino Fundamental Completo' => 5,
            'Ensino Médio Cursando' => 6,
            'Ensino Médio Concluido' => 7,
            'Ensino Superior Cursando' => 8,
            'Ensino Superior Concluido' => 9,
        ];

        $nrGrauInstrucao = $escolaridadeMap[$escolaridade] ?? null;

        $row = [
            'NRGRAUINSTR' => $nrGrauInstrucao,
        ];

        return $row;
    }

    private function getDataFromDadosBancarios(
        array $docValues
     ): array {
        $banco = '033'; // Santander
        $agencia = $docValues['c02eae32-75c3-4198-bc60-35bc99cae383'] ?? null;
        $conta = $docValues['eaf131d7-98f8-4ac0-83a8-f8c6ff7f5be9'] ?? null;
        $tipoConta = 3; // Conta Salário

        $row = [
            'CDBANCO' => $banco,
            'CDAGENCIA' => $agencia,
            'CDCONTCORR' => $conta,
            'NRTPCONTCORR' => $tipoConta,
        ];

        return $row;
    }

    private function getDataFromVR(
        array $docValues
    ): array {
        return [];
    }

    private function getDataFromVT(
        array $docValues
    ): array {
        return [];
    }

    private function getDataFromCondicaoFisica(
        array $docValues
     ): array {
        $condicaoFisica = $docValues['ba187938-1ef7-41da-8c74-292f3331061c'] ?? null;

        $condicaoMap = [
            'Normal' => 1,
            'Deficiência Física' => 2,
            'Deficiência Visual' => 4,
            'Deficiência Mental' => 5,
            'Deficiência Auditiva' => 3,
            'Deficiência Múltipla' => 6,
            'Reabilitado' => 7,
        ];

        $nrCondicao = $condicaoMap[$condicaoFisica] ?? null;

        $row = [
            'NRCONDFISPES' => $nrCondicao,
        ];

        return $row;
    }

    private function getDataFromCertReservista(
        array $docValues
     ): array {
        $nrCertReservista = $docValues['30ed4c3a-ac60-443d-acb0-004eaf11a2fd'] ?? null;
        $dtEmissaoCert = $docValues['ea0b748e-161b-4091-8d5d-54a8f67434a2'] ?? null;

        if ($dtEmissaoCert) {
            $dtEmissaoCert = DateUtil::iso8601ToDDMMYYYY($dtEmissaoCert);
        }

        $row = [
            'NRCERTRESEPES' => $nrCertReservista,
            'DTCERTRESEPES' => $dtEmissaoCert,
        ];

        return $row;
    }

    private function setDependentes(
        array $dependentsDocuments,
        int $nrParcNegocio,
        $nmMae = null,
        $nmPai = null
    ) {
        $cdOperador = $this->environment->getCdOperador();
        $dependentes = [];
        $dtMesCompetencia = date('01/m/Y');

        if ($nmMae) {
            $rowMae = [
                'NMPESSOA' => $nmMae,
                'NRTIPORELACIONA' => 1, // mae
                'NRCPFPESSOA' => null,
                'DTNASCPESSOA' => null,
            ];

            $dependentes[] = $rowMae;
        }

        if ($nmPai) {
            $rowPai = [
                'NMPESSOA' => $nmPai,
                'NRTIPORELACIONA' => 2, // pai
                'NRCPFPESSOA' => null,
                'DTNASCPESSOA' => null,
            ];

            $dependentes[] = $rowPai;
        }

        foreach ($dependentsDocuments as $depDoc) {
            $dependenteRow = $this->getDependentRow($depDoc);

            if ($dependenteRow) {
                $nomeDep = $dependenteRow['NMPESSOA'];
                $nomeIgualMae = strcasecmp($nmMae, $nomeDep) === 0;
                $nomeIgualPai = strcasecmp($nmPai, $nomeDep) === 0;

                if (!$nomeIgualMae && !$nomeIgualPai) {
                    $dependentes[] = $dependenteRow;
                }
            }
        }

        $this->admissaoService->setDependentesAdmissao(
            $dependentes,
            $cdOperador,
            $nrParcNegocio,
            null,
            $dtMesCompetencia
        );
    }

    private function getDependentRow(
        array $depDoc
    ) {
        $documents = $depDoc['documents'] ?? null;
        $nrTipoRelacionamento = 3; // filho

        if (!$documents) {
            return false;
        }

        $dados = $this->getDependentDocumentValues($documents, 'a3c51d53-3c86-4142-aea8-ed9fc84e0e94');

        if (!$dados) {
            return false;
        }

        $dtNascimento   = $dados['birthDate'] ?? null;
        $cpf            = $dados['cpf'] ?? null;
        $nome           = $dados['name'] ?? null;

        if (!$dtNascimento || !$cpf || !$nome) {
            return false;
        }
        
        if ($dtNascimento) {
            $dtNascimento = DateUtil::iso8601ToDDMMYYYY($dtNascimento);
        }

        $row = [
            'NMPESSOA' => $nome,
            'NRCPFPESSOA' => $cpf,
            'DTNASCPESSOA' => $dtNascimento,
            'NRTIPORELACIONA' => $nrTipoRelacionamento,
        ];

        return $row;
    }

    private function getDependentDocumentValues(
        array $documents,
        string $docId
    ) {
        foreach ($documents as $doc) {
            if ($doc['id'] === $docId) {
                $docValues = $doc['values'] ?? null;
                return $docValues;
            }
        }

        return null;
    }


    private function validaDadosPessoais($dadosPessoais, $dtMesCompetencia)
    {
        $dadosPessoais['DTMESCOMPETENC'] = !empty($dtMesCompetencia) 
        ? DateTime::createFromFormat('d/m/Y', $dtMesCompetencia) 
        : null;

        $dadosPessoais['NRCERTNASCPES']  = $dadosPessoais['NRCERTNASCPES'] ?? null;
        $dadosPessoais['DSAPELIDOPESS']  = $dadosPessoais['DSAPELIDOPESS'] ?? null;

        $dadosPessoais['DTNASCPESSOA']   = !empty($dadosPessoais['DTNASCPESSOA']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTNASCPESSOA']) 
        : null;
        $dadosPessoais['IDSEXOPESSOA']   = $dadosPessoais['IDSEXOPESSOA'] ?? null;
        $dadosPessoais['NRCONDFISPES']   = $dadosPessoais['NRCONDFISPES'] ?? null;
        $dadosPessoais['NRCTPSPESSOA']   = $dadosPessoais['NRCTPSPESSOA'] ?? null;
        $dadosPessoais['NRSERIECTPSPES'] = $dadosPessoais['NRSERIECTPSPES'] ?? null;
        $dadosPessoais['NRCPFPESSOA']    = $dadosPessoais['NRCPFPESSOA'] ?? null;
        $dadosPessoais['SGUFCTPSPES']    = $dadosPessoais['SGUFCTPSPES'] ?? null;

        $dadosPessoais['DTCTPSPESSOA']   = !empty($dadosPessoais['DTCTPSPESSOA']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTCTPSPESSOA']) 
        : null;

        $dadosPessoais['NRPISPASEPPES']  = $dadosPessoais['NRPISPASEPPES'] ?? null;
        $dadosPessoais['DTINSCPISPASEP'] = !empty($dadosPessoais['DTINSCPISPASEP']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTINSCPISPASEP']) 
        : null;

        $dadosPessoais['NRINSCINSSPES']  = $dadosPessoais['NRINSCINSSPES'] ?? null;
        $dadosPessoais['NRINSCISSPES']   = $dadosPessoais['NRINSCISSPES'] ?? null;
        $dadosPessoais['CDESTACIVIL']    = $dadosPessoais['CDESTACIVIL'] ?? null;
        $dadosPessoais['NRCERTCASAPES']  = $dadosPessoais['NRCERTCASAPES'] ?? null;       
        $dadosPessoais['DTCASAPESSOA']   = !empty($dadosPessoais['DTCASAPESSOA']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTCASAPESSOA']) 
        : null;
        $dadosPessoais['IDISENTITUELEI'] = $dadosPessoais['IDISENTITUELEI'] ?? null;
        $dadosPessoais['NRTITUELEIPES']  = $dadosPessoais['NRTITUELEIPES'] ?? null;
        $dadosPessoais['NRSECAELEIPES']  = $dadosPessoais['NRSECAELEIPES'] ?? null;
        $dadosPessoais['NRZONAELEIPES']  = $dadosPessoais['NRZONAELEIPES'] ?? null;
        $dadosPessoais['NRCARTHABPES']   = $dadosPessoais['NRCARTHABPES'] ?? null;            
        $dadosPessoais['DTHABCARTPES']   = !empty($dadosPessoais['DTHABCARTPES']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTHABCARTPES']) 
        : null;   
        $dadosPessoais['DTVALHABCARTPES'] = !empty($dadosPessoais['DTVALHABCARTPES']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTVALHABCARTPES']) 
        : null;
        $dadosPessoais['DSCATEGHABCART']  = $dadosPessoais['DSCATEGHABCART'] ?? null;
        $dadosPessoais['NRCATMILITARPE']  = $dadosPessoais['NRCATMILITARPE'] ?? null;
        $dadosPessoais['NRCERTRESEPES']   = $dadosPessoais['NRCERTRESEPES'] ?? null;
        $dadosPessoais['DSCATEGCERTRESE'] = $dadosPessoais['DSCATEGCERTRESE'] ?? null;
        $dadosPessoais['DTCERTRESEPES']   = $dadosPessoais['DTCERTRESEPES'] ?? null;
        $dadosPessoais['DTCERTRESEPES']   = !empty($dadosPessoais['DTCERTRESEPES']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTCERTRESEPES']) 
        : null;
        $dadosPessoais['CDEXCERTRESEPES']  = $dadosPessoais['CDEXCERTRESEPES'] ?? null;              
        $dadosPessoais['DTCTPSPESSOA']     = !empty($dadosPessoais['DTCTPSPESSOA']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTCTPSPESSOA']) 
        : null;
        $dadosPessoais['NRRACAPESSOA']    = $dadosPessoais['NRRACAPESSOA'] ?? null;
        $dadosPessoais['IDGRUPSANGUPES']  = $dadosPessoais['IDGRUPSANGUPES'] ?? null;
        $dadosPessoais['IDFATORRHPES']    = $dadosPessoais['IDFATORRHPES'] ?? null;
        $dadosPessoais['CDPAIS']          = $dadosPessoais['CDPAIS'] ?? null;
        $dadosPessoais['SGESTADO']        = $dadosPessoais['SGESTADO'] ?? null;
        $dadosPessoais['CDMUNICIPIO']     = $dadosPessoais['CDMUNICIPIO'] ?? null;
        $dadosPessoais['NRRGPESSOA']      = $dadosPessoais['NRRGPESSOA'] ?? null;
        $dadosPessoais['CDEXRGPESSOA']    = $dadosPessoais['CDEXRGPESSOA'] ?? null;
        $dadosPessoais['SGUFRGPESSOA']    = $dadosPessoais['SGUFRGPESSOA'] ?? null;
        $dadosPessoais['DSLOCALEXRGPES']  = $dadosPessoais['DSLOCALEXRGPES'] ?? null;
        $dadosPessoais['DTEXRGPESSOA']    = $dadosPessoais['DTEXRGPESSOA'] ?? null;
        $dadosPessoais['DTEXRGPESSOA']    = !empty($dadosPessoais['DTEXRGPESSOA']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTEXRGPESSOA']) 
        : null;
        $dadosPessoais['NRNACIONALID']    = $dadosPessoais['NRNACIONALID'] ?? null;
        $dadosPessoais['NRGRAUINSTR']     = $dadosPessoais['NRGRAUINSTR'] ?? null;
        $dadosPessoais['DSPRONOMETRAT']   = $dadosPessoais['DSPRONOMETRAT'] ?? null;
        $dadosPessoais['NRANOCHEGAPAIS']  = $dadosPessoais['NRANOCHEGAPAIS'] ?? null;
        $dadosPessoais['NRDOCUESTRANG']   = $dadosPessoais['NRDOCUESTRANG'] ?? null;
        $dadosPessoais['IDPESNATURALIZA'] = $dadosPessoais['IDPESNATURALIZA'] ?? null;
        $dadosPessoais['DTNATURALIZAPES'] = !empty($dadosPessoais['DTNATURALIZAPES']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTNATURALIZAPES']) 
        : null;
        $dadosPessoais['IDPOSVISTOPERM']  = $dadosPessoais['IDPOSVISTOPERM'] ?? null;
        $dadosPessoais['DTPERMAVISTO']    = !empty($dadosPessoais['DTPERMAVISTO']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTPERMAVISTO']) 
        : null;
        $dadosPessoais['NRCONSELHOREG']   = $dadosPessoais['NRCONSELHOREG'] ?? null;
        $dadosPessoais['NRINSCRCONSREG']  = $dadosPessoais['NRINSCRCONSREG'] ?? null;
        $dadosPessoais['DTEXCONSREG']     = !empty($dadosPessoais['DTEXCONSREG']) 
        ? DateTime::createFromFormat('d/m/Y', $dadosPessoais['DTEXCONSREG']) 
        : null;
        $dadosPessoais['SGUFCONSREG']     = $dadosPessoais['SGUFCONSREG'] ?? null;
        $dadosPessoais['CDTPBAIXAPAG']    = $dadosPessoais['CDTPBAIXAPAG'] ?? null;
        $dadosPessoais['NRLIVROCERTNASC'] = $dadosPessoais['NRLIVROCERTNASC'] ?? null;
        $dadosPessoais['DTMESCOMPETENC']  = $dadosPessoais['DTMESCOMPETENC'] ?? null;
        $dadosPessoais['NRORG']            = $dadosPessoais['NRORG'] ?? null;
        $dadosPessoais['CDOPERINCLUSAO']  = $dadosPessoais['CDOPERINCLUSAO'] ?? null;

        return $dadosPessoais;
    }


    private function updatePessoaHistorico($pessoaHistorico, $dadosPessoais)
    {   
        $this->entityManager->beginTransaction();
        try {
            $cdOperador = (string) $this->environment->getCdOperador();
            $nrOrg = (int) $this->environment->getNrOrg();
            $pessoaHistorico->setNmpessoa($dadosPessoais['NMPESSOA']);
            $pessoaHistorico->setDsapelidopess($dadosPessoais['DSAPELIDOPESS']);
            $pessoaHistorico->setNrcertnascpes($dadosPessoais['NRCERTNASCPES']);
            $pessoaHistorico->setDtnascpessoa($dadosPessoais['DTNASCPESSOA']);
            $pessoaHistorico->setIdsexopessoa($dadosPessoais['IDSEXOPESSOA']);
            $pessoaHistorico->setNrcondfispes($dadosPessoais['NRCONDFISPES']);
            $pessoaHistorico->setNrctpspessoa($dadosPessoais['NRCTPSPESSOA']);
            $pessoaHistorico->setNrseriectpspes($dadosPessoais['NRSERIECTPSPES']);
            $pessoaHistorico->setSgufctpspes($dadosPessoais['SGUFCTPSPES']);
            $pessoaHistorico->setDtctpspessoa($dadosPessoais['DTCTPSPESSOA']);
            $pessoaHistorico->setNrpispaseppes($dadosPessoais['NRPISPASEPPES']);
            $pessoaHistorico->setDtinscpispasep($dadosPessoais['DTINSCPISPASEP']);
            $pessoaHistorico->setNrinscinsspes($dadosPessoais['NRINSCINSSPES']);
            $pessoaHistorico->setNrinscisspes($dadosPessoais['NRINSCISSPES'] ) ;
            $pessoaHistorico->setCdestacivil($dadosPessoais['CDESTACIVIL']);
            $pessoaHistorico->setNrcpfpessoa($dadosPessoais['NRCPFPESSOA']);

            list($primeiroNome, $meioNome, $ultimoNome) = Utils::decompoeNomePessoa($dadosPessoais['NMPESSOA']);
            
            $pessoaHistorico->setDsprinomepess($primeiroNome);
            $pessoaHistorico->setDsnomemeiopess((strlen($meioNome)> 20) ? substr($meioNome, 0, 20) : $meioNome);
            $pessoaHistorico->setDsultnomepess($ultimoNome);
    
            $pessoaHistorico->setNrcertcasapes($dadosPessoais['NRCERTCASAPES']);
            $pessoaHistorico->setDtcasapessoa($dadosPessoais['DTCASAPESSOA']);
            $pessoaHistorico->setIdisentituelei($dadosPessoais['IDISENTITUELEI']);
            $pessoaHistorico->setNrtitueleipes($dadosPessoais['NRTITUELEIPES']);
            $pessoaHistorico->setNrsecaeleipes($dadosPessoais['NRSECAELEIPES']);
            $pessoaHistorico->setNrzonaeleipes($dadosPessoais['NRZONAELEIPES']);
    
            $pessoaHistorico->setNrcarthabpes($dadosPessoais['NRCARTHABPES'] );
            $pessoaHistorico->setDthabcartpes($dadosPessoais['DTHABCARTPES']);
            $pessoaHistorico->setDtvalhabcartpes($dadosPessoais['DTVALHABCARTPES']);
            $pessoaHistorico->setDscateghabcart($dadosPessoais['DSCATEGHABCART']);
            $pessoaHistorico->setNrcatmilitarpe($dadosPessoais['NRCATMILITARPE']);
            $pessoaHistorico->setNrcertresepes($dadosPessoais['NRCERTRESEPES']);
    
            $pessoaHistorico->setDscategcertrese($dadosPessoais['DSCATEGCERTRESE']);
            $pessoaHistorico->setDtcertresepes($dadosPessoais['DTCERTRESEPES']);
            $pessoaHistorico->setCdexcertresepes($dadosPessoais['CDEXCERTRESEPES']);
            $pessoaHistorico->setNrracapessoa($dadosPessoais['NRRACAPESSOA']);
            $pessoaHistorico->setIdgrupsangupes($dadosPessoais['IDGRUPSANGUPES']);
            $pessoaHistorico->setIdfatorrhpes($dadosPessoais['IDFATORRHPES']);
    
            $pessoaHistorico->setCdpais($dadosPessoais['CDPAIS']);
            $pessoaHistorico->setSgestado($dadosPessoais['SGESTADO']);
            $pessoaHistorico->setCdmunicipio($dadosPessoais['CDMUNICIPIO']);
            $pessoaHistorico->setNrrgpessoa($dadosPessoais['NRRGPESSOA']);
            $pessoaHistorico->setCdexrgpessoa($dadosPessoais['CDEXRGPESSOA']);
            $pessoaHistorico->setSgufrgpessoa($dadosPessoais['SGUFRGPESSOA']);
    
            $pessoaHistorico->setDslocalexrgpes($dadosPessoais['DSLOCALEXRGPES']);
            $pessoaHistorico->setDtexrgpessoa($dadosPessoais['DTEXRGPESSOA']);
            $pessoaHistorico->setNrnacionalid($dadosPessoais['NRNACIONALID']);
            $pessoaHistorico->setNrgrauinstr($dadosPessoais['NRGRAUINSTR']);
            $pessoaHistorico->setDspronometrat($dadosPessoais['DSPRONOMETRAT']);
            $pessoaHistorico->setNranochegapais($dadosPessoais['NRANOCHEGAPAIS']);
    
            $pessoaHistorico->setNrdocuestrang($dadosPessoais['NRDOCUESTRANG']);
            $pessoaHistorico->setIdpesnaturaliza($dadosPessoais['IDPESNATURALIZA']);
            $pessoaHistorico->setDtnaturalizapes($dadosPessoais['DTNATURALIZAPES']);
            $pessoaHistorico->setIdposvistoperm($dadosPessoais['IDPOSVISTOPERM']);
            $pessoaHistorico->setDtpermavisto($dadosPessoais['DTPERMAVISTO']);
            
            $pessoaHistorico->setNrconselhoreg($dadosPessoais['NRCONSELHOREG']);
            $pessoaHistorico->setNrinscrconsreg($dadosPessoais['NRINSCRCONSREG']);
            $pessoaHistorico->setDtexconsreg($dadosPessoais['DTEXCONSREG']);
            $pessoaHistorico->setSgufconsreg($dadosPessoais['SGUFCONSREG']);
            $pessoaHistorico->setCdtpbaixapag($dadosPessoais['CDTPBAIXAPAG']);
            $pessoaHistorico->setNrlivrocertnasc($dadosPessoais['NRLIVROCERTNASC']);
            
    
            $pessoaHistorico->setDtultatu(DateUtil::getDataAtual(false));
            $pessoaHistorico->setNrorgultatu(Utils::getParamters()->getNrorg());
            $pessoaHistorico->setCdoperultatu($cdOperador);
            
            $this->entityManager->persist($pessoaHistorico);
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        } 
    }


    private function setPessoaHistorico($nrpessoa, $dadosPessoais)
    {   
        $this->entityManager->beginTransaction();
        try {

            $pessoaHistorico = Repositories::GPE_PESSOAH;
            $novoHistorico = new $pessoaHistorico;

            list($primeiroNome, $meioNome, $ultimoNome) = Utils::decompoeNomePessoa($dadosPessoais['NMPESSOA']);
            $novoHistorico->setDsprinomepess($primeiroNome);
            $novoHistorico->setDsnomemeiopess((strlen($meioNome)> 20) ? substr($meioNome, 0, 20) : $meioNome);
            $novoHistorico->setDsultnomepess($ultimoNome);
            
            $novoHistorico->setNrpessoah(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_PESSOAH', 1, 12, true));
            $novoHistorico->setNmpessoa($dadosPessoais['NMPESSOA']);
            $novoHistorico->setDsapelidopess($dadosPessoais['DSAPELIDOPESS']);
            $novoHistorico->setNrcertnascpes($dadosPessoais['NRCERTNASCPES']);
            $novoHistorico->setDtnascpessoa($dadosPessoais['DTNASCPESSOA']);
            $novoHistorico->setIdsexopessoa($dadosPessoais['IDSEXOPESSOA']);
            $novoHistorico->setNrcondfispes($dadosPessoais['NRCONDFISPES']);
            $novoHistorico->setNrctpspessoa($dadosPessoais['NRCTPSPESSOA']);
            $novoHistorico->setNrcpfpessoa($dadosPessoais['NRCPFPESSOA']);
            
            $novoHistorico->setNrseriectpspes($dadosPessoais['NRSERIECTPSPES']);
            $novoHistorico->setSgufctpspes($dadosPessoais['SGUFCTPSPES']);
            $novoHistorico->setDtctpspessoa($dadosPessoais['DTCTPSPESSOA']);
            $novoHistorico->setNrpispaseppes($dadosPessoais['NRPISPASEPPES']);
            $novoHistorico->setDtinscpispasep($dadosPessoais['DTINSCPISPASEP']);
            $novoHistorico->setNrinscinsspes($dadosPessoais['NRINSCINSSPES']);
            $novoHistorico->setNrinscisspes($dadosPessoais['NRINSCISSPES'] ) ;
            $novoHistorico->setCdestacivil($dadosPessoais['CDESTACIVIL']);
    
            $novoHistorico->setNrcertcasapes($dadosPessoais['NRCERTCASAPES']);
            $novoHistorico->setDtcasapessoa($dadosPessoais['DTCASAPESSOA']);
            $novoHistorico->setIdisentituelei($dadosPessoais['IDISENTITUELEI']);
            $novoHistorico->setNrtitueleipes($dadosPessoais['NRTITUELEIPES']);
            $novoHistorico->setNrsecaeleipes($dadosPessoais['NRSECAELEIPES']);
            $novoHistorico->setNrzonaeleipes($dadosPessoais['NRZONAELEIPES']);
    
            $novoHistorico->setNrcarthabpes($dadosPessoais['NRCARTHABPES'] );
            $novoHistorico->setDthabcartpes($dadosPessoais['DTHABCARTPES']);
            $novoHistorico->setDtvalhabcartpes($dadosPessoais['DTVALHABCARTPES']);
            $novoHistorico->setDscateghabcart($dadosPessoais['DSCATEGHABCART']);
            $novoHistorico->setNrcatmilitarpe($dadosPessoais['NRCATMILITARPE']);
            $novoHistorico->setNrcertresepes($dadosPessoais['NRCERTRESEPES']);
    
            $novoHistorico->setDscategcertrese($dadosPessoais['DSCATEGCERTRESE']);
            $novoHistorico->setDtcertresepes($dadosPessoais['DTCERTRESEPES']);
            $novoHistorico->setCdexcertresepes($dadosPessoais['CDEXCERTRESEPES']);
            $novoHistorico->setNrracapessoa($dadosPessoais['NRRACAPESSOA']);
            $novoHistorico->setIdgrupsangupes($dadosPessoais['IDGRUPSANGUPES']);
            $novoHistorico->setIdfatorrhpes($dadosPessoais['IDFATORRHPES']);
    
            $novoHistorico->setCdpais($dadosPessoais['CDPAIS']);
            $novoHistorico->setSgestado($dadosPessoais['SGESTADO']);
            $novoHistorico->setCdmunicipio($dadosPessoais['CDMUNICIPIO']);
            $novoHistorico->setNrrgpessoa($dadosPessoais['NRRGPESSOA']);
            $novoHistorico->setCdexrgpessoa($dadosPessoais['CDEXRGPESSOA']);
            $novoHistorico->setSgufrgpessoa($dadosPessoais['SGUFRGPESSOA']);
    
            $novoHistorico->setDslocalexrgpes($dadosPessoais['DSLOCALEXRGPES']);
            $novoHistorico->setDtexrgpessoa($dadosPessoais['DTEXRGPESSOA']);
            $novoHistorico->setNrnacionalid($dadosPessoais['NRNACIONALID']);
            $novoHistorico->setNrgrauinstr($dadosPessoais['NRGRAUINSTR']);
            $novoHistorico->setDspronometrat($dadosPessoais['DSPRONOMETRAT']);
            $novoHistorico->setNranochegapais($dadosPessoais['NRANOCHEGAPAIS']);
    
            $novoHistorico->setNrdocuestrang($dadosPessoais['NRDOCUESTRANG']);
            $novoHistorico->setIdpesnaturaliza($dadosPessoais['IDPESNATURALIZA']);
            $novoHistorico->setDtnaturalizapes($dadosPessoais['DTNATURALIZAPES']);
            $novoHistorico->setIdposvistoperm($dadosPessoais['IDPOSVISTOPERM']);
            $novoHistorico->setDtpermavisto($dadosPessoais['DTPERMAVISTO']);
            
            $novoHistorico->setNrconselhoreg($dadosPessoais['NRCONSELHOREG']);
            $novoHistorico->setNrinscrconsreg($dadosPessoais['NRINSCRCONSREG']);
            $novoHistorico->setDtexconsreg($dadosPessoais['DTEXCONSREG']);
            $novoHistorico->setSgufconsreg($dadosPessoais['SGUFCONSREG']);
            $novoHistorico->setCdtpbaixapag($dadosPessoais['CDTPBAIXAPAG']);
            $novoHistorico->setNrlivrocertnasc($dadosPessoais['NRLIVROCERTNASC']);
            
    
           // $novoHistorico->setDtultatu($dadosPessoais['DTMESCOMPETENC']);
            //$novoHistorico->setNrorgultatu($dadosPessoais['NRORG']);
           // $novoHistorico->setCdoperultatu($dadosPessoais['CDOPERINCLUSAO']);

            $novoHistorico->setDtinclusao(DateUtil::getDataAtual(false));
            $novoHistorico->setNrorginclusao(Utils::getParamters()->getNrorg());
            $novoHistorico->setCdoperinclusao(Utils::getParamters()->getCdoperador());

            $novoHistorico->setNrpessoa($nrpessoa);
            $novoHistorico->setNrorg(Utils::getParamters()->getNrorg());
            $novoHistorico->setDtmescompetenc($dadosPessoais['DTMESCOMPETENC']);
            
            $this->entityManager->persist($novoHistorico);
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
      
    }
}