<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\RelatCustomRescisaoService;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class RelatCustomRescisaoController extends Crud {

    CONST operatorLikeAll = 'LIKE_ALL';
    CONST operatorLike = 'LIKE';

    public function __construct (Manager $dataSourceManager, HCMParameters $hcmParameters, RelatCustomRescisaoService $relatCustomRescisaoService){
        $this->relatCustomRescisaoService = $relatCustomRescisaoService;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getRelatCustomRescisao(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            
            $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'NRTPMODALIDCAL', 'NROCORRECAL', 'P_NRORG_PADRAO', 'NRTIPOHIERARQORG', 'CDMATRICULA', 'CDMATRICULAESOCIAL', 'NRESTRUTLEGAL', 'NRESTRUTGEREN', 'NREVENTOM');
            
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    if($filter['columnName'] === 'NROCORRECAL'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    unset($filters[$key]);
                }
                if($filter['operator'] == self::operatorLikeAll){
                    $filters[0]['operator'] = self::operatorLike;
                    $filters[0]['columnName'] = str_replace('|', '||', $filter['columnName']);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $parameters['NRTPMODALIDCAL'] = $this->hcmParameters->getTipoModalidadeCalculo();
            
            if( !isset($parameters['NROCORRECAL']) || empty($parameters['NROCORRECAL']) ) $parameters['NROCORRECAL'] = $this->hcmParameters->getOcorrenciaCalculo();
            
            // NREVENTOM Filter
            /*if (isset($parameters['NREVENTOM']) && gettype($parameters['NREVENTOM']) === 'string') {
                $parameters['NREVENTOM'] = explode(',', $parameters['NREVENTOM']);
            } else {
                unset($parameters['NREVENTOM']);
            }*/
            
            $result = $this->relatCustomRescisaoService->getRelatCustomRescisao($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRelatCustomRescisao', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
   
} 