<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AlteEscalaService;
use HCM\Geral\Services\ESocialService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;

class AlteEscalaController extends Crud {

    protected $dataSourceName = 'gpe_alteescala';
    protected $dataSourceNameGet = 'getAlteescala';
    protected $dataSourceNameMotivo = 'getMotivoEscala';
    

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_INCLUIVINCMES');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AlteEscalaService $alteEscalaService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->alteEscalaService = $alteEscalaService;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }

    public function getAlteescala (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setAlteescala (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrescalatrabm = $row['NRESCALATRABM'];
                $dtiniescala = $row['DTINIESCALA'];
                $dtfimescala = $row['DTFIMESCALA'];

                if ($row['__is_new']) {
                    // Valida se já existe escala para a mesma data
                    $this->alteEscalaService->beforeSave($nrorg, $nrvinculom, $dtiniescala);
                }
 
                if (!array_key_exists('NRTURNO', $row)) {
                    $row['NRTURNO'] = '';
                }
                if (!array_key_exists('NRHORDIAM', $row)) {
                    $row['NRHORDIAM'] = '';
                }

                if (array_key_exists('NRALTEESCALA', $row)) {
                    if ($row['NRALTEESCALA'] === '') {
                        $row['NRALTEESCALA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true);
                    }
                } else {
                    $row['NRALTEESCALA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true);
                }
                if(!array_key_exists('NRMOTIVOESCALA', $row)){
                    $row['NRMOTIVOESCALA'] = null;
                }
                parent::save($request, $response);

                $dataCompetencia = DateUtil::getDataDeString(substr(
                    $this->hcmParameters->getDtCompetencia(),
                    0,
                    10),
                    DateUtil::FORMATO_BRASILEIRO);
                $dataInicioEscala = DateUtil::setDay($row['DTINIESCALA'], 1);
                if ($row['__is_new'] || $dataInicioEscala > $dataCompetencia) {
                    if ($dataInicioEscala > $dataCompetencia) {
                        $escala = $dataInicioEscala;
                    }
                    if ($dataInicioEscala <= $dataCompetencia) {
                        $escala = $dataCompetencia;
                    }
                    $this->alteEscalaService->afterInsert($nrorg, $nrvinculom, $escala);
                }
                // Atualiza histórico do vínculo
                $verificaAtt = $this->alteEscalaService->afterSave($nrorg, $nrvinculom, $nrescalatrabm, substr($dtiniescala, 0, 10), substr($dtfimescala, 0, 10), true);
                
                if($verificaAtt) {
                    $response->addMessage(new DTO\Response\Message($verificaAtt, DTO\Response\Message::TYPE_MESSAGE));
                }else if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                } 
                
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteAlteescala(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
            $row = $request->getDataSet()->getRows()[0];
            // Atualiza histórico do vínculo de acordo com a escala anterior
            $this->alteEscalaService->afterDelete($row['NRORG'], $row['NRVINCULOM'], $row['DTINIESCALA']);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMotivoEscala (DTO\Request\Filter $request, DTO\Response $response){
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameMotivo);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function geraEscala (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
            $retornoEscala = $this->alteEscalaService->geraEscala($dataSet);  

            if($retornoEscala[1] === 1){
                $message = new \Zeedhi\Framework\DTO\Response\Message('Alterações de escala geradas com sucesso!', DTO\Response\Message::TYPE_MESSAGE);
            } else if($retornoEscala[1] === 0){
                $message = new \Zeedhi\Framework\DTO\Response\Message('Os seguintes vínculos não tiveram alterações de escala geradas!', DTO\Response\Message::TYPE_WARNING);
            }
            
            unset($retornoEscala[1]);
            if (count($retornoEscala[0]) !== 0)
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('geraEscala', $retornoEscala[0]));
                
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message)
                $response->addMessage($message);
                
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}