<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Services\IntegracaoDigitalCertisignService;

class IntegracaoDigitalCertisignController extends Crud {
    /** @var IntegracaoDigitalCertisignService */
    protected $certisignService;


    protected $dataSourceName            = 'getGpe_inteletrdigit';
    protected $setDataSourceName         = 'gpe_inteletrdigit';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (
        Manager $dataSourceManager,
        ParameterBag $parameterBag,
        HCMParameters $hcmParameters,
        EntityManager $entityManager,
        IntegracaoDigitalCertisignService $certisignService
    ){
        $this->parameterBag       = $parameterBag;
        $this->hcmParameters      = $hcmParameters;
        $this->entityManager      = $entityManager;
        $this->pessoa             = Repositories::GPE_PESSOA;
        $this->certisignService   = $certisignService;
        parent::__construct($dataSourceManager);
    }

    public function getAssinaturaEletronica(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setAssinaturaEletronica(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->setDataSourceName;
        try {
            $row = $request->getDataSet()->getRows()[0];
            $row = $row->getArrayCopy();

            if ($row["__is_new"]) {
                $this->certisignService->saveAssinaturaEletronica($row);
            } else {
                $this->certisignService->editAssinaturaEletronica($row);
            }

            $dataset = [
                'success' => true,
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function deleteAssinaturaEletronica(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $row = $row->getArrayCopy();
            $this->certisignService->deleteAssinaturaEletronica($row);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function downloadAssinaturaEletronica(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow()->getArrayCopy();
            $file = $this->certisignService->downloadAssinaturaEletronica($row);

            $response->addDataSet(new DataSet('data', $file));
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 