CREATE OR REPLACE PROCEDURE CTB_DFS_CONTAB(
   P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_NRLANCTONF   IN NOTAFISCAL.NRLANCTONF%TYPE,
   P_NRLANCCTB    OUT LANCCONT.NRLANCCONT%TYPE)
AS
   V_NRLANCTEMP VARCHAR2(20);
   V_DSHISTLANC LANCCONT.DSHISTLANC%TYPE;
   V_COUNT INT;
   V_NOTADEVOLUCAO VARCHAR(01);
   V_VRLANCTO NUMBER;
   V_VRTEMP NUMBER;
   V_NUMPARTIDA INT;
   V_IDCOME VARCHAR2(01);
   V_IDCCUS VARCHAR2(01);
   V_CDHISTPADR HISTPADR.CDHISTPADR%TYPE;
   V_VRSEQLANC LANCCONT.NRSEQLANC%TYPE;
   V_CDCENTCUST CENTCUST.CDCENTCUST%TYPE;
   V_CONTCCUS VARCHAR2(01);
   V_CDCOMERCIAL FORNECEDOR.CDFORNECED%TYPE;
   V_NRSEQLANC LANCCONT.NRSEQLANC%TYPE;
   V_CDCONTCTBL CONTCTBL.CDCONTCTBL%TYPE;
   V_ISEMPTY VARCHAR2(01);
   V_CDIMPSUBST VARCHAR2(02);

   PROCEDURE NUMEROPARTIDA(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                           P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                           P_CDFORNECED IN FORNECEDOR.CDFORNECED%TYPE,
                           P_CDCLIENTE IN CLIENTE.CDCLIENTE%TYPE)
   AS
   BEGIN
      IF V_NUMPARTIDA IS NULL THEN
         /* NUMERO DA PARTIDA */
         BEGIN
            SELECT TO_NUMBER(MAX(NRSEQCTBTEMP))
              INTO V_NUMPARTIDA
              FROM LANCCTBTEMP
             WHERE NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDEMPRESA = P_CDEMPRESA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NUMPARTIDA := 0;
         END;

         /* CONTROLA SE GERA OS RATEIOS */
         SELECT IDCONTGERE
           INTO V_CONTCCUS
           FROM CONTOPECTBL
          WHERE CDEMPRESA = P_CDEMPRESA;

         /* BUSCA O CODIGO COMERCIAL */
         BEGIN
            SELECT DECODE(P_IDORIGDESTNF, 'F', P_CDFORNECED, DECODE(P_IDORIGDESTNF, 'C', P_CDCLIENTE, NULL))
              INTO V_CDCOMERCIAL
              FROM DUAL;
          EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDCOMERCIAL := NULL;
         END;
      END IF;
   END;

   PROCEDURE CONTABILIZA_DESCONTO(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                                  P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                                  P_CDFILIEMIT IN FILIAL.CDFILIAL%TYPE,
                                  P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                                  P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                                  P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                                  P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                                  P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                                  P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                                  P_DTEMISSAO IN NOTAFISCAL.DTENTRSAID%TYPE)
   AS
   BEGIN
      NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);
      V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIEMIT, P_CDEMPRESA);

      /*-------------------------------------------------------------*/
      /* FAZ O SOMATORIO DOS DESCONTOS DA NOTA FISCAL                */
      /*-------------------------------------------------------------*/
      FOR C_DESC IN
         (SELECT ROUND(SUM(VRDESCITEM),2) AS VRITEMENCA, CDCTBLSAIDEN
            FROM ITEMPRODUTO I,
                 (SELECT CDCONTDITSA AS CDCTBLSAIDEN
                    FROM PARAMCTBL
                   WHERE CDCONTDITEN IS NOT NULL) E
            WHERE I.CDFILIAL   = P_CDFILIAL   AND
                  I.NRLANCTONF = P_NRLANCTONF
            GROUP BY CDCTBLSAIDEN
            HAVING ROUND(SUM(VRDESCITEM),2) > 0)LOOP
         BUSCA_TPCOMERCIAL(C_DESC.CDCTBLSAIDEN, V_IDCOME, V_IDCCUS);

         BEGIN
            SELECT COUNT(*), NRSEQCTBTEMP
              INTO V_COUNT, V_NRSEQLANC
              FROM LANCCTBTEMP
             WHERE CDEMPRESA = P_CDEMPRESA
               AND NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDCONTCTBL = C_DESC.CDCTBLSAIDEN
             GROUP BY NRSEQCTBTEMP;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            V_NUMPARTIDA := V_NUMPARTIDA + 1;
            INSERT INTO LANCCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
            VALUES
               (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                C_DESC.CDCTBLSAIDEN, V_CDHISTPADR, V_DSHISTLANC, P_DTEMISSAO, C_DESC.VRITEMENCA * -1, V_IDCOME, V_CDCOMERCIAL, 'DFS');

            IF V_CONTCCUS <> 'N' THEN
               IF V_IDCCUS <> 'O' THEN
                  INSERT INTO RATECTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                      CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                  VALUES
                     (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                      V_CDCENTCUST, C_DESC.VRITEMENCA * -1, P_DTEMISSAO, '00001');
               END IF;
            END IF;
         ELSE
            UPDATE LANCCTBTEMP
               SET VRLANCCTBTEMP = VRLANCCTBTEMP + (C_DESC.VRITEMENCA * -1)
             WHERE CDEMPRESA = P_CDEMPRESA
               AND NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDCONTCTBL = C_DESC.CDCTBLSAIDEN;
            IF V_CONTCCUS <> 'N' THEN
               IF V_IDCCUS <> 'O' THEN
                  BEGIN
                     SELECT COUNT(*), L.NRSEQCTBTEMP
                       INTO V_COUNT, V_NRSEQLANC
                       FROM LANCCTBTEMP L, RATECTBTEMP R
                      WHERE L.CDEMPRESA = P_CDEMPRESA
                        AND L.NRLANCCTBTEMP = V_NRLANCTEMP
                        AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                        AND L.CDEMPRESA = R.CDEMPRESA
                        AND L.CDCONTCTBL = C_DESC.CDCTBLSAIDEN
                        AND L.NRLANCCTBTEMP = L.NRLANCCTBTEMP
                        AND R.CDFILIAL = P_CDFILIEMIT
                        AND R.CDCENTCUST = V_CDCENTCUST
                      GROUP BY L.NRSEQCTBTEMP;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_COUNT := 0;
                        V_NRSEQLANC := NULL;
                  END;

                  IF V_COUNT = 0 THEN
                     INSERT INTO RATECTBTEMP
                        (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                         CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                     VALUES
                        (P_CDEMPRESA, V_NRLANCTEMP, V_NRSEQLANC, P_CDFILIEMIT,
                         V_CDCENTCUST, C_DESC.VRITEMENCA * -1, P_DTEMISSAO, '00001');
                  ELSE
                     UPDATE RATECTBTEMP
                        SET VRRATECTBTEMP = VRRATECTBTEMP + (C_DESC.VRITEMENCA * -1)
                      WHERE CDEMPRESA = P_CDEMPRESA
                        AND NRLANCCTBTEMP = V_NRLANCTEMP
                        AND NRSEQCTBTEMP = V_NRSEQLANC
                        AND CDFILIAL = P_CDFILIEMIT
                        AND CDCENTCUST = V_CDCENTCUST;
                  END IF;
               END IF;
            END IF;
         END IF;
      END LOOP;
   END;

   PROCEDURE CONTABILIZA_ENCARGO(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                                 P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                                 P_CDFILIEMIT IN FILIAL.CDFILIAL%TYPE,
                                 P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                                 P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                                 P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                                 P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                                 P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                                 P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                                 P_DTEMISSAO IN NOTAFISCAL.DTENTRSAID%TYPE)

   AS
   BEGIN
      FOR C_ENC IN
        (SELECT VRITEMENCA, E.IDENCARGO, CDCTBLSAIDEN
          FROM ITEMENCARGO IE, ENCARGO E
         WHERE IE.CDFILIAL   = P_CDFILIAL
           AND IE.NRLANCTONF = P_NRLANCTONF
           AND IE.CDENCARGO  = E.CDENCARGO
           AND IE.VRITEMENCA > 0
           AND E.IDENCARGO IN ('A','D')
           AND (DECODE(P_IDENTRSAID, 'E', CDCTBLENTREN, CDCTBLSAIDEN) IS NOT NULL)) LOOP
         BUSCA_TPCOMERCIAL(C_ENC.CDCTBLSAIDEN, V_IDCOME, V_IDCCUS);
         V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIEMIT, P_CDEMPRESA);
         IF C_ENC.IDENCARGO = 'A' THEN
            V_VRTEMP :=  C_ENC.VRITEMENCA;
         ELSE
            V_VRTEMP := - C_ENC.VRITEMENCA;
         END IF;

         NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);

         IF C_ENC.CDCTBLSAIDEN IS NOT NULL THEN
            BEGIN
               SELECT COUNT(*)
                 INTO V_COUNT
                 FROM LANCCTBTEMP
                WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDCONTCTBL = C_ENC.CDCTBLSAIDEN;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_COUNT := 0;
            END;
            IF V_COUNT = 0 THEN
               V_NUMPARTIDA := V_NUMPARTIDA + 1;
               INSERT INTO LANCCTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                   CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                   VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
               VALUES
                  (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                   C_ENC.CDCTBLSAIDEN, V_CDHISTPADR, V_DSHISTLANC, P_DTEMISSAO, V_VRTEMP, V_IDCOME, V_CDCOMERCIAL, 'DFS');

               IF V_CONTCCUS <> 'N' THEN
                  IF V_IDCCUS <> 'O' THEN
                     INSERT INTO RATECTBTEMP
                        (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                         CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                     VALUES
                        (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                         V_CDCENTCUST, V_VRTEMP, P_DTEMISSAO, '00001');
                  END IF;
               END IF;
            ELSE
               UPDATE LANCCTBTEMP
                  SET VRLANCCTBTEMP = VRLANCCTBTEMP + V_VRTEMP
                WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDCONTCTBL = C_ENC.CDCTBLSAIDEN;
               IF V_CONTCCUS <> 'N' THEN
                  IF V_IDCCUS <> 'O' THEN
                     BEGIN
                        SELECT COUNT(*), L.NRSEQCTBTEMP
                          INTO V_COUNT, V_NRSEQLANC
                          FROM LANCCTBTEMP L, RATECTBTEMP R
                         WHERE L.NRLANCCTBTEMP = V_NRLANCTEMP
                           AND L.CDEMPRESA = P_CDEMPRESA
                           AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                           AND L.CDEMPRESA = R.CDEMPRESA
                           AND L.CDCONTCTBL = C_ENC.CDCTBLSAIDEN
                           AND R.CDFILIAL = P_CDFILIEMIT
                           AND R.CDCENTCUST = V_CDCENTCUST
                         GROUP BY L.NRSEQCTBTEMP;
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_COUNT := 0;
                           V_NRSEQLANC := NULL;
                     END;

                     IF V_COUNT = 0 THEN
                        INSERT INTO RATECTBTEMP
                           (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                            CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                        VALUES
                           (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                            V_CDCENTCUST, V_VRTEMP, P_DTEMISSAO, '00001');
                     ELSE
                        UPDATE RATECTBTEMP
                           SET VRRATECTBTEMP = VRRATECTBTEMP + V_VRTEMP
                         WHERE CDEMPRESA = P_CDEMPRESA
                           AND NRLANCCTBTEMP = V_NRLANCTEMP
                           AND NRSEQCTBTEMP = V_NRSEQLANC
                           AND CDFILIAL = P_CDFILIEMIT
                           AND CDCENTCUST = V_CDCENTCUST;
                     END IF;
                  END IF;
               END IF;
            END IF;
         ELSE
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (P_CDEMPRESA, V_NRLANCTEMP, '', '015');
         END IF;
      END LOOP;
   END;

   PROCEDURE DIFALIQDEVOLUCAO(P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
                              P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                              P_CDFILIEMIT IN FILIAL.CDFILIAL%TYPE,
                              P_NRLANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,
                              P_NRNOTAFISC IN NOTAFISCAL.NRNOTAFISC%TYPE,
                              P_IDENTRSAID IN NOTAFISCAL.IDENTRSAIDOP%TYPE,
                              P_IDORIGDESTNF IN NOTAFISCAL.IDORIGDESTNF%TYPE,
                              P_CDFORNECED IN NOTAFISCAL.CDFORNECED%TYPE,
                              P_CDCLIENTE IN NOTAFISCAL.CDCLIENTE%TYPE,
                              P_DTEMISSAO IN NOTAFISCAL.DTENTRSAID%TYPE)
   AS
      V_VRDIFALIQDEV FLOAT;
   BEGIN
      NUMEROPARTIDA(P_CDEMPRESA, P_IDORIGDESTNF, P_CDFORNECED, P_CDCLIENTE);
      V_CDCENTCUST := BUSCA_CCUSTO(P_CDFILIEMIT, P_CDEMPRESA);
      V_VRDIFALIQDEV := 0;
      FOR C_DIFALIQ IN
         (SELECT (ROUND(IMP.VRIMPOSTOIT, 2) * -1) VRIMPOSTOIT,
                 CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCCTBLPASS
            FROM NOTAFISCAL NF, ITEMPROIMPOS IMP, ITEMPRODUTO IT, IMPOSTO I
           WHERE IMP.CDFILIAL    = P_CDFILIAL      AND
                 IMP.NRLANCTONF  = P_NRLANCTONF    AND
                 IMP.CDFILIAL    = NF.CDFILIAL    AND
                 IMP.NRLANCTONF  = NF.NRLANCTONF  AND
                 IMP.NRSEQITPROD = IT.NRSEQITPROD AND
                 IMP.NRLANCTONF  = IT.NRLANCTONF  AND
                 IMP.CDFILIAL    = IT.CDFILIAL    AND
                 IMP.CDIMPOSTO   = I.CDIMPOSTO    AND
                 IMP.CDFISCALIT  IN ('2', '3')    AND
                 IMP.NRITPROIMPOS = '3'           AND
                 I.IDTIPCONTAB   = 'D'            AND
                 IDIMPORECUP     = 'S'            AND
                 IDDIFALIQCUSPRI = 'S') LOOP
         BUSCA_TPCOMERCIAL(C_DIFALIQ.CDCCTBLPASS, V_IDCOME, V_IDCCUS);

          BEGIN
             SELECT COUNT(*), NRSEQCTBTEMP
               INTO V_COUNT, V_NRSEQLANC
               FROM LANCCTBTEMP
              WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                AND CDEMPRESA = P_CDEMPRESA
                AND CDCONTCTBL = C_DIFALIQ.CDCCTBLPASS
               GROUP BY NRSEQCTBTEMP;
          EXCEPTION
             WHEN NO_DATA_FOUND THEN
                V_COUNT := 0;
                V_NRSEQLANC := NULL;
          END;

          IF C_DIFALIQ.VRIMPOSTOIT <> 0 THEN
             IF V_COUNT = 0 THEN
                V_NUMPARTIDA := V_NUMPARTIDA + 1;
                INSERT INTO LANCCTBTEMP
                   (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                    CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                    VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
                VALUES
                   (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                    C_DIFALIQ.CDCCTBLPASS, V_CDHISTPADR, V_DSHISTLANC, P_DTEMISSAO, C_DIFALIQ.VRIMPOSTOIT, V_IDCOME, V_CDCOMERCIAL, 'DFS');
                IF V_CONTCCUS <> 'N' THEN
                   IF V_IDCCUS <> 'O' THEN
                      INSERT INTO RATECTBTEMP
                         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                          CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                      VALUES
                         (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                          V_CDCENTCUST, C_DIFALIQ.VRIMPOSTOIT, P_DTEMISSAO, '00001');
                   END IF;
                END IF;
             ELSE
                UPDATE LANCCTBTEMP
                   SET VRLANCCTBTEMP = VRLANCCTBTEMP + C_DIFALIQ.VRIMPOSTOIT
                 WHERE CDEMPRESA = P_CDEMPRESA
                   AND NRLANCCTBTEMP = V_NRLANCTEMP
                   AND CDCONTCTBL = C_DIFALIQ.CDCCTBLPASS;

                BEGIN
                   SELECT COUNT(*), L.NRSEQCTBTEMP
                     INTO V_COUNT, V_NRSEQLANC
                     FROM LANCCTBTEMP L, RATECTBTEMP R
                    WHERE L.NRLANCCTBTEMP = V_NRLANCTEMP
                      AND L.CDEMPRESA = P_CDEMPRESA
                      AND L.CDCONTCTBL = C_DIFALIQ.CDCCTBLPASS
                      AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                      AND L.NRSEQCTBTEMP = R.NRSEQCTBTEMP
                      AND L.CDEMPRESA = R.CDEMPRESA
                      AND R.CDFILIAL = P_CDFILIEMIT
                      AND R.CDCENTCUST = V_CDCENTCUST;
                EXCEPTION
                   WHEN NO_DATA_FOUND THEN
                       V_COUNT := 0;
                       V_NRSEQLANC := NULL;
                END;

                IF V_COUNT = 0 THEN
                   INSERT INTO RATECTBTEMP
                      (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                       CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                   VALUES
                      (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                       V_CDCENTCUST, C_DIFALIQ.VRIMPOSTOIT, P_DTEMISSAO, '00001');
                ELSE
                   UPDATE RATECTBTEMP
                      SET VRRATECTBTEMP = VRRATECTBTEMP + C_DIFALIQ.VRIMPOSTOIT
                    WHERE CDEMPRESA = P_CDEMPRESA
                      AND NRLANCCTBTEMP = V_NRLANCTEMP
                      AND NRSEQCTBTEMP = V_NRSEQLANC
                      AND CDFILIAL = P_CDFILIEMIT
                      AND CDCENTCUST = V_CDCENTCUST;
                END IF;
             END IF;
          END IF;
      END LOOP;

      FOR C_DIFALIQSCOMPOR IN
      (SELECT (ROUND(IMP.VRIMPOSTOIT, 2) * -1) VRIMPOSTOIT,
              CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCCTBLPASS
         FROM NOTAFISCAL NF, ITEMPROIMPOS IMP, ITEMPRODUTO IT, IMPOSTO I
        WHERE IMP.CDFILIAL    = P_CDFILIAL      AND
              IMP.NRLANCTONF  = P_NRLANCTONF    AND
              IMP.CDFILIAL    = NF.CDFILIAL    AND
              IMP.NRLANCTONF  = NF.NRLANCTONF  AND
              IMP.NRSEQITPROD = IT.NRSEQITPROD AND
              IMP.NRLANCTONF  = IT.NRLANCTONF  AND
              IMP.CDFILIAL    = IT.CDFILIAL    AND
              IMP.CDIMPOSTO   = I.CDIMPOSTO    AND
              IMP.CDFISCALIT  IN ('2', '3')    AND
              IMP.NRITPROIMPOS = '3'           AND
              I.IDTIPCONTAB   = 'D'            AND
              IDIMPORECUP     = 'S'            AND
              IDDIFALIQCUSPRI = 'N')LOOP
         IF C_DIFALIQSCOMPOR.VRIMPOSTOIT <> 0 THEN
            BUSCA_TPCOMERCIAL(C_DIFALIQSCOMPOR.CDCCTBLPASS, V_IDCOME, V_IDCCUS);
            BEGIN
               SELECT COUNT(*), NRSEQCTBTEMP
                 INTO V_COUNT, V_NRSEQLANC
                 FROM LANCCTBTEMP
                WHERE NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDCONTCTBL = C_DIFALIQSCOMPOR.CDCCTBLPASS
                 GROUP BY NRSEQCTBTEMP;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_COUNT := 0;
                  V_NRSEQLANC := NULL;
            END;

            IF V_COUNT = 0 THEN
               V_NUMPARTIDA := V_NUMPARTIDA + 1;
               INSERT INTO LANCCTBTEMP
                  (CDEMPRESA, NRLANCCTBTEMP, CDDOCUTEMP, NRSEQCTBTEMP,
                   CDCONTCTBL, CDHISTPADR,    DSHISTCTB, DTLANCCTBTEMP,
                   VRLANCCTBTEMP, IDLANCCTBCOME, CDLANCCTBCOME, IDSISTEMA)
               VALUES
                  (P_CDEMPRESA, V_NRLANCTEMP, P_NRNOTAFISC, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'),
                   C_DIFALIQSCOMPOR.CDCCTBLPASS, V_CDHISTPADR, V_DSHISTLANC, P_DTEMISSAO, C_DIFALIQSCOMPOR.VRIMPOSTOIT, V_IDCOME, V_CDCOMERCIAL, 'DFS');
               IF V_CONTCCUS <> 'N' THEN
                  IF V_IDCCUS <> 'O' THEN
                     INSERT INTO RATECTBTEMP
                        (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                         CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                     VALUES
                        (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                         V_CDCENTCUST, C_DIFALIQSCOMPOR.VRIMPOSTOIT, P_DTEMISSAO, '00001');
                  END IF;
               END IF;
            ELSE
               UPDATE LANCCTBTEMP
                  SET VRLANCCTBTEMP = VRLANCCTBTEMP + C_DIFALIQSCOMPOR.VRIMPOSTOIT
                WHERE CDEMPRESA = P_CDEMPRESA
                  AND NRLANCCTBTEMP = V_NRLANCTEMP
                  AND CDCONTCTBL = C_DIFALIQSCOMPOR.CDCCTBLPASS;

               BEGIN
                  SELECT COUNT(*), L.NRSEQCTBTEMP
                    INTO V_COUNT, V_NRSEQLANC
                    FROM LANCCTBTEMP L, RATECTBTEMP R
                   WHERE L.NRLANCCTBTEMP = V_NRLANCTEMP
                     AND L.CDEMPRESA = P_CDEMPRESA
                     AND L.CDCONTCTBL = C_DIFALIQSCOMPOR.CDCCTBLPASS
                     AND L.NRLANCCTBTEMP = R.NRLANCCTBTEMP
                     AND L.NRSEQCTBTEMP = R.NRSEQCTBTEMP
                     AND L.CDEMPRESA = R.CDEMPRESA
                     AND R.CDFILIAL = P_CDFILIEMIT
                     AND R.CDCENTCUST = V_CDCENTCUST;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                      V_COUNT := 0;
                      V_NRSEQLANC := NULL;
               END;

               IF V_COUNT = 0 THEN
                  INSERT INTO RATECTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDFILIAL,
                      CDCENTCUST, VRRATECTBTEMP, DTCOMPCTBTEMP, NRSEQRATTEMP)
                  VALUES
                     (P_CDEMPRESA, V_NRLANCTEMP, LPAD(TO_CHAR(V_NUMPARTIDA),5,'0'), P_CDFILIEMIT,
                      V_CDCENTCUST, C_DIFALIQSCOMPOR.VRIMPOSTOIT, P_DTEMISSAO, '00001');
               ELSE
                  UPDATE RATECTBTEMP
                     SET VRRATECTBTEMP = VRRATECTBTEMP + C_DIFALIQSCOMPOR.VRIMPOSTOIT
                   WHERE CDEMPRESA = P_CDEMPRESA
                     AND NRLANCCTBTEMP = V_NRLANCTEMP
                     AND NRSEQCTBTEMP = V_NRSEQLANC
                     AND CDFILIAL = P_CDFILIEMIT
                     AND CDCENTCUST = V_CDCENTCUST;
               END IF;
            END IF;
         END IF;
      END LOOP;
   END;
BEGIN
   V_VRLANCTO := 0;
   V_VRTEMP := 0;
   FOR C_NOTAFISCAL IN
      (SELECT NF.CDFILIAL,      NF.NRLANCTONF,    NF.CDEMPRESA,
              NF.IDDOCFINANC,   NF.NRNOTAFISC,    NF.CDCLIENTE,
              NF.CDFORNECED,    NF.NRLANCTBDOC,   NF.VRNOTAFISC,
              NF.CDFILIEMIT,    NF.CDTIPOOPER,    TP.IDCONTABOPER,
              TP.IDOPERDEVONF,  NF.IDORIGDESTNF,  NF.VRPRODUTOS,
              NF.CDFILIRECE,    NF.DTENTRSAID,    NF.CDINSCESTA,
              NF.DTEMISSAO,     C.IDPRIPUBCLIE,   NF.IDENTRSAIDOP,
              NVL(FI.CDCENTCUST, EP.CDCENTCUST) AS CDCENTCUST,
              TP.CDTPCTPAGAR AS CDTPCTPAGOP, TP.CDTPCTRECEB AS CDTPCTRECOP,
              TC.CDCLASSFINA AS CDCLAFINOP, FO.DTINIVRPDESC, NF.CDSERIEENTR,
              NF.CDSERIE, NF.IDTPIJURNF, NF.NRINSJURNF, TP.IDDTCONTAB,
              TP.IDCOAVRIMPVRPROD, TP.IDCONTSOMENIMPO, TP.IDCONTCONFORIG,
              TP.IDDESCIMPRECCOFI
         FROM NOTAFISCAL NF, FILIAL FI, EMPRESA EP,
              TIPOOPERAC TP, TIPOCTPAGAR TC, FORNECEDOR FO, CLIENTE C
        WHERE NF.CDFILIAL     = P_CDFILIAL        AND
              NF.NRLANCTONF   = P_NRLANCTONF      AND
              NF.CDEMPRESA    = EP.CDEMPRESA      AND
              NF.IDENTRSAIDOP = TP.IDENTRSAIDOP   AND
              NF.CDTIPOOPER   = TP.CDTIPOOPER     AND
              TP.CDTPCTPAGAR  = TC.CDTPCTPAGAR(+) AND
              DECODE(NF.IDENTRSAIDOP,'E', NF.CDFILIRECE, NF.CDFILIEMIT) = FI.CDFILIAL AND
              NF.CDFORNECED   = FO.CDFORNECED(+)  AND
              NF.CDCLIENTE    = C.CDCLIENTE(+))LOOP
      V_NRLANCTEMP := C_NOTAFISCAL.NRLANCTBDOC;
      IF C_NOTAFISCAL.IDCONTABOPER <> 'N' THEN
         IF V_NRLANCTEMP IS NULL THEN
            /* PEGA UM NOVO NMERO DE LANAMENTO CONTBIL, CASO O LANAMENTO SEJA NOVO */
            NOVO_CODIGO_NT('LANCCONTABIL' || C_NOTAFISCAL.CDEMPRESA || TO_CHAR(C_NOTAFISCAL.DTEMISSAO,'YYYYMM'), V_NRLANCTEMP);
            V_NRLANCTEMP := TO_CHAR(C_NOTAFISCAL.DTEMISSAO,'YYYYMM')||SUBSTR(V_NRLANCTEMP, 15, 6);
         END IF;

         /* DELETA O ANTIGO LANAMENTO CONTBIL PARA REFAZE-LO */
         DELETE
           FROM LANCCCUSTO
          WHERE NRLANCCONT = V_NRLANCTEMP
            AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;

         DELETE
           FROM LANCCONT
          WHERE NRLANCCONT = V_NRLANCTEMP
            AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;

         DELETE
           FROM RATECTBTEMP
          WHERE NRLANCCTBTEMP = V_NRLANCTEMP
            AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;

         DELETE
           FROM LANCCTBTEMP
          WHERE NRLANCCTBTEMP = V_NRLANCTEMP
            AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;

         V_VRLANCTO := - ROUND(C_NOTAFISCAL.VRNOTAFISC, 2);


         IF C_NOTAFISCAL.IDCONTSOMENIMPO = 'N' THEN
            /* Os impostos a reter na fonte devem ser retirados
            no valor da partida referente  nota. Porque quando a nota de devoluo
            possui uma parametrizao de incidncia que seja diferente da nota de origem, esse valor
            no ter imposto a reter na fonte. */

            V_NOTADEVOLUCAO := 'N';
            FOR C_IMPOSTODEV IN
               (SELECT IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPSTII IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                       ROUND(SUM(IMP.VRIMPOSTOIT),    2) AS VRIMPOSTONF,
                       0 AS VRIMPCARGATRIBI,
                       CDCONTATIV,
                       CDCONTPASS,
                       CDCONTDESP, IMP.CDFISCALIT CDFISCALNF, I.CDCCTBLATIV CDCONTATIVO
                  FROM ITEMPROIMPOSORIG IMP, NOTAFISCAL NF, IMPOSTO I, PARAMFILIAL PF,
                       (SELECT DISTINCT CDIMPOSTO,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTATIV,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTPASS,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTDESP
                          FROM ITEMPROIMPOSORIG I, NOTAFISCAL NF
                         WHERE I.CDFILIAL   = NF.CDFILIAL
                           AND I.NRLANCTONF = NF.NRLANCTONF
                           AND I.CDFILIAL   = C_NOTAFISCAL.CDFILIAL
                           AND I.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF) CT
                 WHERE IMP.VRIMPOSTOIT > 0
                   AND IMP.CDIMPOSTO = CT.CDIMPOSTO
                   AND IMP.CDIMPOSTO = I.CDIMPOSTO
                   AND ((IMP.NRITPROIMPOS IN ('1','4')) OR (IMP.NRITPROIMPOS IS NULL))
                   AND I.IDTIPCONTAB  = 'D'
                   AND ((((I.IDIMPORECUP  = 'S'  AND IMP.CDFISCALIT = '1' AND PF.IDCARACFIS  = 'C') OR (PF.IDCARACFIS = 'A')) AND (CT.CDCONTATIV IS NOT NULL))
                       OR
                       ((I.IDIMPORECUP <> 'S') AND (CT.CDCONTPASS IS NOT NULL)))
                   AND NF.CDFILIAL     = C_NOTAFISCAL.CDFILIAL
                   AND NF.NRLANCTONF   = C_NOTAFISCAL.NRLANCTONF
                   AND NF.CDFILIEMIT   = PF.CDFILIAL
                   AND NF.CDFILIAL     = IMP.CDFILIAL
                   AND NF.NRLANCTONF   = IMP.NRLANCTONF
                   AND EXISTS (SELECT 1
                                 FROM (SELECT  A.CDIMPOSTO, A.CDFILIAL, A.NRLANCTONF, A.CDFISCALNF
                                         FROM (SELECT INF.CDIMPOSTO, VRIMPOSTONF, NVL(IDACRPRODENTRNF, 'N') IDACRPRODENTRNF,
                                                      PF.IDCARACFIS, I.IDIMPORECUP, I.IDTRATIMPO, INF.CDFISCALNF,
                                                      CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTATIV,
                                                      CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTPASS,
                                                      CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTDESP,
                                                      CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'E', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTIMPO,
                                                      NF.NRLANCTONF, NF.CDFILIAL
                                                FROM IMPOSTONF INF, IMPOSTO I, NOTAFISCAL NF, PARAMFILIAL PF, ITEMNOTAFISC IT
                                               WHERE NF.CDFILIAL     = IT.CDFILIALIT
                                                 AND NF.NRLANCTONF   = IT.NRLANCTONFIT
                                                 AND PF.CDFILIAL(+)  = NF.CDFILIRECE
                                                 AND INF.CDFILIAL    = IT.CDFILIALIT
                                                 AND INF.NRLANCTONF  = IT.NRLANCTONFIT
                                                 AND IT.CDFILIAL = C_NOTAFISCAL.CDFILIAL
                                                 AND IT.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                                                 AND INF.CDIMPOSTO   = I.CDIMPOSTO
                                                 AND I.IDTIPCONTAB  = 'D') A
                                       WHERE ((((A.IDIMPORECUP  = 'S' AND A.CDFISCALNF = '1' AND NVL(A.IDCARACFIS,'C') = 'C') OR (A.IDCARACFIS = 'A')) AND (A.CDCONTATIV IS NOT NULL)) OR
                                              ((A.IDIMPORECUP <> 'S') AND (A.CDCONTPASS IS NOT NULL)))) IMPORIGEM
                              WHERE IMP.CDIMPOSTO = IMPORIGEM.CDIMPOSTO
                                AND IMP.CDFISCALIT = IMPORIGEM.CDFISCALNF)
                 GROUP BY IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPSTII, I.IDACRPRODENTRNF,
                          CDCONTPASS, CDCONTATIV, CDCONTDESP, IMP.CDFISCALIT, I.CDCCTBLATIV)LOOP
               IF C_NOTAFISCAL.IDCONTCONFORIG = 'S' THEN
                  V_NOTADEVOLUCAO := 'S';
                  /* IMPOSTO A RETER NA FONTE */
                  IF C_IMPOSTODEV.IDTRATIMPO = 'R' THEN
                     /* Imposto a recuperar, Sem conta do ativo na aba padro ou Cdigo fiscal diferente de 1
                        ou... Imposto no recupervel. */
                     IF (((C_IMPOSTODEV.IDIMPORECUP = 'S') AND ((C_IMPOSTODEV.CDCONTATIVO IS NULL) OR (C_IMPOSTODEV.CDFISCALNF <> '1')))
                         OR
                        (C_IMPOSTODEV.IDIMPORECUP <> 'S')) THEN

                        V_VRLANCTO := V_VRLANCTO + ROUND(C_IMPOSTODEV.VRIMPOSTONF, 2);
                     END IF;
                  END IF;
               END IF;
            END LOOP;

            /* Caso a nota seja de devoluo e no tenha os impostos a acrescer em comum
               com a nota de origem, os valores "a mais" na nota devoluo devem ser retirados
               para contabilizar de acordo com o valor de origem. */

            IF ((V_NOTADEVOLUCAO = 'N') OR (C_NOTAFISCAL.IDOPERDEVONF = 'S' AND C_NOTAFISCAL.IDCONTCONFORIG = 'N')) THEN
               BEGIN
                  SELECT NVL(ROUND(SUM(VRIMPOSTONF),2), 0) AS VRTOTIMPONF
                    INTO V_VRTEMP
                    FROM IMPOSTONF INF, IMPOSTO I
                   WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                     AND INF.CDIMPOSTO = I.CDIMPOSTO
                     AND I.IDTRATIMPO = 'R'
                     AND I.IDTIPCONTAB = 'D'
                     AND (((IDIMPORECUP = 'S') AND ((CDCCTBLATIV IS NULL) OR (CDFISCALNF <> '1'))) OR
                            (IDIMPORECUP <> 'S'));
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                      V_VRTEMP := 0;
               END;
               V_VRLANCTO := V_VRLANCTO + NVL(V_VRTEMP, 0);
            ELSE
               FOR C_RETIMPACREDEVFORN IN
               (SELECT IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                       ROUND(SUM(IMP.VRIMPOSTONF),    2) AS VRIMPOSTONF,
                       ROUND(SUM(IMP.VRIMPCARGATRIBI),2) AS VRIMPCARGATRIBI,
                       CDCONTATIV,
                       CDCONTPASS,
                       CDCONTDESP
                  FROM IMPOSTONF IMP, NOTAFISCAL NF, IMPOSTO I, PARAMFILIAL PF,
                       (SELECT DISTINCT CDIMPOSTO,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTATIV,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTPASS,
                                        CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTDESP
                          FROM IMPOSTONF I, NOTAFISCAL NF
                         WHERE I.CDFILIAL   = NF.CDFILIAL
                            AND I.NRLANCTONF = NF.NRLANCTONF
                           AND I.CDFILIAL   = C_NOTAFISCAL.CDFILIAL
                           AND I.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF) CT
                 WHERE IMP.VRIMPOSTONF > 0          AND
                       IMP.CDIMPOSTO = CT.CDIMPOSTO AND
                       IMP.CDIMPOSTO = I.CDIMPOSTO  AND
                       ((IMP.IDIMPOSTONF IN ('1','4')) OR (IMP.IDIMPOSTONF IS NULL)) AND
                       I.IDTIPCONTAB  = 'D'         AND
                       ((((I.IDIMPORECUP  = 'S'  AND IMP.CDFISCALNF = '1' AND PF.IDCARACFIS  = 'C') OR (PF.IDCARACFIS = 'A')) AND (CT.CDCONTPASS IS NOT NULL))
                        OR
                        ((I.IDIMPORECUP <> 'S') AND (CT.CDCONTPASS IS NOT NULL)))     AND
                       NF.CDFILIAL     = C_NOTAFISCAL.CDFILIAL     AND
                       NF.NRLANCTONF   = C_NOTAFISCAL.NRLANCTONF   AND
                       NF.CDFILIEMIT   = PF.CDFILIAL    AND
                       NF.CDFILIAL     = IMP.CDFILIAL   AND
                       NF.NRLANCTONF   = IMP.NRLANCTONF AND
                       I.IDTRATIMPO = 'A'
                 GROUP BY IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                              CDCONTPASS, CDCONTATIV, CDCONTDESP) LOOP
                  V_VRLANCTO := V_VRLANCTO + C_RETIMPACREDEVFORN.VRIMPOSTONF;
               END LOOP;
            END IF;
         END IF;
         /* CASO O TIPO DE OPERAO DA NOTA CONTABILIZE SOMENTE OS IMPOSTOS, O VALOR DA PARTIDA
            PARA O CLIENTE DEVE SER IGNORADO. */
         IF C_NOTAFISCAL.IDCONTSOMENIMPO = 'N' THEN
            CTB_DFS_CLIENTE(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_VRLANCTO);
         END IF;

         IF C_NOTAFISCAL.IDOPERDEVONF = 'S' and C_NOTAFISCAL.IDCONTCONFORIG = 'S' THEN
            DIFALIQDEVOLUCAO(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIEMIT, C_NOTAFISCAL.NRLANCTONF,
                             C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                             C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTEMISSAO);
         END IF;

         V_VRTEMP := 0;
         /* SOMA OS VALORES DOS ENCARGOS AO VALOR DO LANAMENTO */
         FOR C_ENC IN
            (SELECT VRITEMENCA, E.IDENCARGO
              FROM ITEMENCARGO IE, ENCARGO E
             WHERE IE.CDFILIAL   = P_CDFILIAL
               AND IE.NRLANCTONF = P_NRLANCTONF
               AND IE.CDENCARGO  = E.CDENCARGO
               AND IE.VRITEMENCA > 0
               AND E.IDENCARGO IN ('A','D')
               AND (DECODE(C_NOTAFISCAL.IDENTRSAIDOP, 'E', CDCTBLENTREN, CDCTBLSAIDEN) IS  NOT NULL))LOOP
            IF C_ENC.IDENCARGO = 'A' THEN
               V_VRTEMP := V_VRTEMP + NVL(C_ENC.VRITEMENCA, 0);
            ELSE
               V_VRTEMP := V_VRTEMP - NVL(C_ENC.VRITEMENCA, 0);
            END IF;
         END LOOP;
         V_VRLANCTO := V_VRLANCTO + NVL(V_VRTEMP, 0);

         IF C_NOTAFISCAL.IDCONTSOMENIMPO = 'N' THEN
            V_ISEMPTY := 'S';
            FOR C_IMPDEVNFS IN
               (SELECT IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPSTII IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                       ROUND(SUM(IMP.VRIMPOSTOIT),    2) AS VRIMPOSTONF,
                       0 AS VRIMPCARGATRIBI,
                       CDCONTATIV,
                       CDCONTPASS,
                       CDCONTDESP, IMP.CDFISCALIT CDFISCALNF, I.CDCCTBLATIV CDCONTATIVO
                  FROM ITEMPROIMPOSORIG IMP, NOTAFISCAL NF, IMPOSTO I, PARAMFILIAL PF,
                      (SELECT DISTINCT CDIMPOSTO,
                                       CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTATIV,
                                       CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTPASS,
                                       CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTDESP
                         FROM ITEMPROIMPOSORIG I, NOTAFISCAL NF
                        WHERE I.CDFILIAL   = NF.CDFILIAL
                          AND I.NRLANCTONF = NF.NRLANCTONF
                          AND I.CDFILIAL   = C_NOTAFISCAL.CDFILIAL
                          AND I.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF) CT
                WHERE IMP.VRIMPOSTOIT > 0
                  AND IMP.CDIMPOSTO = CT.CDIMPOSTO
                  AND IMP.CDIMPOSTO = I.CDIMPOSTO
                  AND ((IMP.NRITPROIMPOS IN ('1','4')) OR (IMP.NRITPROIMPOS IS NULL))
                  AND I.IDTIPCONTAB  = 'D'
                  AND ((((I.IDIMPORECUP  = 'S'  AND IMP.CDFISCALIT = '1' AND PF.IDCARACFIS  = 'C') OR (PF.IDCARACFIS = 'A')) AND (CT.CDCONTATIV IS NOT NULL))
                      OR
                      ((I.IDIMPORECUP <> 'S') AND (CT.CDCONTPASS IS NOT NULL)))
                  AND NF.CDFILIAL     = C_NOTAFISCAL.CDFILIAL
                  AND NF.NRLANCTONF   = C_NOTAFISCAL.NRLANCTONF
                  AND NF.CDFILIEMIT   = PF.CDFILIAL
                  AND NF.CDFILIAL     = IMP.CDFILIAL
                  AND NF.NRLANCTONF   = IMP.NRLANCTONF
                  AND EXISTS (SELECT 1
                                FROM (SELECT  A.CDIMPOSTO, A.CDFILIAL, A.NRLANCTONF, A.CDFISCALNF
                                        FROM (SELECT INF.CDIMPOSTO, VRIMPOSTONF, NVL(IDACRPRODENTRNF, 'N') IDACRPRODENTRNF,
                                                     PF.IDCARACFIS, I.IDIMPORECUP, I.IDTRATIMPO, INF.CDFISCALNF,
                                                     CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTATIV,
                                                     CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTPASS,
                                                     CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTDESP,
                                                     CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'E', NF.IDENTRSAIDOP, NF.CDTIPOOPER, NF.IDTPIJURNF) CDCONTIMPO,
                                                     NF.NRLANCTONF, NF.CDFILIAL
                                                FROM IMPOSTONF INF, IMPOSTO I, NOTAFISCAL NF, PARAMFILIAL PF, ITEMNOTAFISC IT
                                               WHERE NF.CDFILIAL     = IT.CDFILIALIT
                                                 AND NF.NRLANCTONF   = IT.NRLANCTONFIT
                                                 AND PF.CDFILIAL(+)  = NF.CDFILIRECE
                                                 AND INF.CDFILIAL    = IT.CDFILIALIT
                                                 AND INF.NRLANCTONF  = IT.NRLANCTONFIT
                                                 AND IT.CDFILIAL = C_NOTAFISCAL.CDFILIAL
                                                 AND IT.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                                                 AND INF.CDIMPOSTO   = I.CDIMPOSTO
                                                 AND I.IDTIPCONTAB  = 'D') A
                                       WHERE ((((A.IDIMPORECUP  = 'S' AND A.CDFISCALNF = '1' AND NVL(A.IDCARACFIS,'C') = 'C') OR (A.IDCARACFIS = 'A')) AND (A.CDCONTATIV IS NOT NULL)) OR
                                              ((A.IDIMPORECUP <> 'S') AND (A.CDCONTPASS IS NOT NULL)))) IMPORIGEM
                              WHERE IMP.CDIMPOSTO = IMPORIGEM.CDIMPOSTO
                                AND IMP.CDFISCALIT = IMPORIGEM.CDFISCALNF)
                GROUP BY IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPSTII, I.IDACRPRODENTRNF,
                             CDCONTPASS, CDCONTATIV, CDCONTDESP, IMP.CDFISCALIT, I.CDCCTBLATIV) LOOP
               V_ISEMPTY := 'N';
               /* Acrescenta o valor dos impostos que tiveram suas alquotas ou incidncias modificadas
                 na devoluo. Os valores que so somados so os valores dos impostos calculados com as
                 alquotas da nota fiscal de origem. */
               IF C_IMPDEVNFS.VRIMPOSTONF > 0 THEN
                  V_VRLANCTO := V_VRLANCTO + ROUND(C_IMPDEVNFS.VRIMPOSTONF, 2);
               END IF;
            END LOOP;
            IF ((V_ISEMPTY = 'S') OR (C_NOTAFISCAL.IDCONTCONFORIG = 'N')) THEN
               FOR C_SOMAIMPOSTO IN
                  (SELECT IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                           ROUND(SUM(IMP.VRIMPOSTONF),    2) AS VRIMPOSTONF,
                          ROUND(SUM(IMP.VRIMPCARGATRIBI),2) AS VRIMPCARGATRIBI,
                          CDCONTATIV,
                          CDCONTPASS,
                          CDCONTDESP
                     FROM IMPOSTONF IMP, NOTAFISCAL NF, IMPOSTO I, PARAMFILIAL PF,
                          (SELECT DISTINCT CDIMPOSTO,
                                           CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'A', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTATIV,
                                           CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'P', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTPASS,
                                           CONTCTBL_IMPOSTO(I.CDIMPOSTO, 'D', 'S', NF.CDTIPOOPER, NF.IDTPIJURNF) AS CDCONTDESP
                             FROM IMPOSTONF I, NOTAFISCAL NF
                            WHERE I.CDFILIAL   = NF.CDFILIAL
                              AND I.NRLANCTONF = NF.NRLANCTONF
                              AND I.CDFILIAL   = C_NOTAFISCAL.CDFILIAL
                              AND I.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF) CT
                    WHERE IMP.VRIMPOSTONF > 0          AND
                          IMP.CDIMPOSTO = CT.CDIMPOSTO AND
                          IMP.CDIMPOSTO = I.CDIMPOSTO  AND
                          ((IMP.IDIMPOSTONF IN ('1','4')) OR (IMP.IDIMPOSTONF IS NULL)) AND
                          I.IDTIPCONTAB  = 'D'         AND
                          ((((I.IDIMPORECUP  = 'S'  AND IMP.CDFISCALNF = '1' AND PF.IDCARACFIS  = 'C') OR (PF.IDCARACFIS = 'A')) AND (CT.CDCONTPASS IS NOT NULL))
                           OR
                           ((I.IDIMPORECUP <> 'S') AND (CT.CDCONTPASS IS NOT NULL)))     AND
                          NF.CDFILIAL     = C_NOTAFISCAL.CDFILIAL      AND
                          NF.NRLANCTONF   = C_NOTAFISCAL.NRLANCTONF    AND
                          NF.CDFILIEMIT   = PF.CDFILIAL    AND
                          NF.CDFILIAL     = IMP.CDFILIAL   AND
                          NF.NRLANCTONF   = IMP.NRLANCTONF
                    GROUP BY IMP.CDIMPOSTO, I.IDTRATIMPO,  I.IDIMPORECUP, IMP.IDTRIMPOSTNF, I.IDACRPRODENTRNF,
                                  CDCONTPASS, CDCONTATIV, CDCONTDESP) LOOP

                  IF C_SOMAIMPOSTO.VRIMPOSTONF > 0 THEN
                     IF C_NOTAFISCAL.IDOPERDEVONF <> 'S' THEN
                        BEGIN
                           SELECT MAX(CDIMPOSTO) AS CDIMPOSTRIB
                             INTO V_CDIMPSUBST
                             FROM IMPOSTO
                            WHERE CDIMPOSTRIB = C_SOMAIMPOSTO.CDIMPOSTO;
                        EXCEPTION
                           WHEN NO_DATA_FOUND THEN
                              V_CDIMPSUBST := NULL;
                        END;
                        IF V_CDIMPSUBST IS NULL THEN
                           IF C_SOMAIMPOSTO.IDTRATIMPO = 'R' THEN
                              V_VRLANCTO := V_VRLANCTO - ROUND(C_SOMAIMPOSTO.VRIMPOSTONF, 2);
                           ELSE
                              IF ((C_SOMAIMPOSTO.IDTRATIMPO = 'V') AND (C_NOTAFISCAL.IDORIGDESTNF = 'L')) THEN
                                 V_VRLANCTO := V_VRLANCTO + ROUND(C_SOMAIMPOSTO.VRIMPOSTONF, 2);
                              END IF;
                           END IF;
                        END IF;
                     ELSE
                        V_VRLANCTO := V_VRLANCTO + ROUND(C_SOMAIMPOSTO.VRIMPOSTONF, 2);
                     END IF;
                  END IF;
               END LOOP;
            END IF;
         END IF;
         CTB_DFE_HISTORICO(P_CDFILIAL, P_NRLANCTONF, V_CDHISTPADR, V_DSHISTLANC);

         IF C_NOTAFISCAL.IDCONTSOMENIMPO = 'N' THEN
            CTB_DFS_RECEITAPROD(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, V_VRLANCTO);


            CONTABILIZA_ENCARGO(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIEMIT, C_NOTAFISCAL.NRLANCTONF,
                                C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                                C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTEMISSAO);

            CONTABILIZA_DESCONTO(C_NOTAFISCAL.CDEMPRESA, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.CDFILIEMIT, C_NOTAFISCAL.NRLANCTONF,
                                 C_NOTAFISCAL.NRNOTAFISC, C_NOTAFISCAL.IDENTRSAIDOP, C_NOTAFISCAL.IDORIGDESTNF,
                                 C_NOTAFISCAL.CDFORNECED, C_NOTAFISCAL.CDCLIENTE, C_NOTAFISCAL.DTEMISSAO);
         END IF;
         IF V_NOTADEVOLUCAO = 'S' THEN
            CTB_DFS_IMPOSTO(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, 'S', V_VRLANCTO);
         ELSE
            CTB_DFS_IMPOSTO(V_NRLANCTEMP, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF, 'N', V_VRLANCTO);
         END IF;

         /*VERIFICACAO DE ERROS NAS PARTIDAS E NO LANCAMENTO CONTABIL*/
         VERIFICA_ERRO_CTB(V_NRLANCTEMP, C_NOTAFISCAL.CDEMPRESA);
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM ERROCTBTEMP
             WHERE NRLANCCTBTEMP = V_NRLANCTEMP
               AND CDEMPRESA = C_NOTAFISCAL.CDEMPRESA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;

         IF V_COUNT = 0 THEN
            IF C_NOTAFISCAL.NRLANCTBDOC IS NULL THEN
               UPDATE NOTAFISCAL
                  SET NRLANCTBDOC = V_NRLANCTEMP
                WHERE CDFILIAL    = P_CDFILIAL
                  AND NRLANCTONF  = P_NRLANCTONF;
            END IF;
         END IF;

         /* TRMINO DAS PARTIDAS DO LANCAMENTO CONTABIL*/
         /* GRAVA O LANAMENTO TEMPORRIO SE NO CONTER ERRO NA LANCCONT E LANCCUSTO, APS GRAVAR IR APAGAR O LANCAMENTO TEMPORRIO */
         GRAVA_LANC_CTB(V_NRLANCTEMP, C_NOTAFISCAL.CDEMPRESA);
         P_NRLANCCTB := V_NRLANCTEMP;
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CTB_DFS_CONTAB','5.50.000'); END;
/

